@@ -1,10 +1,10 @@
-#  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
+#  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization
 #  dedicated to making software imaging solutions freely available.
 #
 #  You may not use this file except in compliance with the License.  You may
 #  obtain a copy of the License at
 #
-#    http://www.imagemagick.org/script/license.php
+#    https://imagemagick.org/script/license.php
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
@@ -1050,7 +1050,7 @@ sub testMontage {
       } else {
         # Check montage directory
         my $directory = $montage->Get('directory');
-        my $expected = join( "\n", @colors ) . "\n";
+        my $expected = join( "\xff", @colors ) . "\xff";
         if ( !defined($directory) ) {
           print "ok $test\n";
         } elsif ( $directory  ne $expected) {
