@@ -19,13 +19,13 @@
 %                              September 2013                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -1414,7 +1414,7 @@ static void ComputePrincipleComponent(const float *covariance,
     w.z = (row2.z * v.z) + w.z;
     w.w = (row2.w * v.z) + w.w;
 
-    a = 1.0f / MagickMax(w.x,MagickMax(w.y,w.z));
+    a = (float) PerceptibleReciprocal(MagickMax(w.x,MagickMax(w.y,w.z)));
 
     v.x = w.x * a;
     v.y = w.y * a;
@@ -1656,12 +1656,12 @@ static Image *ReadDDSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -1673,9 +1673,8 @@ static Image *ReadDDSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Initialize image structure.
   */
-  if (ReadDDSInfo(image, &dds_info) != MagickTrue) {
+  if (ReadDDSInfo(image, &dds_info) != MagickTrue)
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  }
 
   if (dds_info.ddscaps2 & DDSCAPS2_CUBEMAP)
     cubemap = MagickTrue;
@@ -1772,6 +1771,12 @@ static Image *ReadDDSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (volume)
     num_images = dds_info.depth;
 
+  if ((num_images == 0) || (num_images > GetBlobSize(image)))
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+
+  if (AcquireMagickResource(ListLengthResource,num_images) == MagickFalse)
+    ThrowReaderException(ResourceLimitError,"ListLengthExceedsLimit");
+
   for (n = 0; n < num_images; n++)
   {
     if (n != 0)
@@ -1803,7 +1808,7 @@ static Image *ReadDDSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         InheritException(exception,&image->exception);
         return(DestroyImageList(image));
       }
-
+    (void) SetImageBackgroundColor(image);
     if ((decoder)(image, &dds_info, exception) != MagickTrue)
       {
         (void) CloseBlob(image);
@@ -1909,6 +1914,8 @@ static MagickBooleanType ReadDXT1(Image *image,DDSInfo *dds_info,
       bits = ReadBlobLSBLong(image);
 
       CalculateColors(c0, c1, &colors, MagickFalse);
+      if (EOFBlob(image) != MagickFalse)
+        break;
 
       /* Write the pixels */
       for (j = 0; j < 4; j++)
@@ -1933,6 +1940,8 @@ static MagickBooleanType ReadDXT1(Image *image,DDSInfo *dds_info,
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
         return MagickFalse;
     }
+    if (EOFBlob(image) != MagickFalse)
+      break;
   }
 
   return(SkipDXTMipmaps(image,dds_info,8,exception));
@@ -1989,6 +1998,8 @@ static MagickBooleanType ReadDXT3(Image *image, DDSInfo *dds_info,
       bits = ReadBlobLSBLong(image);
 
       CalculateColors(c0, c1, &colors, MagickTrue);
+      if (EOFBlob(image) != MagickFalse)
+        break;
 
       /* Write the pixels */
       for (j = 0; j < 4; j++)
@@ -2018,6 +2029,8 @@ static MagickBooleanType ReadDXT3(Image *image, DDSInfo *dds_info,
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
         return MagickFalse;
     }
+    if (EOFBlob(image) != MagickFalse)
+      break;
   }
 
   return(SkipDXTMipmaps(image,dds_info,16,exception));
@@ -2081,6 +2094,8 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info,
       bits = ReadBlobLSBLong(image);
 
       CalculateColors(c0, c1, &colors, MagickTrue);
+      if (EOFBlob(image) != MagickFalse)
+        break;
 
       /* Write the pixels */
       for (j = 0; j < 4; j++)
@@ -2117,6 +2132,8 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info,
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
         return MagickFalse;
     }
+    if (EOFBlob(image) != MagickFalse)
+      break;
   }
 
   return(SkipDXTMipmaps(image,dds_info,16,exception));
@@ -2386,6 +2403,8 @@ static MagickBooleanType SkipDXTMipmaps(Image *image,DDSInfo *dds_info,
         offset = (MagickOffsetType) ((w + 3) / 4) * ((h + 3) / 4) * texel_size;
         if (SeekBlob(image,offset,SEEK_CUR) < 0)
           break;
+        if ((w == 1) && (h == 1))
+          break;
         w = DIV2(w);
         h = DIV2(h);
       }
@@ -2435,6 +2454,8 @@ static MagickBooleanType SkipRGBMipmaps(Image *image,DDSInfo *dds_info,
           break;
         w = DIV2(w);
         h = DIV2(h);
+        if ((w == 1) && (h == 1))
+          break;
       }
     }
   return(MagickTrue);
@@ -2605,9 +2626,9 @@ static MagickBooleanType WriteDDSImage(const ImageInfo *image_info,
     weightByAlpha;
 
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
@@ -2745,7 +2766,7 @@ static void WriteDDSInfo(Image *image, const size_t pixelFormat,
 
   (void) WriteBlobLSBLong(image,0x00);
   (void) WriteBlobLSBLong(image,(unsigned int) mipmaps+1);
-  (void) ResetMagickMemory(software,0,sizeof(software));
+  (void) memset(software,0,sizeof(software));
   (void) CopyMagickString(software,"IMAGEMAGICK",MaxTextExtent);
   (void) WriteBlob(image,44,(unsigned char *) software);
 
@@ -2755,7 +2776,7 @@ static void WriteDDSInfo(Image *image, const size_t pixelFormat,
   if (pixelFormat == DDPF_FOURCC)
     {
       (void) WriteBlobLSBLong(image,(unsigned int) compression);
-      for(i=0;i < 5;i++) // bitcount / masks
+      for(i=0;i < 5;i++)  /* bitcount / masks */
         (void) WriteBlobLSBLong(image,0x00);
     }
   else
@@ -2780,7 +2801,7 @@ static void WriteDDSInfo(Image *image, const size_t pixelFormat,
     }
   
   (void) WriteBlobLSBLong(image,caps);
-  for(i=0;i < 4;i++) // ddscaps2 + reserved region
+  for(i=0;i < 4;i++)  /* ddscaps2 + reserved region */
     (void) WriteBlobLSBLong(image,0x00);
 }
 
