@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -209,7 +209,7 @@ static LZWInfo *AcquireLZWInfo(Image *image,const size_t data_size)
   lzw_info=(LZWInfo *) AcquireMagickMemory(sizeof(*lzw_info));
   if (lzw_info == (LZWInfo *) NULL)
     return((LZWInfo *) NULL);
-  (void) ResetMagickMemory(lzw_info,0,sizeof(*lzw_info));
+  (void) memset(lzw_info,0,sizeof(*lzw_info));
   lzw_info->image=image;
   lzw_info->data_size=data_size;
   one=1;
@@ -217,15 +217,19 @@ static LZWInfo *AcquireLZWInfo(Image *image,const size_t data_size)
   lzw_info->clear_code=lzw_info->maximum_data_value+1;
   lzw_info->end_code=lzw_info->maximum_data_value+2;
   lzw_info->table[0]=(size_t *) AcquireQuantumMemory(MaximumLZWCode,
-    sizeof(*lzw_info->table));
+    sizeof(**lzw_info->table));
   lzw_info->table[1]=(size_t *) AcquireQuantumMemory(MaximumLZWCode,
-    sizeof(*lzw_info->table));
+    sizeof(**lzw_info->table));
   if ((lzw_info->table[0] == (size_t *) NULL) ||
       (lzw_info->table[1] == (size_t *) NULL))
     {
       lzw_info=RelinquishLZWInfo(lzw_info);
       return((LZWInfo *) NULL);
     }
+  (void) memset(lzw_info->table[0],0,MaximumLZWCode*
+    sizeof(**lzw_info->table));
+  (void) memset(lzw_info->table[1],0,MaximumLZWCode*
+    sizeof(**lzw_info->table));
   for (i=0; i <= (ssize_t) lzw_info->maximum_data_value; i++)
   {
     lzw_info->table[0][i]=0;
@@ -414,9 +418,10 @@ static MagickBooleanType DecodeImage(Image *image,const ssize_t opacity)
     Allocate decoder tables.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  exception=(&image->exception);
   data_size=(unsigned char) ReadBlobByte(image);
   if (data_size > MaximumLZWBits)
     ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
@@ -424,19 +429,18 @@ static MagickBooleanType DecodeImage(Image *image,const ssize_t opacity)
   if (lzw_info == (LZWInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  exception=(&image->exception);
   pass=0;
   offset=0;
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register ssize_t
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     q=QueueAuthenticPixels(image,0,offset,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
@@ -447,7 +451,7 @@ static MagickBooleanType DecodeImage(Image *image,const ssize_t opacity)
       c=ReadBlobLZWByte(lzw_info);
       if (c < 0)
         break;
-      index=ConstrainColormapIndex(image,(size_t) c);
+      index=ConstrainColormapIndex(image,(ssize_t) c);
       SetPixelIndex(indexes+x,index);
       SetPixelRGBO(q,image->colormap+(ssize_t) index);
       SetPixelOpacity(q,(ssize_t) index == opacity ? TransparentOpacity :
@@ -550,9 +554,9 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
     Emit a code. \
   */ \
   if (bits > 0) \
-    datum|=(code) << bits; \
+    datum|=(size_t) (code) << bits; \
   else \
-    datum=code; \
+    datum=(size_t) (code); \
   bits+=number_bits; \
   while (bits >= 8) \
   { \
@@ -582,9 +586,6 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   IndexPacket
     index;
 
-  register ssize_t
-    i;
-
   short
     *hash_code,
     *hash_prefix,
@@ -640,6 +641,10 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   /*
     Initialize GIF encoder.
   */
+  (void) memset(packet,0,256*sizeof(*packet));
+  (void) memset(hash_code,0,MaxHashTable*sizeof(*hash_code));
+  (void) memset(hash_prefix,0,MaxHashTable*sizeof(*hash_prefix));
+  (void) memset(hash_suffix,0,MaxHashTable*sizeof(*hash_suffix));
   number_bits=data_size;
   max_code=MaxCode(number_bits);
   clear_code=((short) one << (data_size-1));
@@ -648,8 +653,6 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   length=0;
   datum=0;
   bits=0;
-  for (i=0; i < MaxHashTable; i++)
-    hash_code[i]=0;
   GIFOutputCode(clear_code);
   /*
     Encode pixels.
@@ -660,10 +663,10 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register const IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       x;
@@ -673,7 +676,10 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
       break;
     indexes=GetVirtualIndexQueue(image);
     if (y == 0)
-      waiting_code=(short) (*indexes);
+      {
+        waiting_code=(short) (*indexes);
+        p++;
+      }
     for (x=(ssize_t) (y == 0 ? 1 : 0); x < (ssize_t) image->columns; x++)
     {
       /*
@@ -714,7 +720,7 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
           if (next_pixel != MagickFalse)
             continue;
         }
-      GIFOutputCode((size_t) waiting_code);
+      GIFOutputCode(waiting_code);
       if (free_code < MaxGIFTable)
         {
           hash_code[k]=(short) free_code++;
@@ -784,7 +790,7 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   /*
     Flush out the buffered code.
   */
-  GIFOutputCode((size_t) waiting_code);
+  GIFOutputCode(waiting_code);
   GIFOutputCode(end_of_information_code);
   if (bits > 0)
     {
@@ -868,7 +874,7 @@ static MagickBooleanType IsGIF(const unsigned char *magick,const size_t length)
 %
 %  The format of the ReadBlobBlock method is:
 %
-%      size_t ReadBlobBlock(Image *image,unsigned char *data)
+%      ssize_t ReadBlobBlock(Image *image,unsigned char *data)
 %
 %  A description of each parameter follows:
 %
@@ -887,7 +893,7 @@ static ssize_t ReadBlobBlock(Image *image,unsigned char *data)
     block_count;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   assert(data != (unsigned char *) NULL);
   count=ReadBlob(image,1,&block_count);
   if (count != 1)
@@ -925,18 +931,9 @@ static ssize_t ReadBlobBlock(Image *image,unsigned char *data)
 %
 */
 
-static inline size_t MagickMax(const size_t x,const size_t y)
+static void *DestroyGIFProfile(void *profile)
 {
-  if (x > y)
-    return(x);
-  return(y);
-}
-
-static inline size_t MagickMin(const size_t x,const size_t y)
-{
-  if (x < y)
-    return(x);
-  return(y);
+  return((void *) DestroyStringInfo((StringInfo *) profile));
 }
 
 static MagickBooleanType PingGIFImage(Image *image)
@@ -947,21 +944,26 @@ static MagickBooleanType PingGIFImage(Image *image)
     data_size;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (ReadBlob(image,1,&data_size) != 1)
-    ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
+    ThrowBinaryImageException(CorruptImageError,"CorruptImage",
+      image->filename);
   if (data_size > MaximumLZWBits)
-    ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
+    ThrowBinaryImageException(CorruptImageError,"CorruptImage",
+      image->filename);
   if (ReadBlob(image,1,&length) != 1)
-    ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
+    ThrowBinaryImageException(CorruptImageError,"CorruptImage",
+      image->filename);
   while (length != 0)
   {
     if (ReadBlob(image,length,buffer) != (ssize_t) length)
-      ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
+      ThrowBinaryImageException(CorruptImageError,"CorruptImage",
+        image->filename);
     if (ReadBlob(image,1,&length) != 1)
-      ThrowBinaryException(CorruptImageError,"CorruptImage",image->filename);
+      ThrowBinaryImageException(CorruptImageError,"CorruptImage",
+        image->filename);
   }
   return(MagickTrue);
 }
@@ -970,20 +972,27 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
 #define BitSet(byte,bit)  (((byte) & (bit)) == (bit))
 #define LSBFirstOrder(x,y)  (((y) << 8) | (x))
+#define ThrowGIFException(exception,message) \
+{ \
+  if (profiles != (LinkedListInfo *) NULL) \
+    profiles=DestroyLinkedList(profiles,DestroyGIFProfile); \
+  if (global_colormap != (unsigned char *) NULL) \
+    global_colormap=(unsigned char *) RelinquishMagickMemory(global_colormap); \
+  if (meta_image != (Image *) NULL) \
+    meta_image=DestroyImage(meta_image); \
+  ThrowReaderException((exception),(message)); \
+}
 
   Image
     *image,
     *meta_image;
 
-  int
-    number_extensionss=0;
+  LinkedListInfo
+    *profiles;
 
   MagickBooleanType
     status;
 
-  RectangleInfo
-    page;
-
   register ssize_t
     i;
 
@@ -991,12 +1000,10 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *p;
 
   size_t
-    delay,
-    dispose,
     duration,
     global_colors,
     image_count,
-    iterations,
+    local_colors,
     one;
 
   ssize_t
@@ -1005,22 +1012,22 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   unsigned char
     background,
+    buffer[257],
     c,
     flag,
     *global_colormap,
-    header[MaxTextExtent],
     magick[12];
 
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -1035,26 +1042,31 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((count != 6) || ((LocaleNCompare((char *) magick,"GIF87",5) != 0) &&
       (LocaleNCompare((char *) magick,"GIF89",5) != 0)))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  page.width=ReadBlobLSBShort(image);
-  page.height=ReadBlobLSBShort(image);
+  (void) memset(buffer,0,sizeof(buffer));
+  meta_image=AcquireImage(image_info);  /* metadata container */
+  meta_image->page.width=ReadBlobLSBShort(image);
+  meta_image->page.height=ReadBlobLSBShort(image);
   flag=(unsigned char) ReadBlobByte(image);
   background=(unsigned char) ReadBlobByte(image);
   c=(unsigned char) ReadBlobByte(image);  /* reserved */
+  profiles=(LinkedListInfo *) NULL;
   one=1;
   global_colors=one << (((size_t) flag & 0x07)+1);
   global_colormap=(unsigned char *) AcquireQuantumMemory((size_t)
     MagickMax(global_colors,256),3UL*sizeof(*global_colormap));
   if (global_colormap == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    ThrowGIFException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) memset(global_colormap,0,3*MagickMax(global_colors,256)*
+    sizeof(*global_colormap));
   if (BitSet((int) flag,0x80) != 0)
-    count=ReadBlob(image,(size_t) (3*global_colors),global_colormap);
-  delay=0;
-  dispose=0;
+    {
+      count=ReadBlob(image,(size_t) (3*global_colors),global_colormap);
+      if (count != (ssize_t) (3*global_colors))
+        ThrowGIFException(CorruptImageError,"InsufficientImageDataInFile");
+    }
   duration=0;
-  iterations=1;
   opacity=(-1);
   image_count=0;
-  meta_image=AcquireImage(image_info);  /* metadata container */
   for ( ; ; )
   {
     count=ReadBlob(image,1,&c);
@@ -1067,14 +1079,10 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           GIF Extension block.
         */
-
         count=ReadBlob(image,1,&c);
         if (count != 1)
-          {
-            global_colormap=(unsigned char *) RelinquishMagickMemory(
-              global_colormap);
-            ThrowReaderException(CorruptImageError,"UnableToReadExtensionBlock");
-          }
+          ThrowGIFException(CorruptImageError,"UnableToReadExtensionBlock");
+        (void) memset(buffer,0,sizeof(buffer));
         switch (c)
         {
           case 0xf9:
@@ -1082,11 +1090,11 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               Read graphics control extension.
             */
-            while (ReadBlobBlock(image,header) != 0) ;
-            dispose=(size_t) (header[0] >> 2);
-            delay=(size_t) ((header[2] << 8) | header[1]);
-            if ((ssize_t) (header[0] & 0x01) == 0x01)
-              opacity=(ssize_t) header[3];
+            while (ReadBlobBlock(image,buffer) != 0) ;
+            meta_image->dispose=(DisposeType) ((buffer[0] >> 2) & 0x07);
+            meta_image->delay=((size_t) buffer[2] << 8) | buffer[1];
+            if ((ssize_t) (buffer[0] & 0x01) == 0x01)
+              opacity=(ssize_t) buffer[3];
             break;
           }
           case 0xfe:
@@ -1103,11 +1111,11 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
             comments=AcquireString((char *) NULL);
             for (length=0; ; length+=count)
             {
-              count=(ssize_t) ReadBlobBlock(image,header);
+              count=ReadBlobBlock(image,buffer);
               if (count == 0)
                 break;
-              header[count]='\0';
-              (void) ConcatenateString(&comments,(const char *) header);
+              buffer[count]='\0';
+              (void) ConcatenateString(&comments,(const char *) buffer);
             }
             (void) SetImageProperty(meta_image,"comment",comments);
             comments=DestroyString(comments);
@@ -1122,14 +1130,15 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
               Read Netscape Loop extension.
             */
             loop=MagickFalse;
-            if (ReadBlobBlock(image,header) != 0)
-              loop=LocaleNCompare((char *) header,"NETSCAPE2.0",11) == 0 ?
+            if (ReadBlobBlock(image,buffer) != 0)
+              loop=LocaleNCompare((char *) buffer,"NETSCAPE2.0",11) == 0 ?
                 MagickTrue : MagickFalse;
             if (loop != MagickFalse)
+              while (ReadBlobBlock(image,buffer) != 0)
               {
-                while (ReadBlobBlock(image,header) != 0)
-                  iterations=(size_t) ((header[2] << 8) | header[1]);
-                break;
+                meta_image->iterations=((size_t) buffer[2] << 8) | buffer[1];
+                if (meta_image->iterations != 0)
+                  meta_image->iterations++;
               }
             else
               {
@@ -1156,18 +1165,22 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 /*
                   Store GIF application extension as a generic profile.
                 */
-                icc=LocaleNCompare((char *) header,"ICCRGBG1012",11) == 0 ?
+                icc=LocaleNCompare((char *) buffer,"ICCRGBG1012",11) == 0 ?
                   MagickTrue : MagickFalse;
-                magick=LocaleNCompare((char *) header,"ImageMagick",11) == 0 ?
+                magick=LocaleNCompare((char *) buffer,"ImageMagick",11) == 0 ?
                   MagickTrue : MagickFalse;
-                i8bim=LocaleNCompare((char *) header,"MGK8BIM0000",11) == 0 ?
+                i8bim=LocaleNCompare((char *) buffer,"MGK8BIM0000",11) == 0 ?
                   MagickTrue : MagickFalse;
-                iptc=LocaleNCompare((char *) header,"MGKIPTC0000",11) == 0 ?
+                iptc=LocaleNCompare((char *) buffer,"MGKIPTC0000",11) == 0 ?
                   MagickTrue : MagickFalse;
-                number_extensionss++;
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                   "    Reading GIF application extension");
-                info=(unsigned char *) AcquireQuantumMemory(255UL,sizeof(*info));
+                info=(unsigned char *) AcquireQuantumMemory(255UL,
+                  sizeof(*info));
+                if (info == (unsigned char *) NULL)
+                  ThrowGIFException(ResourceLimitError,
+                    "MemoryAllocationFailed");
+                (void) memset(info,0,255UL*sizeof(*info));
                 reserved_length=255;
                 for (info_length=0; ; )
                 {
@@ -1181,14 +1194,20 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       info=(unsigned char *) ResizeQuantumMemory(info,(size_t)
                         reserved_length,sizeof(*info));
                       if (info == (unsigned char *) NULL)
-                        ThrowReaderException(ResourceLimitError,
-                          "MemoryAllocationFailed");
+                        {
+                          info=(unsigned char *) RelinquishMagickMemory(info);
+                          ThrowGIFException(ResourceLimitError,
+                            "MemoryAllocationFailed");
+                        }
                     }
                 }
                 profile=BlobToStringInfo(info,(size_t) info_length);
                 if (profile == (StringInfo *) NULL)
-                  ThrowReaderException(ResourceLimitError,
-                    "MemoryAllocationFailed");
+                  {
+                    info=(unsigned char *) RelinquishMagickMemory(info);
+                    ThrowGIFException(ResourceLimitError,
+                      "MemoryAllocationFailed");
+                  }
                 if (i8bim != MagickFalse)
                   (void) CopyMagickString(name,"8bim",sizeof(name));
                 else if (icc != MagickFalse)
@@ -1198,23 +1217,30 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 else if (magick != MagickFalse)
                   {
                     (void) CopyMagickString(name,"magick",sizeof(name));
-                    image->gamma=StringToDouble((char *) info+6,(char **) NULL);
+                    meta_image->gamma=StringToDouble((char *) info+6,
+                      (char **) NULL);
                   }
                 else
                   (void) FormatLocaleString(name,sizeof(name),"gif:%.11s",
-                    header);
+                    buffer);
                 info=(unsigned char *) RelinquishMagickMemory(info);
-                if (magick == MagickFalse)
-                  (void) SetImageProfile(meta_image,name,profile);
-                profile=DestroyStringInfo(profile);
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                   "      profile name=%s",name);
+                if (magick != MagickFalse)
+                  profile=DestroyStringInfo(profile);
+                else
+                  {
+                    if (profiles == (LinkedListInfo *) NULL)
+                      profiles=NewLinkedList(0);
+                    SetStringInfoName(profile,name);
+                    (void) AppendValueToLinkedList(profiles,profile);
+                  }
               }
             break;
           }
           default:
           {
-            while (ReadBlobBlock(image,header) != 0) ;
+            while (ReadBlobBlock(image,buffer) != 0) ;
             break;
           }
         }
@@ -1229,10 +1255,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
-            image=DestroyImageList(image);
-            global_colormap=(unsigned char *) RelinquishMagickMemory(
-              global_colormap);
-            return((Image *) NULL);
+            status=MagickFalse;
+            break;
           }
         image=SyncNextImageInList(image);
       }
@@ -1240,49 +1264,32 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Read image attributes.
     */
+    meta_image->page.x=(ssize_t) ReadBlobLSBShort(image);
+    meta_image->page.y=(ssize_t) ReadBlobLSBShort(image);
     meta_image->scene=image->scene;
-    CloneImageProperties(image,meta_image);
+    (void) CloneImageProperties(image,meta_image);
     DestroyImageProperties(meta_image);
-    CloneImageProfiles(image,meta_image);
-    DestroyImageProfiles(meta_image);
     image->storage_class=PseudoClass;
     image->compression=LZWCompression;
-    page.x=(ssize_t) ReadBlobLSBShort(image);
-    page.y=(ssize_t) ReadBlobLSBShort(image);
     image->columns=ReadBlobLSBShort(image);
     image->rows=ReadBlobLSBShort(image);
     image->depth=8;
     flag=(unsigned char) ReadBlobByte(image);
     image->interlace=BitSet((int) flag,0x40) != 0 ? GIFInterlace : NoInterlace;
-    image->colors=BitSet((int) flag,0x80) == 0 ? global_colors :
-      one << ((size_t) (flag & 0x07)+1);
+    local_colors=BitSet((int) flag,0x80) == 0 ? global_colors : one <<
+      ((size_t) (flag & 0x07)+1);
+    image->colors=local_colors;
     if (opacity >= (ssize_t) image->colors)
-      opacity=(-1);
-    image->page.width=page.width;
-    image->page.height=page.height;
-    image->page.y=page.y;
-    image->page.x=page.x;
-    image->delay=delay;
-    image->iterations=iterations;
+      image->colors=(size_t) (opacity+1);
     image->ticks_per_second=100;
-    image->dispose=(DisposeType) dispose;
     image->matte=opacity >= 0 ? MagickTrue : MagickFalse;
-    delay=0;
-    dispose=0;
     if ((image->columns == 0) || (image->rows == 0))
-      {
-        global_colormap=(unsigned char *) RelinquishMagickMemory(
-          global_colormap);
-        ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
-      }
+      ThrowGIFException(CorruptImageError,"NegativeOrZeroImageSize");
     /*
       Inititialize colormap.
     */
     if (AcquireImageColormap(image,image->colors) == MagickFalse)
-      {
-        global_colormap=(unsigned char *) RelinquishMagickMemory(global_colormap);
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      }
+      ThrowGIFException(ResourceLimitError,"MemoryAllocationFailed");
     if (BitSet((int) flag,0x80) == 0)
       {
         /*
@@ -1300,8 +1307,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
               image->transparent_color=image->colormap[opacity];
             }
         }
-        image->background_color=image->colormap[MagickMin(background,
-          image->colors-1)];
+        image->background_color=image->colormap[MagickMin((ssize_t) background,
+          (ssize_t) image->colors-1)];
       }
     else
       {
@@ -1311,22 +1318,17 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Read local colormap.
         */
-        colormap=(unsigned char *) AcquireQuantumMemory(image->colors,3*
-          sizeof(*colormap));
+        colormap=(unsigned char *) AcquireQuantumMemory((size_t)
+          MagickMax(local_colors,256),3UL*sizeof(*colormap));
         if (colormap == (unsigned char *) NULL)
+          ThrowGIFException(ResourceLimitError,"MemoryAllocationFailed");
+        (void) memset(colormap,0,3*MagickMax(local_colors,256)*
+          sizeof(*colormap));
+        count=ReadBlob(image,(3*local_colors)*sizeof(*colormap),colormap);
+        if (count != (ssize_t) (3*local_colors))
           {
-            global_colormap=(unsigned char *) RelinquishMagickMemory(
-              global_colormap);
-            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-          }
-        count=ReadBlob(image,(3*image->colors)*sizeof(*colormap),colormap);
-        if (count != (ssize_t) (3*image->colors))
-          {
-            global_colormap=(unsigned char *) RelinquishMagickMemory(
-              global_colormap);
             colormap=(unsigned char *) RelinquishMagickMemory(colormap);
-            ThrowReaderException(CorruptImageError,
-              "InsufficientImageDataInFile");
+            ThrowGIFException(CorruptImageError,"InsufficientImageDataInFile");
           }
         p=colormap;
         for (i=0; i < (ssize_t) image->colors; i++)
@@ -1345,11 +1347,22 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (IsGrayPixel(image->colormap+i) == MagickFalse)
             break;
         (void) SetImageColorspace(image,i == (ssize_t) image->colors ?
-          GRAYColorspace : RGBColorspace);
+          LinearGRAYColorspace : RGBColorspace);
       }
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
+    status=SetImageExtent(image,image->columns,image->rows);
+    if (status == MagickFalse)
+      {
+        if (profiles != (LinkedListInfo *) NULL)
+          profiles=DestroyLinkedList(profiles,DestroyGIFProfile);
+        global_colormap=(unsigned char *) RelinquishMagickMemory(
+          global_colormap);
+        meta_image=DestroyImage(meta_image);
+        InheritException(exception,&image->exception);
+        return(DestroyImageList(image));
+      }
     /*
       Decode image.
     */
@@ -1357,28 +1370,46 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       status=PingGIFImage(image);
     else
       status=DecodeImage(image,opacity);
+    InheritException(exception,&image->exception);
     if ((image_info->ping == MagickFalse) && (status == MagickFalse))
+      ThrowGIFException(CorruptImageError,"CorruptImage");
+    if (profiles != (LinkedListInfo *) NULL)
       {
-        global_colormap=(unsigned char *) RelinquishMagickMemory(
-          global_colormap);
-        ThrowReaderException(CorruptImageError,"CorruptImage");
+        StringInfo
+          *profile;
+
+        /*
+          Set image profiles.
+        */
+        ResetLinkedListIterator(profiles);
+        profile=(StringInfo *) GetNextValueInLinkedList(profiles);
+        while (profile != (StringInfo *) NULL)
+        {
+          (void) SetImageProfile(image,GetStringInfoName(profile),profile);
+          profile=(StringInfo *) GetNextValueInLinkedList(profiles);
+        }
+        profiles=DestroyLinkedList(profiles,DestroyGIFProfile);
       }
     duration+=image->delay*image->iterations;
     if (image_info->number_scenes != 0)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     opacity=(-1);
-    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) image->scene-1,
-      image->scene);
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) image->scene-
+      1,image->scene);
     if (status == MagickFalse)
       break;
   }
   image->duration=duration;
+  if (profiles != (LinkedListInfo *) NULL)
+    profiles=DestroyLinkedList(profiles,DestroyGIFProfile);
   meta_image=DestroyImage(meta_image);
   global_colormap=(unsigned char *) RelinquishMagickMemory(global_colormap);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
   (void) CloseBlob(image);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
@@ -1472,7 +1503,8 @@ ModuleExport void UnregisterGIFImage(void)
 %
 %  The format of the WriteGIFImage method is:
 %
-%      MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteGIFImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
@@ -1489,9 +1521,6 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   ImageInfo
     *write_info;
 
-  InterlaceType
-    interlace;
-
   MagickBooleanType
     status;
 
@@ -1510,6 +1539,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   size_t
     bits_per_pixel,
     delay,
+    imageListLength,
     length,
     one;
 
@@ -1525,9 +1555,9 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
@@ -1541,7 +1571,14 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   colormap=(unsigned char *) AcquireQuantumMemory(768UL,sizeof(*colormap));
   if ((global_colormap == (unsigned char *) NULL) ||
       (colormap == (unsigned char *) NULL))
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    {
+      if (global_colormap != (unsigned char *) NULL)
+        global_colormap=(unsigned char *) RelinquishMagickMemory(
+          global_colormap);
+      if (colormap != (unsigned char *) NULL)
+        colormap=(unsigned char *) RelinquishMagickMemory(colormap);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    }
   for (i=0; i < 768; i++)
     colormap[i]=(unsigned char) 0;
   /*
@@ -1571,12 +1608,12 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
   /*
     Write images to file.
   */
-  interlace=write_info->interlace;
   if ((write_info->adjoin != MagickFalse) &&
       (GetNextImageInList(image) != (Image *) NULL))
-    interlace=NoInterlace;
+    write_info->interlace=NoInterlace;
   scene=0;
   one=1;
+  imageListLength=GetImageListLength(image);
   do
   {
     (void) TransformImageColorspace(image,sRGBColorspace);
@@ -1588,7 +1625,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
       }
     else
       {
-        MagickRealType
+        double
           alpha,
           beta;
 
@@ -1605,10 +1642,10 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
                   opacity=i;
                   continue;
                 }
-              alpha=(MagickRealType) TransparentOpacity-(MagickRealType)
-                image->colormap[i].opacity;
-              beta=(MagickRealType) TransparentOpacity-(MagickRealType)
-                image->colormap[opacity].opacity;
+              alpha=fabs((double) image->colormap[i].opacity-
+                TransparentOpacity);
+              beta=fabs((double) image->colormap[opacity].opacity-
+                TransparentOpacity);
               if (alpha < beta)
                 opacity=i;
             }
@@ -1623,10 +1660,10 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
                       opacity=i;
                       continue;
                     }
-                  alpha=(Quantum) TransparentOpacity-(MagickRealType)
-                    image->colormap[i].opacity;
-                  beta=(Quantum) TransparentOpacity-(MagickRealType)
-                    image->colormap[opacity].opacity;
+                  alpha=fabs((double) image->colormap[i].opacity-
+                    TransparentOpacity);
+                  beta=fabs((double) image->colormap[opacity].opacity-
+                    TransparentOpacity);
                   if (alpha < beta)
                     opacity=i;
                 }
@@ -1679,6 +1716,9 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
       }
     if (LocaleCompare(write_info->magick,"GIF87") != 0)
       {
+        const char
+          *value;
+
         /*
           Write graphics control extension.
         */
@@ -1692,14 +1732,13 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
         delay=(size_t) (100*image->delay/MagickMax((size_t)
           image->ticks_per_second,1));
         (void) WriteBlobLSBShort(image,(unsigned short) delay);
-        (void) WriteBlobByte(image,(unsigned char) (opacity >= 0 ? opacity : 0));
+        (void) WriteBlobByte(image,(unsigned char) (opacity >= 0 ? opacity :
+          0));
         (void) WriteBlobByte(image,(unsigned char) 0x00);
+        value=GetImageProperty(image,"comment");
         if ((LocaleCompare(write_info->magick,"GIF87") != 0) &&
-            (GetImageProperty(image,"comment") != (const char *) NULL))
+            (value != (const char *) NULL))
           {
-            const char
-              *value;
-
             register const char
               *p;
 
@@ -1711,7 +1750,6 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
             */
             (void) WriteBlobByte(image,(unsigned char) 0x21);
             (void) WriteBlobByte(image,(unsigned char) 0xfe);
-            value=GetImageProperty(image,"comment");
             for (p=value; *p != '\0'; )
             {
               count=MagickMin(strlen(p),255);
@@ -1736,7 +1774,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
             (void) WriteBlob(image,11,(unsigned char *) "NETSCAPE2.0");
             (void) WriteBlobByte(image,(unsigned char) 0x03);
             (void) WriteBlobByte(image,(unsigned char) 0x01);
-            (void) WriteBlobLSBShort(image,(unsigned short) image->iterations);
+            (void) WriteBlobLSBShort(image,(unsigned short) (image->iterations ?
+              image->iterations-1 : 0));
             (void) WriteBlobByte(image,(unsigned char) 0x00);
           }
         if ((image->gamma != 1.0f/2.2f))
@@ -1745,7 +1784,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
               attributes[MaxTextExtent];
 
             ssize_t
-              length;
+              count;
 
             /*
               Write ImageMagick extension.
@@ -1756,10 +1795,10 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
             (void) WriteBlobByte(image,(unsigned char) 0xff);
             (void) WriteBlobByte(image,(unsigned char) 0x0b);
             (void) WriteBlob(image,11,(unsigned char *) "ImageMagick");
-            length=FormatLocaleString(attributes,MaxTextExtent,"gamma=%g",
+            count=FormatLocaleString(attributes,MaxTextExtent,"gamma=%g",
               image->gamma);
-            (void) WriteBlobByte(image,(unsigned char) length);
-            (void) WriteBlob(image,length,(unsigned char *) attributes);
+            (void) WriteBlobByte(image,(unsigned char) count);
+            (void) WriteBlob(image,(size_t) count,(unsigned char *) attributes);
             (void) WriteBlobByte(image,(unsigned char) 0x00);
           }
         ResetImageProfileIterator(image);
@@ -1874,7 +1913,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobLSBShort(image,(unsigned short) image->columns);
     (void) WriteBlobLSBShort(image,(unsigned short) image->rows);
     c=0x00;
-    if (interlace != NoInterlace)
+    if (write_info->interlace != NoInterlace)
       c|=0x40;  /* pixel data is interlaced */
     for (j=0; j < (ssize_t) (3*image->colors); j++)
       if (colormap[j] != global_colormap[j])
@@ -1908,8 +1947,7 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
       break;
     image=SyncNextImageInList(image);
     scene++;
-    status=SetImageProgress(image,SaveImagesTag,scene,
-      GetImageListLength(image));
+    status=SetImageProgress(image,SaveImagesTag,scene,imageListLength);
     if (status == MagickFalse)
       break;
   } while (write_info->adjoin != MagickFalse);
