@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -55,6 +55,7 @@
 #include "magick/pixel-accessor.h"
 #include "magick/property.h"
 #include "magick/quantum-private.h"
+#include "magick/resource_.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/module.h"
@@ -92,9 +93,6 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
 {
   char
     geometry[MaxTextExtent],
-    *property;
-
-  const char
     *label;
 
   DrawInfo
@@ -117,21 +115,31 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
     Initialize Image structure.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   (void) ResetImagePage(image,"0x0+0+0");
-  property=InterpretImageProperties(image_info,image,image_info->filename);
-  (void) SetImageProperty(image,"label",property);
-  property=DestroyString(property);
-  label=GetImageProperty(image,"label");
+  if ((image->columns != 0) && (image->rows != 0))
+    (void) SetImageBackgroundColor(image);
+  label=InterpretImageProperties(image_info,image,image_info->filename);
+  if (label == (char *) NULL)
+    return(DestroyImageList(image));
+  (void) SetImageProperty(image,"label",label);
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
+  width=(size_t) floor(draw_info->pointsize*strlen(label)+0.5);
+  if (AcquireMagickResource(WidthResource,width) == MagickFalse)
+    {
+      label=DestroyString(label);
+      draw_info=DestroyDrawInfo(draw_info);
+      ThrowReaderException(ImageError,"WidthOrHeightExceedsLimit");
+    }
   draw_info->text=ConstantString(label);
-  metrics.width=0;
+  metrics.width=0.0;
+  metrics.height=0.0;
   metrics.ascent=0.0;
   status=GetMultilineTypeMetrics(image,draw_info,&metrics);
   if ((image->columns == 0) && (image->rows == 0))
@@ -140,7 +148,7 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
       image->rows=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
     }
   else
-    if ((strlen(label) > 0) &&
+    if ((status != MagickFalse) && (strlen(label) > 0) &&
         (((image->columns == 0) || (image->rows == 0)) ||
          (fabs(image_info->pointsize) < MagickEpsilon)))
       {
@@ -148,16 +156,21 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
           high,
           low;
 
+        ssize_t
+          n;
+
         /*
           Auto fit text into bounding box.
         */
-        for ( ; ; draw_info->pointsize*=2.0)
+        for (n=0; n < 32; n++, draw_info->pointsize*=2.0)
         {
           (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
             -metrics.bounds.x1,metrics.ascent);
           if (draw_info->gravity == UndefinedGravity)
             (void) CloneString(&draw_info->geometry,geometry);
-          (void) GetMultilineTypeMetrics(image,draw_info,&metrics);
+          status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+          if (status == MagickFalse)
+            break;
           width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
           height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
           if ((image->columns != 0) && (image->rows != 0))
@@ -170,6 +183,13 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
                 ((image->rows != 0) && (height >= image->rows)))
               break;
         }
+        if (status == MagickFalse)
+          {
+            label=DestroyString(label);
+            draw_info=DestroyDrawInfo(draw_info);
+            image=DestroyImageList(image);
+            return((Image *) NULL);
+          }
         high=draw_info->pointsize;
         for (low=1.0; (high-low) > 0.5; )
         {
@@ -178,7 +198,9 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
             -metrics.bounds.x1,metrics.ascent);
           if (draw_info->gravity == UndefinedGravity)
             (void) CloneString(&draw_info->geometry,geometry);
-          (void) GetMultilineTypeMetrics(image,draw_info,&metrics);
+          status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+          if (status == MagickFalse)
+            break;
           width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
           height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
           if ((image->columns != 0) && (image->rows != 0))
@@ -195,9 +217,13 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
             else
               high=draw_info->pointsize-0.5;
         }
-        draw_info->pointsize=(low+high)/2.0-0.5;
+        if (status != MagickFalse)
+          {
+            draw_info->pointsize=(low+high)/2.0-0.5;
+            status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+          }
       }
-  status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+  label=DestroyString(label);
   if (status == MagickFalse)
     {
       draw_info=DestroyDrawInfo(draw_info);
@@ -234,9 +260,9 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
     Draw label.
   */
   (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
-    draw_info->direction == RightToLeftDirection ? image->columns-
+    draw_info->direction == RightToLeftDirection ? (double) image->columns-
     metrics.bounds.x2 : 0.0,draw_info->gravity == UndefinedGravity ?
-    metrics.ascent : 0.0);
+    MagickMax(metrics.ascent,metrics.bounds.y2) : 0.0);
   (void) CloneString(&draw_info->geometry,geometry);
   status=AnnotateImage(image,draw_info);
   if (image_info->pointsize == 0.0)
