@@ -17,13 +17,13 @@
 %                                 July 1999                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -171,7 +171,8 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
 
   Image
     *image,
-    *images;
+    *images,
+    *next;
 
   ImageInfo
     *read_info;
@@ -183,12 +184,12 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -201,15 +202,27 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
   /*
     Convert MPEG to PAM with delegate.
   */
+  images=(Image *) NULL;
   read_info=CloneImageInfo(image_info);
   image=AcquireImage(image_info);
-  (void) InvokeDelegate(read_info,image,"mpeg:decode",(char *) NULL,exception);
-  image=DestroyImage(image);
-  (void) FormatLocaleString(read_info->filename,MaxTextExtent,"%s.%s",
-    read_info->unique,ReadMPEGIntermediateFormat);
-  images=ReadImage(read_info,exception);
-  (void) RelinquishUniqueFileResource(read_info->filename);
+  status=InvokeDelegate(read_info,image,"mpeg:decode",(char *) NULL,exception);
+  if (status != MagickFalse)
+    {
+      (void) FormatLocaleString(read_info->filename,MaxTextExtent,"%s.%s",
+        read_info->unique,ReadMPEGIntermediateFormat);
+      *read_info->magick='\0';
+      images=ReadImage(read_info,exception);
+      if (images != (Image *) NULL)
+        for (next=images; next != (Image *) NULL; next=next->next)
+        {
+          (void) CopyMagickString(next->filename,image->filename,
+            MagickPathExtent);
+          (void) CopyMagickString(next->magick,image->magick,MagickPathExtent);
+        }
+      (void) RelinquishUniqueFileResource(read_info->filename);
+    }
   read_info=DestroyImageInfo(read_info);
+  image=DestroyImage(image);
   return(images);
 }
 
@@ -241,6 +254,20 @@ ModuleExport size_t RegisterMPEGImage(void)
   MagickInfo
     *entry;
 
+  entry=SetMagickInfo("3GP");
+  entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
+  entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
+  entry->description=ConstantString("Media Container");
+  entry->module=ConstantString("MPEG");
+  (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("3G2");
+  entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
+  entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
+  entry->description=ConstantString("Media Container");
+  entry->module=ConstantString("MPEG");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("AVI");
   entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
   entry->magick=(IsImageFormatHandler *) IsAVI;
@@ -354,6 +381,8 @@ ModuleExport void UnregisterMPEGImage(void)
   (void) UnregisterMagickInfo("MPEG");
   (void) UnregisterMagickInfo("MKV");
   (void) UnregisterMagickInfo("AVI");
+  (void) UnregisterMagickInfo("3G2");
+  (void) UnregisterMagickInfo("3GP");
 }
 
 /*
@@ -499,9 +528,9 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
