@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -65,6 +65,7 @@
 #include "magick/pixel-accessor.h"
 #include "magick/quantum-private.h"
 #include "magick/resize.h"
+#include "magick/resource_.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/module.h"
@@ -177,7 +178,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     Initialize Huffman tables.
   */
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(luma != (unsigned char *) NULL);
@@ -185,7 +186,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   assert(chroma2 != (unsigned char *) NULL);
   buffer=(unsigned char *) AcquireQuantumMemory(0x800,sizeof(*buffer));
   if (buffer == (unsigned char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+    ThrowBinaryImageException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   sum=0;
   bits=32;
@@ -204,7 +205,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     if (pcd_table[i] == (PCDTable *) NULL)
       {
         buffer=(unsigned char *) RelinquishMagickMemory(buffer);
-        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        ThrowBinaryImageException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
       }
     r=pcd_table[i];
@@ -283,7 +284,10 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           }
           default:
           {
-            ThrowBinaryException(CorruptImageError,"CorruptImage",
+            for (i=0; i < (image->columns > 1536 ? 3 : 1); i++)
+              pcd_table[i]=(PCDTable *) RelinquishMagickMemory(pcd_table[i]);
+            buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+            ThrowBinaryImageException(CorruptImageError,"CorruptImage",
               image->filename);
           }
         }
@@ -385,6 +389,7 @@ static MagickBooleanType IsPCD(const unsigned char *magick,const size_t length)
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
 static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   ExceptionInfo *exception)
 {
@@ -412,7 +417,7 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   montage_info=DestroyMontageInfo(montage_info);
   if (montage_image == (Image *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  image=DestroyImage(image);
+  image=DestroyImageList(image);
   return(montage_image);
 }
 
@@ -465,11 +470,24 @@ static void Upsample(const size_t width,const size_t height,
   }
   p=pixels+(2*height-2)*scaled_width;
   q=pixels+(2*height-1)*scaled_width;
-  (void) CopyMagickMemory(q,p,(size_t) (2*width));
+  (void) memcpy(q,p,(size_t) (2*width));
 }
 
 static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+#define ThrowPCDException(exception,message) \
+{ \
+  if (header != (unsigned char *) NULL) \
+    header=(unsigned char *) RelinquishMagickMemory(header); \
+  if (luma != (unsigned char *) NULL) \
+    luma=(unsigned char *) RelinquishMagickMemory(luma); \
+  if (chroma2 != (unsigned char *) NULL) \
+    chroma2=(unsigned char *) RelinquishMagickMemory(chroma2); \
+  if (chroma1 != (unsigned char *) NULL) \
+    chroma1=(unsigned char *) RelinquishMagickMemory(chroma1); \
+  ThrowReaderException((exception),(message)); \
+}
+
   Image
     *image;
 
@@ -518,12 +536,12 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -537,16 +555,24 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   header=(unsigned char *) AcquireQuantumMemory(0x800,3UL*sizeof(*header));
   if (header == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  chroma1=(unsigned char *) NULL;
+  chroma2=(unsigned char *) NULL;
+  luma=(unsigned char *) NULL;
   count=ReadBlob(image,3*0x800,header);
+  if (count != (3*0x800))
+    ThrowPCDException(CorruptImageError,"ImproperImageHeader");
   overview=LocaleNCompare((char *) header,"PCD_OPA",7) == 0;
-  if ((count != (3*0x800)) ||
-      ((LocaleNCompare((char *) header+0x800,"PCD",3) != 0) && (overview == 0)))
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if ((LocaleNCompare((char *) header+0x800,"PCD",3) != 0) && (overview == 0))
+    {
+      header=(unsigned char *) RelinquishMagickMemory(header);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    }
   rotate=header[0x0e02] & 0x03;
-  number_images=(header[10] << 8) | header[11];
-  if (number_images > 65535)
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  number_images=((header[10] << 8) | header[11]) & 0xffff;
   header=(unsigned char *) RelinquishMagickMemory(header);
+  if ((overview != 0) &&
+      (AcquireMagickResource(ListLengthResource,number_images) == MagickFalse))
+    ThrowPCDException(ResourceLimitError,"ListLengthExceedsLimit");
   /*
     Determine resolution by scene specification.
   */
@@ -587,6 +613,12 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->rows<<=1;
   }
   status=SetImageExtent(image,image->columns,image->rows);
+  if (status == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
+  status=ResetImagePixels(image,exception);
   if (status == MagickFalse)
     {
       InheritException(exception,&image->exception);
@@ -597,16 +629,22 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   number_pixels=(MagickSizeType) image->columns*image->rows;
   if (number_pixels != (size_t) number_pixels)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    ThrowPCDException(ResourceLimitError,"MemoryAllocationFailed");
   chroma1=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
-    sizeof(*chroma1));
+    10*sizeof(*chroma1));
   chroma2=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
-    sizeof(*chroma2));
+    10*sizeof(*chroma2));
   luma=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
-    sizeof(*luma));
+    10*sizeof(*luma));
   if ((chroma1 == (unsigned char *) NULL) ||
       (chroma2 == (unsigned char *) NULL) || (luma == (unsigned char *) NULL))
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    ThrowPCDException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) memset(chroma1,0,(image->columns+1UL)*image->rows*
+    10*sizeof(*chroma1));
+  (void) memset(chroma2,0,(image->columns+1UL)*image->rows*
+    10*sizeof(*chroma2));
+  (void) memset(luma,0,(image->columns+1UL)*image->rows*
+    10*sizeof(*luma));
   /*
     Advance to image data.
   */
@@ -620,12 +658,10 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (scene <= 1)
         offset=1;
   for (i=0; i < (ssize_t) (offset*0x800); i++)
-    (void) ReadBlobByte(image);
+    if (ReadBlobByte(image) == EOF)
+      ThrowPCDException(CorruptImageError,"UnexpectedEndOfFile");
   if (overview != 0)
     {
-      Image
-        *overview_image;
-
       MagickProgressMonitor
         progress_monitor;
 
@@ -660,6 +696,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           c1+=image->columns;
           count=ReadBlob(image,width >> 1,c2);
           c2+=image->columns;
+          if (EOFBlob(image) != MagickFalse)
+            ThrowPCDException(CorruptImageError,"UnexpectedEndOfFile");
         }
         Upsample(image->columns >> 1,image->rows >> 1,image->columns,chroma1);
         Upsample(image->columns >> 1,image->rows >> 1,image->columns,chroma2);
@@ -686,7 +724,9 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         image->colorspace=YCCColorspace;
         if (LocaleCompare(image_info->magick,"PCDS") == 0)
-          SetImageColorspace(image,sRGBColorspace);
+          (void) SetImageColorspace(image,sRGBColorspace);
+        if (EOFBlob(image) != MagickFalse)
+          break;
         if (j < (ssize_t) number_images)
           {
             /*
@@ -695,8 +735,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             AcquireNextImage(image_info,image);
             if (GetNextImageInList(image) == (Image *) NULL)
               {
-                image=DestroyImageList(image);
-                return((Image *) NULL);
+                status=MagickFalse;
+                break;
               }
             image=SyncNextImageInList(image);
           }
@@ -712,9 +752,9 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
       chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
       luma=(unsigned char *) RelinquishMagickMemory(luma);
-      image=GetFirstImageInList(image);
-      overview_image=OverviewImage(image_info,image,exception);
-      return(overview_image);
+      if (status == MagickFalse)
+        return(DestroyImageList(image));
+      return(OverviewImage(image_info,GetFirstImageInList(image),exception));
     }
   /*
     Read interleaved image.
@@ -732,6 +772,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     c1+=image->columns;
     count=ReadBlob(image,width >> 1,c2);
     c2+=image->columns;
+    if (EOFBlob(image) != MagickFalse)
+      ThrowPCDException(CorruptImageError,"UnexpectedEndOfFile");
   }
   if (scene >= 4)
     {
@@ -840,7 +882,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->gamma=1.000f/2.200f;
   image->colorspace=YCCColorspace;
   if (LocaleCompare(image_info->magick,"PCDS") == 0)
-    SetImageColorspace(image,sRGBColorspace);
+    (void) SetImageColorspace(image,sRGBColorspace);
   return(GetFirstImageInList(image));
 }
 
@@ -1072,9 +1114,9 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
     i;
 
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   pcd_image=image;
@@ -1090,13 +1132,19 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
       if (rotate_image == (Image *) NULL)
         return(MagickFalse);
       pcd_image=rotate_image;
+      DestroyBlob(rotate_image);
+      pcd_image->blob=ReferenceBlob(image->blob);
     }
   /*
     Open output image file.
   */
   status=OpenBlob(image_info,pcd_image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
-    return(status);
+    {
+      if (pcd_image != image)
+        pcd_image=DestroyImage(pcd_image);
+      return(status);
+    }
   if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
     (void) TransformImageColorspace(pcd_image,sRGBColorspace);
   /*
