@@ -18,13 +18,13 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -61,7 +61,7 @@
 #include "magick/utility.h"
 #if defined(MAGICKCORE_XML_DELEGATE)
 #  if defined(MAGICKCORE_WINDOWS_SUPPORT)
-#    if !defined(__MINGW32__) && !defined(__MINGW64__)
+#    if !defined(__MINGW32__)
 #      include <win32config.h>
 #    endif
 #  endif
@@ -71,7 +71,7 @@
 #  include <libxml/nanohttp.h>
 #endif
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !(defined(__MINGW32__) || defined(__MINGW64__))
+    !defined(__MINGW32__)
 #  include <urlmon.h>
 #  pragma comment(lib, "urlmon.lib")
 #endif
@@ -176,7 +176,7 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   LocaleLower(filename);
   (void) ConcatenateMagickString(filename,image_info->filename,MaxTextExtent);
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !(defined(__MINGW32__) || defined(__MINGW64__))
+    !defined(__MINGW32__)
   (void) fclose(file);
   if (URLDownloadToFile(NULL,filename,read_info->filename,0,NULL) != S_OK)
     {
@@ -281,7 +281,7 @@ ModuleExport size_t RegisterURLImage(void)
 
   entry=SetMagickInfo("HTTP");
 #if (defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !(defined(__MINGW32__) || defined(__MINGW64__))) || \
+    !defined(__MINGW32__)) || \
     (defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_HTTP_ENABLED))
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
@@ -291,7 +291,7 @@ ModuleExport size_t RegisterURLImage(void)
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("HTTPS");
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !(defined(__MINGW32__) || defined(__MINGW64__))
+    !defined(__MINGW32__)
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
   entry->description=ConstantString("Uniform Resource Locator (https://)");
@@ -300,7 +300,7 @@ ModuleExport size_t RegisterURLImage(void)
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("FTP");
 #if (defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !(defined(__MINGW32__) || defined(__MINGW64__))) || \
+    !defined(__MINGW32__)) || \
     (defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED))
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
