@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -102,7 +102,8 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   char
     **filelist,
-    *label;
+    *label,
+    **list;
 
   Image
     *image,
@@ -133,19 +134,22 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Expand the filename.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
-  filelist=(char **) AcquireMagickMemory(sizeof(*filelist));
-  if (filelist == (char **) NULL)
+  list=(char **) AcquireMagickMemory(sizeof(*filelist));
+  if (list == (char **) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  filelist[0]=ConstantString(image_info->filename);
+  list[0]=ConstantString(image_info->filename);
+  filelist=list;
   number_files=1;
   status=ExpandFilenames(&number_files,&filelist);
+  list[0]=DestroyString(list[0]);
+  list=(char **) RelinquishMagickMemory(list);
   if ((status == MagickFalse) || (number_files == 0))
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   image=DestroyImage(image);
