@@ -17,13 +17,13 @@
 %                               December 2000                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -62,13 +62,15 @@
 #include "magick/module.h"
 #include "magick/type.h"
 #include "magick/module.h"
+#if defined(MAGICKCORE_WMF_DELEGATE)
 #include "wand/MagickWand.h"
-
-#if defined(__CYGWIN__)
-#undef MAGICKCORE_WMF_DELEGATE
 #endif
 
-#if defined(MAGICKCORE_WMF_DELEGATE)
+#if defined(__CYGWIN__)
+#undef MAGICKCORE_SANS_DELEGATE
+#endif
+
+#if defined(MAGICKCORE_SANS_DELEGATE)
 #include "libwmf/api.h"
 #include "libwmf/eps.h"
 
@@ -178,6 +180,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void *) image);
   if (wmf_status != wmf_E_None)
     {
+      ipa_device_close(wmf_info);
       wmf_api_destroy(wmf_info);
       ThrowFileException(exception,FileOpenError,"UnableToOpenFile",
         image->filename);
@@ -187,6 +190,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   wmf_status=wmf_scan(wmf_info,0,&bounding_box);
   if (wmf_status != wmf_E_None)
     {
+      ipa_device_close(wmf_info);
       wmf_api_destroy(wmf_info);
       ThrowReaderException(DelegateError,"FailedToScanFile");
     }
@@ -197,6 +201,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     file=fdopen(unique_file,"wb");
   if ((unique_file == -1) || (file == (FILE *) NULL))
     {
+      ipa_device_close(wmf_info);
       wmf_api_destroy(wmf_info);
       ThrowReaderException(FileOpenError,"UnableToCreateTemporaryFile");
     }
@@ -205,6 +210,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   wmf_status=wmf_play(wmf_info,0,&bounding_box);
   if (wmf_status != wmf_E_None)
     {
+      ipa_device_close(wmf_info);
       wmf_api_destroy(wmf_info);
       ThrowReaderException(DelegateError,"FailedToRenderFile");
     }
@@ -232,7 +238,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) RelinquishUniqueFileResource(filename);
   return(GetFirstImageInList(image));
 }
-#elif defined(MAGICKCORE_WMFLITE_DELEGATE)
+#elif defined(MAGICKCORE_WMF_DELEGATE)
 
 #define ERR(API)  ((API)->err != wmf_E_None)
 #define XC(x) ((double) x)
@@ -297,7 +303,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #define CENTIMETERS_PER_INCH  2.54
 #define POINTS_PER_INCH       72
 
-#if defined(MAGICKCORE_WMFLITE_DELEGATE)
+#if defined(MAGICKCORE_WMF_DELEGATE)
 # define wmf_api_create(api,flags,options) wmf_lite_create(api,flags,options)
 # define wmf_api_destroy(api) wmf_lite_destroy(api)
 # undef WMF_FONT_PSNAME
@@ -388,7 +394,7 @@ typedef enum
 }
 magick_arc_t;
 
-#if defined(MAGICKCORE_WMFLITE_DELEGATE)
+#if defined(MAGICKCORE_WMF_DELEGATE)
 static void  lite_font_init (wmfAPI* API, wmfAPI_Options* options);
 static void  lite_font_map(wmfAPI* API,wmfFont* font);
 static float lite_font_stringwidth(wmfAPI* API, wmfFont* font, char* str);
@@ -414,7 +420,7 @@ static void         ipa_draw_line(wmfAPI * API, wmfDrawLine_t * draw_line);
 static void         ipa_draw_pie(wmfAPI * API, wmfDrawArc_t * draw_arc);
 static void         ipa_draw_pixel(wmfAPI * API, wmfDrawPixel_t * draw_pixel);
 static void         ipa_draw_polygon(wmfAPI * API, wmfPolyLine_t * poly_line);
-#if defined(MAGICKCORE_WMFLITE_DELEGATE)
+#if defined(MAGICKCORE_WMF_DELEGATE)
 static void         ipa_draw_polypolygon(wmfAPI * API, wmfPolyPoly_t* polypolygon);
 #endif
 static void         ipa_draw_rectangle(wmfAPI * API, wmfDrawRectangle_t * draw_rect);
@@ -433,7 +439,7 @@ static void         ipa_udata_init(wmfAPI * API, wmfUserData_t * userdata);
 static void         ipa_udata_set(wmfAPI * API, wmfUserData_t * userdata);
 static int          magick_progress_callback(void* wand,float quantum);
 static void         util_draw_arc(wmfAPI * API, wmfDrawArc_t * draw_arc,magick_arc_t finish);
-#if defined(MAGICKCORE_WMFLITE_DELEGATE)
+#if defined(MAGICKCORE_WMF_DELEGATE)
 /*static int          util_font_weight( const char* font );*/
 #endif
 static double       util_pointsize( wmfAPI* API, wmfFont* font, char* str, double font_height);
@@ -450,7 +456,7 @@ int magick_progress_callback(void *context,float quantum)
     status;
 
   image=(Image *) context;
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
     GetBlobSize(image));
   return(status != MagickFalse ? 0 : 1);
@@ -525,7 +531,7 @@ static void draw_pattern_push( wmfAPI* API,
                                unsigned long rows )
 {
   char
-    pattern_id[30];
+    pattern_id[MaxTextExtent];
 
   (void) FormatLocaleString(pattern_id,MaxTextExtent,"brush_%lu",id);
   (void) DrawPushPattern(WmfDrawingWand,pattern_id,0,0,columns,rows);
@@ -812,7 +818,7 @@ static void ipa_device_open(wmfAPI * API)
 
   ddata->push_depth = 0;
 
-  ddata->draw_wand = DrawAllocateWand(ddata->draw_info,ddata->image);
+  ddata->draw_wand = AcquireDrawingWand(ddata->draw_info,ddata->image);
 }
 
 /*
@@ -823,9 +829,19 @@ static void ipa_device_close(wmfAPI * API)
   wmf_magick_t
     *ddata = WMF_MAGICK_GetData(API);
 
-  DestroyDrawingWand(ddata->draw_wand);
-  DestroyDrawInfo(ddata->draw_info);
-  RelinquishMagickMemory(WMF_MAGICK_GetFontData(API)->ps_name);
+  if (ddata->draw_wand != (DrawingWand *) NULL)
+    {
+      DestroyDrawingWand(ddata->draw_wand);
+      ddata->draw_wand=(DrawingWand *) NULL;
+    }
+  if (ddata->draw_info != (DrawInfo *) NULL)
+    {
+      DestroyDrawInfo(ddata->draw_info);
+      ddata->draw_info=(DrawInfo *)NULL;
+    }
+  if (WMF_MAGICK_GetFontData(API)->ps_name)
+    WMF_MAGICK_GetFontData(API)->ps_name=(char *) RelinquishMagickMemory(
+      WMF_MAGICK_GetFontData(API)->ps_name);
 }
 
 /*
@@ -834,7 +850,8 @@ static void ipa_device_close(wmfAPI * API)
 static void ipa_device_begin(wmfAPI * API)
 {
   char
-    comment[MaxTextExtent];
+    comment[MaxTextExtent],
+    *url;
 
   wmf_magick_t
     *ddata = WMF_MAGICK_GetData(API);
@@ -844,8 +861,10 @@ static void ipa_device_begin(wmfAPI * API)
 
   DrawSetViewbox(WmfDrawingWand, 0, 0, ddata->image->columns, ddata->image->rows );
 
+  url=GetMagickHomeURL();
   (void) FormatLocaleString(comment,MaxTextExtent,"Created by ImageMagick %s",
-    GetMagickVersion((size_t *) NULL));
+    url);
+  url=DestroyString(url);
   DrawComment(WmfDrawingWand,comment);
 
   /* Scale width and height to image */
@@ -896,7 +915,7 @@ static void ipa_device_begin(wmfAPI * API)
       if (image)
         {
           char
-            pattern_id[30];
+            pattern_id[MaxTextExtent];
 
           MagickWand
             *magick_wand;
@@ -1212,7 +1231,7 @@ static void ipa_draw_polygon(wmfAPI * API, wmfPolyLine_t * polyline)
 }
 
 /* Draw a polypolygon.  A polypolygon is a list of polygons */
-#if defined(MAGICKCORE_WMFLITE_DELEGATE)
+#if defined(MAGICKCORE_WMF_DELEGATE)
 static void ipa_draw_polypolygon(wmfAPI * API, wmfPolyPoly_t* polypolygon)
 {
   if (TO_FILL(polypolygon) || TO_DRAW(polypolygon))
@@ -1354,7 +1373,7 @@ static void ipa_region_clip(wmfAPI *API, wmfPolyRectangle_t *poly_rect)
   if (poly_rect->count > 0)
     {
       char
-        clip_mask_id[30];
+        clip_mask_id[MaxTextExtent];
 
       /* Define clip path */
       ddata->clip_mask_id++;
@@ -1405,7 +1424,7 @@ static void ipa_functions(wmfAPI *API)
   FR->draw_line = ipa_draw_line;
   FR->poly_line = ipa_poly_line;
   FR->draw_polygon = ipa_draw_polygon;
-#if defined(MAGICKCORE_WMFLITE_DELEGATE)
+#if defined(MAGICKCORE_WMF_DELEGATE)
   FR->draw_polypolygon = ipa_draw_polypolygon;
 #endif
   FR->draw_rectangle = ipa_draw_rectangle;
@@ -1429,7 +1448,7 @@ static void ipa_functions(wmfAPI *API)
   if (ERR(API))
     return;
 
-  (void) ResetMagickMemory((void *) ddata, 0, sizeof(wmf_magick_t));
+  (void) memset((void *) ddata, 0, sizeof(wmf_magick_t));
   API->device_data = (void *) ddata;
 
   /*
@@ -1544,7 +1563,7 @@ static void ipa_draw_text(wmfAPI * API, wmfDrawText_t * draw_text)
               double
                 text_width = metrics.width * (ddata->scale_y / ddata->scale_x);
 
-#if defined(MAGICKCORE_WMFLITE_DELEGATE)
+#if defined(MAGICKCORE_WMF_DELEGATE)
               point.x -= text_width / 2;
 #else
               point.x += bbox_width / 2 - text_width / 2;
@@ -1714,7 +1733,7 @@ static void ipa_udata_set(wmfAPI * API, wmfUserData_t * userdata)
 
 }
 
-static void ipa_udata_free(wmfAPI * API, wmfUserData_t * userdata)
+static void ipa_udata_free(wmfAPI *API, wmfUserData_t *userdata)
 {
   (void) API;
   (void) userdata;
@@ -1722,14 +1741,7 @@ static void ipa_udata_free(wmfAPI * API, wmfUserData_t * userdata)
 
 }
 
-static inline double MagickMin(const double x,const double y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
-static void util_set_brush(wmfAPI * API, wmfDC * dc, const BrushApply brush_apply)
+static void util_set_brush(wmfAPI *API, wmfDC *dc,const BrushApply brush_apply)
 {
   wmf_magick_t
     *ddata = WMF_MAGICK_GetData(API);
@@ -1841,7 +1853,7 @@ static void util_set_brush(wmfAPI * API, wmfDC * dc, const BrushApply brush_appl
         DrawPopDefs(WmfDrawingWand);
         {
           char
-            pattern_id[30];
+            pattern_id[MaxTextExtent];
 
           (void) FormatLocaleString(pattern_id,MaxTextExtent,"#brush_%lu",
             ddata->pattern_id);
@@ -1954,7 +1966,7 @@ static void util_set_brush(wmfAPI * API, wmfDC * dc, const BrushApply brush_appl
 
             {
               char
-                pattern_id[30];
+                pattern_id[MaxTextExtent];
 
               (void) FormatLocaleString(pattern_id,MaxTextExtent,"#brush_%lu",
                 ddata->pattern_id);
@@ -1994,13 +2006,6 @@ static void util_set_brush(wmfAPI * API, wmfDC * dc, const BrushApply brush_appl
     }
 }
 
-static inline double MagickMax(const double x,const double y)
-{
-  if (x > y)
-    return(x);
-  return(y);
-}
-
 static void util_set_pen(wmfAPI * API, wmfDC * dc)
 {
   wmf_magick_t
@@ -2014,8 +2019,7 @@ static void util_set_pen(wmfAPI * API, wmfDC * dc)
     pixel_width;
 
   unsigned int
-    pen_style,
-    pen_type;
+    pen_style;
 
   pen = WMF_DC_PEN(dc);
 
@@ -2030,8 +2034,6 @@ static void util_set_pen(wmfAPI * API, wmfDC * dc)
   pen_width = MagickMax(pen_width, pixel_width*0.8);
 
   pen_style = (unsigned int) WMF_PEN_STYLE(pen);
-  pen_type = (unsigned int) WMF_PEN_TYPE(pen);
-  (void) pen_type;
 
   /* Pen style specified? */
   if (pen_style == PS_NULL)
@@ -2144,7 +2146,7 @@ static void util_set_pen(wmfAPI * API, wmfDC * dc)
       case PS_SOLID:
       default:
         {
-          (void) DrawSetStrokeDashArray(WmfDrawingWand,0,(double *)NULL);
+          (void) DrawSetStrokeDashArray(WmfDrawingWand,0,(double *) NULL);
           break;
         }
       }
@@ -2223,7 +2225,7 @@ static double util_pointsize( wmfAPI* API, wmfFont* font, char* str, double font
   return floor(pointsize);
 }
 
-#if defined(MAGICKCORE_WMFLITE_DELEGATE)
+#if defined(MAGICKCORE_WMF_DELEGATE)
 /* Estimate weight based on font name */
 /*
 static int util_font_weight( const char* font )
@@ -2544,7 +2546,7 @@ static void lite_font_init( wmfAPI* API, wmfAPI_Options* options)
   ((wmf_magick_font_t*)font_data->user_data)->pointsize = 0;
 }
 
-#endif /* MAGICKCORE_WMFLITE_DELEGATE */
+#endif /* MAGICKCORE_WMF_DELEGATE */
 
 /* BLOB read byte */
 static int ipa_blob_read(void* wand)
@@ -2584,6 +2586,9 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
+  MagickBooleanType
+    status;
+
   unsigned long
     wmf_options_flags = 0;
 
@@ -2623,7 +2628,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   /* Register callbacks */
   wmf_options_flags |= WMF_OPT_FUNCTION;
-  (void) ResetMagickMemory(&wmf_api_options, 0, sizeof(wmf_api_options));
+  (void) memset(&wmf_api_options, 0, sizeof(wmf_api_options));
   wmf_api_options.function = ipa_functions;
 
   /* Ignore non-fatal errors */
@@ -2632,8 +2637,6 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   wmf_error = wmf_api_create(&API, wmf_options_flags, &wmf_api_options);
   if (wmf_error != wmf_E_None)
     {
-      if (API)
-        wmf_api_destroy(API);
       if (image->debug != MagickFalse)
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -2641,6 +2644,8 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "leave ReadWMFImage()");
         }
+      if (API)
+        wmf_api_destroy(API);
       ThrowReaderException(DelegateError,"UnableToInitializeWMFLibrary");
     }
 
@@ -2656,7 +2661,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ddata->draw_info->text=(char *)
     RelinquishMagickMemory(ddata->draw_info->text);
 
-#if defined(MAGICKCORE_WMFLITE_DELEGATE)
+#if defined(MAGICKCORE_WMF_DELEGATE)
   /* Must initialize font subystem for WMFlite interface */
   lite_font_init (API,&wmf_api_options); /* similar to wmf_ipa_font_init in src/font.c */
   /* wmf_arg_fontdirs (API,options); */ /* similar to wmf_arg_fontdirs in src/wmf.c */
@@ -2671,7 +2676,11 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     ipa_blob_tell,(void*)image);
   if (wmf_error != wmf_E_None)
     {
-      wmf_api_destroy(API);
+      if (ddata->draw_info != (DrawInfo *) NULL)
+        {
+          DestroyDrawInfo(ddata->draw_info);
+          ddata->draw_info=(DrawInfo *)NULL;
+        }
       if (image->debug != MagickFalse)
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -2679,6 +2688,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "leave ReadWMFImage()");
         }
+      wmf_api_destroy(API);
       ThrowFileException(exception,FileOpenError,"UnableToOpenFile",
         image->filename);
       image=DestroyImageList(image);
@@ -2695,7 +2705,6 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   wmf_error=wmf_scan(API, 0, &bbox);
   if (wmf_error != wmf_E_None)
     {
-      wmf_api_destroy(API);
       if (image->debug != MagickFalse)
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -2703,6 +2712,8 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "leave ReadWMFImage()");
         }
+      ipa_device_close(API);
+      wmf_api_destroy(API);
       ThrowReaderException(DelegateError,"FailedToScanFile");
     }
 
@@ -2733,7 +2744,6 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   wmf_error=wmf_size(API,&wmf_width,&wmf_height);
   if (wmf_error != wmf_E_None)
     {
-      wmf_api_destroy(API);
       if (image->debug != MagickFalse)
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -2741,6 +2751,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "leave ReadWMFImage()");
         }
+      wmf_api_destroy(API);
       ThrowReaderException(DelegateError,"FailedToComputeOutputSize");
     }
 
@@ -2847,7 +2858,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     printf("dc->Window.height = %d\n", dc->Window.height);
     printf("dc->pixel_width   = %g\n", dc->pixel_width);
     printf("dc->pixel_height  = %g\n", dc->pixel_height);
-#if defined(MAGICKCORE_WMFLITE_DELEGATE)  /* Only in libwmf 0.3 */
+#if defined(MAGICKCORE_WMF_DELEGATE)  /* Only in libwmf 0.3 */
     printf("dc->Ox            = %.d\n", dc->Ox);
     printf("dc->Oy            = %.d\n", dc->Oy);
     printf("dc->width         = %.d\n", dc->width);
@@ -2875,6 +2886,12 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           "leave ReadWMFImage()");
       return(GetFirstImageInList(image));
     }
+  status=SetImageExtent(image,image->columns,image->rows);
+  if (status == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
        "  Creating canvas image with size %lux%lu",(unsigned long) image->rows,
@@ -2901,7 +2918,6 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   wmf_error = wmf_play(API, 0, &bbox);
   if (wmf_error != wmf_E_None)
     {
-      wmf_api_destroy(API);
       if (image->debug != MagickFalse)
         {
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -2909,6 +2925,7 @@ static Image *ReadWMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "leave ReadWMFImage()");
         }
+      wmf_api_destroy(API);
       ThrowReaderException(DelegateError,"FailedToRenderFile");
     }
 
@@ -2963,15 +2980,15 @@ ModuleExport size_t RegisterWMFImage(void)
     *entry;
 
   entry = SetMagickInfo("WMZ");
-#if defined(MAGICKCORE_WMF_DELEGATE) || defined(MAGICKCORE_WMFLITE_DELEGATE)
+#if defined(MAGICKCORE_SANS_DELEGATE) || defined(MAGICKCORE_WMF_DELEGATE)
   entry->decoder=ReadWMFImage;
 #endif
   entry->description=ConstantString("Compressed Windows Meta File");
-  entry->module=ConstantString("WMZ");
+  entry->module=ConstantString("WMF");
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("WMF");
-#if defined(MAGICKCORE_WMF_DELEGATE) || defined(MAGICKCORE_WMFLITE_DELEGATE)
+#if defined(MAGICKCORE_SANS_DELEGATE) || defined(MAGICKCORE_WMF_DELEGATE)
   entry->decoder=ReadWMFImage;
 #endif
   entry->description=ConstantString("Windows Meta File");
