@@ -23,7 +23,7 @@
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -746,8 +746,8 @@ static MagickBooleanType load_hierarchy(Image *image,XCFDocInfo *inDocInfo,
     offset,
     junk;
 
-  (void) ReadBlobMSBLong(image); // width
-  (void) ReadBlobMSBLong(image); // height
+  (void) ReadBlobMSBLong(image);  /* width */
+  (void) ReadBlobMSBLong(image);  /* height */
   inDocInfo->bytes_per_pixel=ReadBlobMSBLong(image);
 
   /* load in the levels...we make sure that the number of levels
@@ -756,7 +756,7 @@ static MagickBooleanType load_hierarchy(Image *image,XCFDocInfo *inDocInfo,
    */
   offset=(MagickOffsetType) ReadBlobMSBLong(image);  /* top level */
   if ((MagickSizeType) offset >= GetBlobSize(image))
-    ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
+    ThrowBinaryImageException(CorruptImageError,"InsufficientImageDataInFile",
       image->filename);
 
   /* discard offsets for layers below first, if any.
@@ -774,7 +774,7 @@ static MagickBooleanType load_hierarchy(Image *image,XCFDocInfo *inDocInfo,
 
   /* seek to the level offset */
   if (SeekBlob(image, offset, SEEK_SET) != offset)
-    ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
+    ThrowBinaryImageException(CorruptImageError,"InsufficientImageDataInFile",
       image->filename);
 
   /* read in the level */
@@ -821,10 +821,10 @@ static MagickBooleanType ReadOneLayer(const ImageInfo *image_info,Image* image,
   (void) ReadBlobStringWithLongSize(image, outLayer->name,
     sizeof(outLayer->name),&image->exception);
   if (EOFBlob(image) != MagickFalse)
-    ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
+    ThrowBinaryImageException(CorruptImageError,"InsufficientImageDataInFile",
       image->filename);
   if ((outLayer->width == 0) || (outLayer->height == 0))
-    ThrowBinaryException(CorruptImageError,"ImproperImageHeader",
+    ThrowBinaryImageException(CorruptImageError,"ImproperImageHeader",
       image->filename);
   /* read the layer properties! */
   foundPropEnd = 0;
@@ -907,7 +907,7 @@ static MagickBooleanType ReadOneLayer(const ImageInfo *image_info,Image* image,
         amount = (ssize_t) MagickMin(16, prop_size);
         amount = ReadBlob(image, (size_t) amount, (unsigned char *) &buf);
         if (!amount)
-          ThrowBinaryException(CorruptImageError,"CorruptImage",
+          ThrowBinaryImageException(CorruptImageError,"CorruptImage",
             image->filename);
         prop_size -= (size_t) MagickMin(16, (size_t) amount);
         }
@@ -916,7 +916,7 @@ static MagickBooleanType ReadOneLayer(const ImageInfo *image_info,Image* image,
     }
   }
   if (EOFBlob(image) != MagickFalse)
-    ThrowBinaryException(CorruptImageError,"UnexpectedEndOfFile",
+    ThrowBinaryImageException(CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
   if (foundPropEnd == MagickFalse)
     return(MagickFalse);
