@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
   
-    http://www.imagemagick.org/script/license.php
+    https://imagemagick.org/script/license.php
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -142,6 +142,9 @@ typedef struct _OpenCLCacheInfo
 
   PixelPacket
     *pixels;
+
+  SemaphoreInfo
+    *events_semaphore;
 } OpenCLCacheInfo;
 
 typedef struct _CacheInfo
@@ -159,7 +162,8 @@ typedef struct _CacheInfo
     type;
 
   MapMode
-    mode;
+    mode,
+    disk_mode;
 
   MagickBooleanType
     mapped;
@@ -257,7 +261,7 @@ extern MagickExport const IndexPacket
   *GetVirtualIndexesFromNexus(const Cache,NexusInfo *);
 
 extern MagickExport const PixelPacket
-  *GetVirtualPixelsFromNexus(const Image *,const VirtualPixelMethod,
+  *GetVirtualPixelCacheNexus(const Image *,const VirtualPixelMethod,
     const ssize_t,const ssize_t,const size_t,const size_t,NexusInfo *,
     ExceptionInfo *) magick_hot_spot,
   *GetVirtualPixelsNexus(const Cache,NexusInfo *);
@@ -298,7 +302,7 @@ extern MagickPrivate MagickBooleanType
   SyncImagePixelCache(Image *,ExceptionInfo *);
 
 #if defined(MAGICKCORE_OPENCL_SUPPORT)
-extern MagickPrivate const cl_event
+extern MagickPrivate cl_event
   *GetOpenCLEvents(const Image *,cl_uint *);
 
 extern MagickPrivate cl_mem
