@@ -17,13 +17,13 @@
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -176,8 +176,8 @@ static LinkedListInfo *AcquireConfigureCache(const char *filename,
     option=(const StringInfo *) GetNextValueInLinkedList(options);
     while (option != (const StringInfo *) NULL)
     {
-      status&=LoadConfigureCache(cache,(const char *) GetStringInfoDatum(option),
-        GetStringInfoPath(option),0,exception);
+      status&=LoadConfigureCache(cache,(const char *)
+        GetStringInfoDatum(option),GetStringInfoPath(option),0,exception);
       option=(const StringInfo *) GetNextValueInLinkedList(options);
     }
     options=DestroyConfigureOptions(options);
@@ -203,12 +203,12 @@ static LinkedListInfo *AcquireConfigureCache(const char *filename,
           ResourceLimitError,"MemoryAllocationFailed","`%s'",p->name);
         continue;
       }
-    (void) ResetMagickMemory(configure_info,0,sizeof(*configure_info));
+    (void) memset(configure_info,0,sizeof(*configure_info));
     configure_info->path=(char *) "[built-in]";
     configure_info->name=(char *) p->name;
     configure_info->value=(char *) p->value;
     configure_info->exempt=MagickTrue;
-    configure_info->signature=MagickSignature;
+    configure_info->signature=MagickCoreSignature;
     status&=AppendValueToLinkedList(cache,configure_info);
     if (status == MagickFalse)
       (void) ThrowMagickException(exception,GetMagickModule(),
@@ -960,6 +960,9 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
       }
   }
 #endif
+  if (GetNumberOfElementsInLinkedList(paths) == 0)
+    (void) ThrowMagickException(exception,GetMagickModule(),ConfigureWarning,
+      "no configuration paths found","`%s'",filename);
   return(paths);
 }
 
@@ -989,7 +992,7 @@ MagickExport const char *GetConfigureValue(const ConfigureInfo *configure_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(configure_info != (ConfigureInfo *) NULL);
-  assert(configure_info->signature == MagickSignature);
+  assert(configure_info->signature == MagickCoreSignature);
   return(configure_info->value);
 }
 
@@ -1216,7 +1219,7 @@ static MagickBooleanType LoadConfigureCache(LinkedListInfo *cache,
           GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
-              if (depth > 200)
+              if (depth > MagickMaxRecursionDepth)
                 (void) ThrowMagickException(exception,GetMagickModule(),
                   ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
               else
@@ -1254,10 +1257,10 @@ static MagickBooleanType LoadConfigureCache(LinkedListInfo *cache,
           sizeof(*configure_info));
         if (configure_info == (ConfigureInfo *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-        (void) ResetMagickMemory(configure_info,0,sizeof(*configure_info));
+        (void) memset(configure_info,0,sizeof(*configure_info));
         configure_info->path=ConstantString(filename);
         configure_info->exempt=MagickFalse;
-        configure_info->signature=MagickSignature;
+        configure_info->signature=MagickCoreSignature;
         continue;
       }
     if (configure_info == (ConfigureInfo *) NULL)
