@@ -17,13 +17,13 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -47,6 +47,7 @@
 #include "magick/exception-private.h"
 #include "magick/cache.h"
 #include "magick/client.h"
+#include "magick/coder.h"
 #include "magick/colorspace-private.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
@@ -135,6 +136,9 @@ MagickExport Image *ConstituteImage(const size_t columns,
   MagickBooleanType
     status;
 
+  register ssize_t
+    i;
+
   /*
     Allocate image structure.
   */
@@ -142,15 +146,60 @@ MagickExport Image *ConstituteImage(const size_t columns,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",map);
   assert(pixels != (void *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage((ImageInfo *) NULL);
   if (image == (Image *) NULL)
     return((Image *) NULL);
-  if ((columns == 0) || (rows == 0))
-    ThrowImageException(OptionError,"NonZeroWidthAndHeightRequired");
-  image->columns=columns;
-  image->rows=rows;
-  (void) SetImageBackgroundColor(image);
+  for (i=0; i < (ssize_t) strlen(map); i++)
+  {
+    switch (map[i])
+    {
+      case 'a':
+      case 'A':
+      case 'O':
+      case 'o':
+      {
+        image->matte=MagickTrue;
+        break;
+      }
+      case 'C':
+      case 'c':
+      case 'm':
+      case 'M':
+      case 'Y':
+      case 'y':
+      case 'K':
+      case 'k':
+      {
+        image->colorspace=CMYKColorspace;
+        break;
+      }
+      case 'I':
+      case 'i':
+      {
+        image->colorspace=GRAYColorspace;
+        break;
+      }
+      default:
+      {
+        if (strlen(map) == 1)
+          image->colorspace=GRAYColorspace;
+        break;
+      }
+    }
+  }
+  status=SetImageExtent(image,columns,rows);
+  if (status == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      image=DestroyImage(image);
+    }
+  status=ResetImagePixels(image,exception);
+  if (status == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      image=DestroyImage(image);
+    }
   status=ImportImagePixels(image,0,0,columns,rows,map,storage,pixels);
   if (status == MagickFalse)
     {
@@ -216,7 +265,7 @@ MagickExport Image *PingImage(const ImageInfo *image_info,
     *ping_info;
 
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
@@ -275,7 +324,7 @@ MagickExport Image *PingImages(const ImageInfo *image_info,
     Ping image list from a file.
   */
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
@@ -349,6 +398,20 @@ MagickExport Image *PingImages(const ImageInfo *image_info,
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
+static MagickBooleanType IsCoderAuthorized(const char *coder,
+  const PolicyRights rights,ExceptionInfo *exception)
+{
+  if (IsRightsAuthorized(CoderPolicyDomain,rights,coder) == MagickFalse)
+    {
+      errno=EPERM;
+      (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
+        "NotAuthorized","`%s'",coder);
+      return(MagickFalse);
+    }
+  return(MagickTrue);
+}
+
 MagickExport Image *ReadImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
@@ -379,21 +442,18 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
   ImageInfo
     *read_info;
 
+  MagickBooleanType
+    status;
+
   MagickStatusType
     flags,
     thread_support;
 
-  PolicyDomain
-    domain;
-
-  PolicyRights
-    rights;
-
   /*
     Determine image type from filename prefix or suffix (e.g. image.jpg).
   */
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image_info->filename != (char *) NULL);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
@@ -404,16 +464,6 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
   (void) SetImageInfo(read_info,0,exception);
   (void) CopyMagickString(filename,read_info->filename,MaxTextExtent);
   (void) CopyMagickString(magick,read_info->magick,MaxTextExtent);
-  domain=CoderPolicyDomain;
-  rights=ReadPolicyRights;
-  if (IsRightsAuthorized(domain,rights,read_info->magick) == MagickFalse)
-    {
-      errno=EPERM;
-      (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
-        "NotAuthorized","`%s'",read_info->filename);
-      read_info=DestroyImageInfo(read_info);
-      return((Image *) NULL);
-    }
   /*
     Call appropriate image reader based on image type.
   */
@@ -486,10 +536,16 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
   if ((magick_info != (const MagickInfo *) NULL) &&
       (GetImageDecoder(magick_info) != (DecodeImageHandler *) NULL))
     {
+      /*
+        Call appropriate image reader based on image type.
+      */
       thread_support=GetMagickThreadSupport(magick_info);
       if ((thread_support & DecoderThreadSupport) == 0)
         LockSemaphoreInfo(magick_info->semaphore);
-      image=GetImageDecoder(magick_info)(read_info,exception);
+      status=IsCoderAuthorized(read_info->magick,ReadPolicyRights,exception);
+      image=(Image *) NULL;
+      if (status != MagickFalse)
+        image=GetImageDecoder(magick_info)(read_info,exception);
       if ((thread_support & DecoderThreadSupport) == 0)
         UnlockSemaphoreInfo(magick_info->semaphore);
     }
@@ -545,10 +601,16 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
           read_info=DestroyImageInfo(read_info);
           return((Image *) NULL);
         }
+      /*
+        Call appropriate image reader based on image type.
+      */
       thread_support=GetMagickThreadSupport(magick_info);
       if ((thread_support & DecoderThreadSupport) == 0)
         LockSemaphoreInfo(magick_info->semaphore);
-      image=(Image *) (GetImageDecoder(magick_info))(read_info,exception);
+      status=IsCoderAuthorized(read_info->magick,ReadPolicyRights,exception);
+      image=(Image *) NULL;
+      if (status != MagickFalse)
+        image=(Image *) (GetImageDecoder(magick_info))(read_info,exception);
       if ((thread_support & DecoderThreadSupport) == 0)
         UnlockSemaphoreInfo(magick_info->semaphore);
     }
@@ -570,8 +632,10 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       "notify the developers",image->magick,exception->severity);
   if (IsBlobTemporary(image) != MagickFalse)
     (void) RelinquishUniqueFileResource(read_info->filename);
-  if ((GetNextImageInList(image) != (Image *) NULL) &&
-      (IsSceneGeometry(read_info->scenes,MagickFalse) != MagickFalse))
+  if ((IsSceneGeometry(read_info->scenes,MagickFalse) != MagickFalse) &&
+      ((GetNextImageInList(image) != (Image *) NULL) ||
+       ((read_info->scenes != (char *) NULL) &&
+        (strchr(read_info->scenes,',') != (char *) NULL))))
     {
       Image
         *clones;
@@ -599,6 +663,9 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     const StringInfo
       *profile;
 
+    ssize_t
+      option_type;
+
     next->taint=MagickFalse;
     GetPathComponent(magick_filename,MagickPath,magick_path);
     if (*magick_path == '\0' && *next->magick == '\0')
@@ -649,8 +716,10 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       value=GetImageProperty(next,"exif:ResolutionUnit");
     if (value != (char *) NULL)
       {
-        next->units=(ResolutionType) ParseCommandOption(MagickResolutionOptions,
-          MagickFalse,value);
+        option_type=ParseCommandOption(MagickResolutionOptions,MagickFalse,
+          value);
+        if (option_type >= 0)
+          next->units=(ResolutionType) option_type;
         (void) DeleteImageProperty(next,"exif:ResolutionUnit");
         (void) DeleteImageProperty(next,"tiff:ResolutionUnit");
       }
@@ -687,6 +756,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         RectangleInfo
           geometry;
 
+        SetGeometry(next,&geometry);
         flags=ParseAbsoluteGeometry(read_info->extract,&geometry);
         if ((next->columns != geometry.width) ||
             (next->rows != geometry.height))
@@ -731,11 +801,11 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         next->iptc_profile.length=GetStringInfoLength(profile);
         next->iptc_profile.info=GetStringInfoDatum(profile);
       }
-    (void) FormatMagickTime(GetBlobProperties(next)->st_mtime,MaxTextExtent,
-      timestamp);
+    (void) FormatMagickTime((time_t) GetBlobProperties(next)->st_mtime,
+      MaxTextExtent,timestamp);
     (void) SetImageProperty(next,"date:modify",timestamp);
-    (void) FormatMagickTime(GetBlobProperties(next)->st_ctime,MaxTextExtent,
-      timestamp);
+    (void) FormatMagickTime((time_t) GetBlobProperties(next)->st_ctime,
+      MaxTextExtent,timestamp);
     (void) SetImageProperty(next,"date:create",timestamp);
     option=GetImageOption(image_info,"delay");
     if (option != (const char *) NULL)
@@ -762,8 +832,12 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       }
     option=GetImageOption(image_info,"dispose");
     if (option != (const char *) NULL)
-      next->dispose=(DisposeType) ParseCommandOption(MagickDisposeOptions,
-        MagickFalse,option);
+      {
+        option_type=ParseCommandOption(MagickDisposeOptions,MagickFalse,
+          option);
+        if (option_type >= 0)
+          next->dispose=(DisposeType) option_type;
+      }
     if (read_info->verbose != MagickFalse)
       (void) IdentifyImage(next,stderr,MagickFalse);
     image=next;
@@ -813,7 +887,7 @@ MagickExport Image *ReadImages(const ImageInfo *image_info,
     Read image list from a file.
   */
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
@@ -921,7 +995,10 @@ MagickExport Image *ReadInlineImage(const ImageInfo *image_info,
   length=0;
   blob=Base64Decode(p,&length);
   if (length == 0)
-    ThrowReaderException(CorruptImageError,"CorruptImage");
+    {
+      blob=(unsigned char *) RelinquishMagickMemory(blob);
+      ThrowReaderException(CorruptImageError,"CorruptImage");
+    }
   read_info=CloneImageInfo(image_info);
   (void) SetImageInfoProgressMonitor(read_info,(MagickProgressMonitor) NULL,
     (void *) NULL);
@@ -990,22 +1067,16 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
   MagickStatusType
     thread_support;
 
-  PolicyDomain
-    domain;
-
-  PolicyRights
-    rights;
-
   /*
     Determine image type from filename prefix or suffix (e.g. image.jpg).
   */
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
+  assert(image != (Image *) NULL);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   exception=(&image->exception);
   sans_exception=AcquireExceptionInfo();
   write_info=CloneImageInfo(image_info);
@@ -1027,15 +1098,6 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
     }
   (void) CopyMagickString(filename,image->filename,MaxTextExtent);
   (void) CopyMagickString(image->filename,write_info->filename,MaxTextExtent);
-  domain=CoderPolicyDomain;
-  rights=WritePolicyRights;
-  if (IsRightsAuthorized(domain,rights,write_info->magick) == MagickFalse)
-    {
-      sans_exception=DestroyExceptionInfo(sans_exception);
-      write_info=DestroyImageInfo(write_info);
-      errno=EPERM;
-      ThrowBinaryException(PolicyError,"NotAuthorized",filename);
-    }
   /*
     Call appropriate image writer based on image type.
   */
@@ -1120,7 +1182,9 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
       thread_support=GetMagickThreadSupport(magick_info);
       if ((thread_support & EncoderThreadSupport) == 0)
         LockSemaphoreInfo(magick_info->semaphore);
-      status=GetImageEncoder(magick_info)(write_info,image);
+      status=IsCoderAuthorized(write_info->magick,WritePolicyRights,exception);
+      if (status != MagickFalse)
+        status=GetImageEncoder(magick_info)(write_info,image);
       if ((thread_support & EncoderThreadSupport) == 0)
         UnlockSemaphoreInfo(magick_info->semaphore);
     }
@@ -1190,7 +1254,10 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
               thread_support=GetMagickThreadSupport(magick_info);
               if ((thread_support & EncoderThreadSupport) == 0)
                 LockSemaphoreInfo(magick_info->semaphore);
-              status=GetImageEncoder(magick_info)(write_info,image);
+              status=IsCoderAuthorized(write_info->magick,WritePolicyRights,
+                exception);
+              if (status != MagickFalse)
+                status=GetImageEncoder(magick_info)(write_info,image);
               if ((thread_support & EncoderThreadSupport) == 0)
                 UnlockSemaphoreInfo(magick_info->semaphore);
             }
@@ -1285,9 +1352,9 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
     *p;
 
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(images != (Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   assert(exception != (ExceptionInfo *) NULL);
