@@ -17,13 +17,13 @@
 %                                July 1993                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -53,9 +53,9 @@
 #include "magick/utility.h"
 
 /*
-  Global declarations.
+  Define declarations.
 */
-#define MaxExceptions  128
+#define MaxExceptionList  64
 
 /*
   Forward declarations.
@@ -157,7 +157,7 @@ static void *DestroyExceptionElement(void *exception)
 MagickExport void ClearMagickException(ExceptionInfo *exception)
 {
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   if (exception->exceptions == (void *) NULL)
     return;
   LockSemaphoreInfo(exception->semaphore);
@@ -195,6 +195,9 @@ MagickExport void ClearMagickException(ExceptionInfo *exception)
 */
 MagickExport void CatchException(ExceptionInfo *exception)
 {
+  LinkedListInfo
+    *exceptions;
+
   register const ExceptionInfo
     *p;
 
@@ -202,31 +205,22 @@ MagickExport void CatchException(ExceptionInfo *exception)
     i;
 
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   if (exception->exceptions  == (void *) NULL)
     return;
   LockSemaphoreInfo(exception->semaphore);
-  ResetLinkedListIterator((LinkedListInfo *) exception->exceptions);
-  p=(const ExceptionInfo *) GetNextValueInLinkedList((LinkedListInfo *)
-    exception->exceptions);
+  exceptions=(LinkedListInfo *) exception->exceptions;
+  ResetLinkedListIterator(exceptions);
+  p=(const ExceptionInfo *) GetNextValueInLinkedList(exceptions);
   for (i=0; p != (const ExceptionInfo *) NULL; i++)
   {
     if (p->severity >= FatalErrorException)
       MagickFatalError(p->severity,p->reason,p->description);
-    if (i < MaxExceptions)
-      {
-        if ((p->severity >= ErrorException) && 
-            (p->severity < FatalErrorException))
-          MagickError(p->severity,p->reason,p->description);
-        if ((p->severity >= WarningException) && (p->severity < ErrorException))
-          MagickWarning(p->severity,p->reason,p->description);
-      }
-    else
-      if (i == MaxExceptions)
-        MagickError(ResourceLimitError,"too many exceptions",
-          "exception processing suspended");
-    p=(const ExceptionInfo *) GetNextValueInLinkedList((LinkedListInfo *)
-      exception->exceptions);
+    if ((p->severity >= ErrorException) && (p->severity < FatalErrorException))
+      MagickError(p->severity,p->reason,p->description);
+    if ((p->severity >= WarningException) && (p->severity < ErrorException))
+      MagickWarning(p->severity,p->reason,p->description);
+    p=(const ExceptionInfo *) GetNextValueInLinkedList(exceptions);
   }
   UnlockSemaphoreInfo(exception->semaphore);
   ClearMagickException(exception);
@@ -421,7 +415,7 @@ MagickPrivate MagickBooleanType ClearExceptionInfo(ExceptionInfo *exception,
   MagickBooleanType relinquish)
 {
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   if (exception->semaphore == (SemaphoreInfo *) NULL)
     ActivateSemaphoreInfo(&exception->semaphore);
   LockSemaphoreInfo(exception->semaphore);
@@ -430,7 +424,7 @@ MagickPrivate MagickBooleanType ClearExceptionInfo(ExceptionInfo *exception,
   exception->severity=UndefinedException;
   if (relinquish != MagickFalse)
     {
-      exception->signature=(~MagickSignature);
+      exception->signature=(~MagickCoreSignature);
       if (exception->exceptions != (void *) NULL)
         exception->exceptions=(void *) DestroyLinkedList((LinkedListInfo *)
           exception->exceptions,DestroyExceptionElement);
@@ -643,9 +637,9 @@ MagickExport void InheritException(ExceptionInfo *exception,
     *p;
 
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   assert(relative != (ExceptionInfo *) NULL);
-  assert(relative->signature == MagickSignature);
+  assert(relative->signature == MagickCoreSignature);
   assert(exception != relative);
   if (relative->exceptions == (void *) NULL)
     return;
@@ -687,11 +681,11 @@ MagickExport void InheritException(ExceptionInfo *exception,
 MagickPrivate void InitializeExceptionInfo(ExceptionInfo *exception)
 {
   assert(exception != (ExceptionInfo *) NULL);
-  (void) ResetMagickMemory(exception,0,sizeof(*exception));
+  (void) memset(exception,0,sizeof(*exception));
   exception->severity=UndefinedException;
   exception->exceptions=(void *) NewLinkedList(0);
   exception->semaphore=AllocateSemaphoreInfo();
-  exception->signature=MagickSignature;
+  exception->signature=MagickCoreSignature;
 }
 
 /*
@@ -932,14 +926,22 @@ MagickExport WarningHandler SetWarningHandler(WarningHandler handler)
 MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
   const ExceptionType severity,const char *reason,const char *description)
 {
+  LinkedListInfo
+    *exceptions;
+
   register ExceptionInfo
     *p;
 
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   LockSemaphoreInfo(exception->semaphore);
-  p=(ExceptionInfo *) GetLastValueInLinkedList((LinkedListInfo *)
-    exception->exceptions);
+  exceptions=(LinkedListInfo *) exception->exceptions;
+  if (GetNumberOfElementsInLinkedList(exceptions) > MaxExceptionList)
+    {
+      UnlockSemaphoreInfo(exception->semaphore);
+      return(MagickTrue);
+    }
+  p=(ExceptionInfo *) GetLastValueInLinkedList(exceptions);
   if ((p != (ExceptionInfo *) NULL) && (p->severity == severity) &&
       (LocaleCompare(exception->reason,reason) == 0) &&
       (LocaleCompare(exception->description,description) == 0))
@@ -953,21 +955,24 @@ MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
       UnlockSemaphoreInfo(exception->semaphore);
       ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
     }
-  (void) ResetMagickMemory(p,0,sizeof(*p));
+  (void) memset(p,0,sizeof(*p));
   p->severity=severity;
   if (reason != (const char *) NULL)
     p->reason=ConstantString(reason);
   if (description != (const char *) NULL)
     p->description=ConstantString(description);
-  p->signature=MagickSignature;
-  (void) AppendValueToLinkedList((LinkedListInfo *) exception->exceptions,p);
-  if (p->severity >= exception->severity)
+  p->signature=MagickCoreSignature;
+  (void) AppendValueToLinkedList(exceptions,p);
+  if (p->severity > exception->severity)
     {
       exception->severity=p->severity;
       exception->reason=p->reason;
       exception->description=p->description;
     }
   UnlockSemaphoreInfo(exception->semaphore);
+  if (GetNumberOfElementsInLinkedList(exceptions) == MaxExceptionList)
+    (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
+      "TooManyExceptions","(exception processing is suspended)");
   return(MagickTrue);
 }
 
@@ -1033,7 +1038,7 @@ MagickExport MagickBooleanType ThrowMagickExceptionList(
     length;
 
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   locale=GetLocaleExceptionMessage(severity,tag);
   (void) CopyMagickString(reason,locale,MaxTextExtent);
   (void) ConcatenateMagickString(reason," ",MaxTextExtent);
