@@ -18,13 +18,13 @@
 %                                August 2009                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -78,6 +78,9 @@ typedef struct _NodeInfo
   ColorPacket
     *list;
 
+  size_t
+    extent;
+
   MagickSizeType
     number_unique;
 
@@ -215,7 +218,7 @@ static CubeInfo *ClassifyImageColors(const Image *image,
     Initialize color description tree.
   */
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   cube_info=GetCubeInfo();
@@ -271,11 +274,18 @@ static CubeInfo *ClassifyImageColors(const Image *image,
       else
         {
           if (node_info->number_unique == 0)
-            node_info->list=(ColorPacket *) AcquireMagickMemory(
-              sizeof(*node_info->list));
+            {
+              node_info->extent=1;
+              node_info->list=(ColorPacket *) AcquireQuantumMemory(
+                node_info->extent,sizeof(*node_info->list));
+            }
           else
-            node_info->list=(ColorPacket *) ResizeQuantumMemory(node_info->list,
-              (size_t) (i+1),sizeof(*node_info->list));
+            if (i >= (ssize_t) node_info->extent)
+              {
+                node_info->extent<<=1;
+                node_info->list=(ColorPacket *) ResizeQuantumMemory(
+                  node_info->list,node_info->extent,sizeof(*node_info->list));
+              }
           if (node_info->list == (ColorPacket *) NULL)
             {
               (void) ThrowMagickException(exception,GetMagickModule(),
@@ -486,7 +496,7 @@ static CubeInfo *GetCubeInfo(void)
   cube_info=(CubeInfo *) AcquireMagickMemory(sizeof(*cube_info));
   if (cube_info == (CubeInfo *) NULL)
     return((CubeInfo *) NULL);
-  (void) ResetMagickMemory(cube_info,0,sizeof(*cube_info));
+  (void) memset(cube_info,0,sizeof(*cube_info));
   /*
     Initialize root node.
   */
@@ -604,7 +614,7 @@ static NodeInfo *GetNodeInfo(CubeInfo *cube_info,const size_t level)
     }
   cube_info->free_nodes--;
   node_info=cube_info->node_info++;
-  (void) ResetMagickMemory(node_info,0,sizeof(*node_info));
+  (void) memset(node_info,0,sizeof(*node_info));
   node_info->level=level;
   return(node_info);
 }
@@ -771,7 +781,7 @@ MagickExport MagickBooleanType IdentifyPaletteImage(const Image *image,
   ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   return(CheckImageColors(image,exception,256));
@@ -842,7 +852,7 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
     y;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->storage_class == PseudoClass) &&
@@ -973,7 +983,7 @@ MagickExport MagickBooleanType IsPaletteImage(const Image *image,
   ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->storage_class != PseudoClass)
