@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
 
-    http://www.imagemagick.org/script/license.php
+    https://imagemagick.org/script/license.php
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -22,14 +22,6 @@
 extern "C" {
 #endif
 
-#if defined(MAGICKCORE_HAVE_ISNAN)
-#  define IsNaN(a) isnan(a)
-#elif defined(_MSC_VER) && (_MSC_VER >= 1310)
-#  include <float.h>
-#  define IsNaN(a) _isnan(a)
-#else
-#  define IsNaN(a) (a != a)
-#endif
 #define MagickAbsoluteValue(x)  ((x) < 0 ? -(x) : (x))
 #define MagickMax(x,y)  (((x) > (y)) ? (x) : (y))
 #define MagickMin(x,y)  (((x) < (y)) ? (x) : (y))
