@@ -17,13 +17,13 @@
 %                               December 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -45,6 +45,7 @@
 #include "magick/blob-private.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/string_.h"
@@ -83,7 +84,7 @@ MagickExport void AppendImageToList(Image **images,const Image *append)
   assert(images != (Image **) NULL);
   if (append == (Image *) NULL)
     return;
-  assert(append->signature == MagickSignature);
+  assert(append->signature == MagickCoreSignature);
   if (append->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",append->filename);
   if ((*images) == (Image *) NULL)
@@ -91,7 +92,7 @@ MagickExport void AppendImageToList(Image **images,const Image *append)
       *images=(Image *) append;
       return;
     }
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
   p=GetLastImageInList(*images);
   q=GetFirstImageInList(append);
   p->next=q;
@@ -133,12 +134,16 @@ MagickExport Image *CloneImageList(const Image *images,ExceptionInfo *exception)
 
   if (images == (Image *) NULL)
     return((Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   while (images->previous != (Image *) NULL)
+  {
+    assert(images != images->previous);
     images=images->previous;
+  }
   image=(Image *) NULL;
   for (p=(Image *) NULL; images != (Image *) NULL; images=images->next)
   {
+    assert(images != images->next);
     clone=CloneImage(images,0,0,MagickTrue,exception);
     if (clone == (Image *) NULL)
       {
@@ -175,8 +180,8 @@ MagickExport Image *CloneImageList(const Image *images,ExceptionInfo *exception)
 %
 %  The numbers start at 0 for the first image in the list, while negative
 %  numbers refer to images starting counting from the end of the range. Images
-%  may be refered to multiple times to clone them multiple times. Images
-%  refered beyond the available number of images in list are ignored.
+%  may be referred to multiple times to clone them multiple times. Images
+%  referred beyond the available number of images in list are ignored.
 %
 %  Images referenced may be reversed, and results in a clone of those images
 %  also being made with a reversed order.
@@ -209,24 +214,24 @@ MagickExport Image *CloneImages(const Image *images,const char *scenes,
     *clone_images,
     *image;
 
-  long
-    first,
-    last,
-    step;
-
   register ssize_t
     i;
 
   size_t
     length;
 
+  ssize_t
+    first,
+    last,
+    step;
+
   assert(images != (const Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   assert(scenes != (char *) NULL);
   if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   clone_images=NewImageList();
   images=GetFirstImageInList(images);
   length=GetImageListLength(images);
@@ -234,19 +239,28 @@ MagickExport Image *CloneImages(const Image *images,const char *scenes,
   {
     while ((isspace((int) ((unsigned char) *p)) != 0) || (*p == ','))
       p++;
-    first=strtol(p,&p,10);
+    first=(ssize_t) strtol(p,&p,10);
     if (first < 0)
-      first+=(long) length;
+      first+=(ssize_t) length;
+    else
+      if (first > (ssize_t) length)
+        first=(ssize_t) length;
     last=first;
     while (isspace((int) ((unsigned char) *p)) != 0)
       p++;
     if (*p == '-')
       {
-        last=strtol(p+1,&p,10);
+        last=(ssize_t) strtol(p+1,&p,10);
         if (last < 0)
-          last+=(long) length;
+          last+=(ssize_t) length;
+        else
+          if (last > (ssize_t) length)
+            last=(ssize_t) length;
       }
-    for (step=first > last ? -1 : 1; first != (last+step); first+=step)
+    first=MagickMin(MagickMax(first,0),(ssize_t) length);
+    last=MagickMin(MagickMax(last,0),(ssize_t) length);
+    step=(ssize_t) (first > last ? -1 : 1);
+    for ( ; first != (last+step); first+=step)
     {
       i=0;
       for (next=images; next != (Image *) NULL; next=GetNextImageInList(next))
@@ -313,8 +327,8 @@ MagickExport void DeleteImageFromList(Image **images)
 %  comma separated list of image numbers or ranges.
 %
 %  The numbers start at 0 for the first image, while negative numbers refer to
-%  images starting counting from the end of the range. Images may be refered to
-%  multiple times without problems. Image refered beyond the available number
+%  images starting counting from the end of the range. Images may be referred to
+%  multiple times without problems. Image referred beyond the available number
 %  of images in list are ignored.
 %
 %  If the referenced images are in the reverse order, that range will be
@@ -357,13 +371,13 @@ MagickExport void DeleteImages(Image **images,const char *scenes,
     length;
 
   assert(images != (Image **) NULL);
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
   assert(scenes != (char *) NULL);
   if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       (*images)->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   *images=GetFirstImageInList(*images);
   length=GetImageListLength(*images);
   delete_list=(MagickBooleanType *) AcquireQuantumMemory(length,
@@ -443,7 +457,7 @@ MagickExport Image *DestroyImageList(Image *images)
 {
   if (images == (Image *) NULL)
     return((Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   while (images != (Image *) NULL)
@@ -466,8 +480,8 @@ MagickExport Image *DestroyImageList(Image *images)
 %  using a count and a comma separated list of image numbers or ranges.
 %
 %  The numbers start at 0 for the first image, while negative numbers refer to
-%  images starting counting from the end of the range. Images may be refered to
-%  multiple times without problems. Image refered beyond the available number
+%  images starting counting from the end of the range. Images may be referred to
+%  multiple times without problems. Image referred beyond the available number
 %  of images in list are ignored.
 %
 %  The format of the DuplicateImages method is:
@@ -501,12 +515,12 @@ MagickExport Image *DuplicateImages(Image *images,
     Duplicate images.
   */
   assert(images != (Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   assert(scenes != (char *) NULL);
   if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   duplicate_images=NewImageList();
   for (i=0; i < (ssize_t) number_duplicates; i++)
   {
@@ -545,7 +559,7 @@ MagickExport Image *GetFirstImageInList(const Image *images)
 
   if (images == (Image *) NULL)
     return((Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   for (p=images; p->previous != (Image *) NULL; p=p->previous) ;
   return((Image *) p);
 }
@@ -592,21 +606,13 @@ MagickExport Image *GetImageFromList(const Image *images,const ssize_t index)
 
   if (images == (Image *) NULL)
     return((Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
-
-  /*
-    Designed to efficiently find first image (index == 0), or last image
-    (index == -1) as appropriate, without to go through the whole image list,
-    unless the offset is outside of the list length range.
-
-    That is it tries to avoid 'counting the whole list' to handle the index.
-  */
-  if ( index < 0 )
+  if (index < 0)
     {
       p=GetLastImageInList(images);
-      for (i=-1; p != (Image *) NULL; p=p->previous)
+      for (i=(-1); p != (Image *) NULL; p=p->previous)
         if (i-- == index)
           break;
     }
@@ -649,9 +655,12 @@ MagickExport ssize_t GetImageIndexInList(const Image *images)
 
   if (images == (const Image *) NULL)
     return(-1);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   for (i=0; images->previous != (Image *) NULL; i++)
+  {
+    assert(images != images->previous);
     images=images->previous;
+  }
   return(i);
 }
 
@@ -685,12 +694,15 @@ MagickExport size_t GetImageListLength(const Image *images)
 
   if (images == (Image *) NULL)
     return(0);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   images=GetLastImageInList(images);
   for (i=0; images != (Image *) NULL; images=images->previous)
+  {
+    assert(images != images->previous);
     i++;
+  }
   return((size_t) i);
 }
 
@@ -723,7 +735,7 @@ MagickExport Image *GetLastImageInList(const Image *images)
 
   if (images == (Image *) NULL)
     return((Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   for (p=images; p->next != (Image *) NULL; p=p->next) ;
   return((Image *) p);
 }
@@ -754,7 +766,7 @@ MagickExport Image *GetNextImageInList(const Image *images)
 {
   if (images == (Image *) NULL)
     return((Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   return(images->next);
@@ -786,7 +798,7 @@ MagickExport Image *GetPreviousImageInList(const Image *images)
 {
   if (images == (Image *) NULL)
     return((Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   return(images->previous);
 }
 
@@ -834,7 +846,7 @@ MagickExport Image **ImageListToArray(const Image *images,
 
   if (images == (Image *) NULL)
     return((Image **) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   group=(Image **) AcquireQuantumMemory((size_t) GetImageListLength(images)+1UL,
@@ -847,7 +859,10 @@ MagickExport Image **ImageListToArray(const Image *images,
     }
   images=GetFirstImageInList(images);
   for (i=0; images != (Image *) NULL; images=images->next)
+  {
+    assert(images != images->next);
     group[i++]=(Image *) images;
+  }
   group[i]=(Image *) NULL;
   return(group);
 }
@@ -885,12 +900,12 @@ MagickExport void InsertImageInList(Image **images,Image *insert)
 
   assert(images != (Image **) NULL);
   assert(insert != (Image *) NULL);
-  assert(insert->signature == MagickSignature);
+  assert(insert->signature == MagickCoreSignature);
   if (insert->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",insert->filename);
   if ((*images) == (Image *) NULL)
     return;
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
   split=SplitImageList(*images);
   AppendImageToList(images,insert);
   AppendImageToList(images,split);
@@ -982,7 +997,7 @@ MagickExport Image *RemoveImageFromList(Image **images)
   assert(images != (Image **) NULL);
   if ((*images) == (Image *) NULL)
     return((Image *) NULL);
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
   if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       (*images)->filename);
@@ -1041,7 +1056,7 @@ MagickExport Image *RemoveFirstImageFromList(Image **images)
   assert(images != (Image **) NULL);
   if ((*images) == (Image *) NULL)
     return((Image *) NULL);
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
   if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       (*images)->filename);
@@ -1092,7 +1107,7 @@ MagickExport Image *RemoveLastImageFromList(Image **images)
   assert(images != (Image **) NULL);
   if ((*images) == (Image *) NULL)
     return((Image *) NULL);
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
   if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       (*images)->filename);
@@ -1141,12 +1156,12 @@ MagickExport void ReplaceImageInList(Image **images,Image *replace)
 {
   assert(images != (Image **) NULL);
   assert(replace != (Image *) NULL);
-  assert(replace->signature == MagickSignature);
+  assert(replace->signature == MagickCoreSignature);
   if (replace->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",replace->filename);
   if ((*images) == (Image *) NULL)
     return;
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
 
   /* link next pointer */
   replace=GetLastImageInList(replace);
@@ -1199,12 +1214,12 @@ MagickExport void ReplaceImageInListReturnLast(Image **images,Image *replace)
 {
   assert(images != (Image **) NULL);
   assert(replace != (Image *) NULL);
-  assert(replace->signature == MagickSignature);
+  assert(replace->signature == MagickCoreSignature);
   if (replace->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",replace->filename);
   if ((*images) == (Image *) NULL)
     return;
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
 
   /* link previous pointer */
   replace=GetFirstImageInList(replace);
@@ -1257,7 +1272,7 @@ MagickExport void ReverseImageList(Image **images)
   assert(images != (Image **) NULL);
   if ((*images) == (Image *) NULL)
     return;
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
   if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       (*images)->filename);
@@ -1311,10 +1326,10 @@ MagickExport Image *SpliceImageIntoList(Image **images,
 
   assert(images != (Image **) NULL);
   assert(splice != (Image *) NULL);
-  assert(splice->signature == MagickSignature);
+  assert(splice->signature == MagickCoreSignature);
   if ((*images) == (Image *) NULL)
     return((Image *) NULL);
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
   if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       (*images)->filename);
@@ -1390,7 +1405,7 @@ MagickExport void SyncImageList(Image *images)
 
   if (images == (Image *) NULL)
     return;
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   for (p=images; p != (Image *) NULL; p=p->next)
   {
     for (q=p->next; q != (Image *) NULL; q=q->next)
@@ -1432,7 +1447,7 @@ MagickExport Image *SyncNextImageInList(const Image *images)
 {
   if (images == (Image *) NULL)
     return((Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   if (images->next == (Image *) NULL)
     return((Image *) NULL);
   if (images->blob != images->next->blob)
@@ -1440,7 +1455,9 @@ MagickExport Image *SyncNextImageInList(const Image *images)
       DestroyBlob(images->next);
       images->next->blob=ReferenceBlob(images->blob);
     }
-  images->next->compression=images->compression;
-  images->next->endian=images->endian;
+  if (images->next->compression == UndefinedCompression)
+    images->next->compression=images->compression;
+  if (images->next->endian == UndefinedEndian)
+    images->next->endian=images->endian;
   return(images->next);
 }
