@@ -18,13 +18,13 @@
 %                             November 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -72,6 +72,7 @@
 #include "magick/semaphore-private.h"
 #include "magick/signature-private.h"
 #include "magick/splay-tree.h"
+#include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/string-private.h"
 #include "magick/thread_.h"
@@ -116,7 +117,8 @@ static SplayTreeInfo
 
 static volatile MagickBooleanType
   instantiate_magickcore = MagickFalse,
-  magickcore_signal_in_progress = MagickFalse;
+  magickcore_signal_in_progress = MagickFalse,
+  magick_list_initialized = MagickFalse;
 
 /*
   Forward declarations.
@@ -150,7 +152,7 @@ MagickExport DecodeImageHandler *GetImageDecoder(const MagickInfo *magick_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
-  assert(magick_info->signature == MagickSignature);
+  assert(magick_info->signature == MagickCoreSignature);
   return(magick_info->decoder);
 }
 
@@ -180,7 +182,7 @@ MagickExport EncodeImageHandler *GetImageEncoder(const MagickInfo *magick_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
-  assert(magick_info->signature == MagickSignature);
+  assert(magick_info->signature == MagickCoreSignature);
   return(magick_info->encoder);
 }
 
@@ -277,7 +279,7 @@ MagickExport MagickBooleanType GetMagickAdjoin(const MagickInfo *magick_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
-  assert(magick_info->signature == MagickSignature);
+  assert(magick_info->signature == MagickCoreSignature);
   return(magick_info->adjoin);
 }
 
@@ -308,7 +310,7 @@ MagickExport MagickBooleanType GetMagickBlobSupport(
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
-  assert(magick_info->signature == MagickSignature);
+  assert(magick_info->signature == MagickCoreSignature);
   return(magick_info->blob_support);
 }
 
@@ -338,7 +340,7 @@ MagickExport const char *GetMagickDescription(const MagickInfo *magick_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
-  assert(magick_info->signature == MagickSignature);
+  assert(magick_info->signature == MagickCoreSignature);
   return(magick_info->description);
 }
 
@@ -370,7 +372,7 @@ MagickExport MagickBooleanType GetMagickEndianSupport(
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
-  assert(magick_info->signature == MagickSignature);
+  assert(magick_info->signature == MagickCoreSignature);
   return(magick_info->endian_support);
 }
 
@@ -404,51 +406,42 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
   ExceptionInfo *exception)
 {
   register const MagickInfo
-    *p;
+    *magick_info;
 
+  /*
+    Find named module attributes.
+  */
   assert(exception != (ExceptionInfo *) NULL);
   if (IsMagickTreeInstantiated(exception) == MagickFalse)
     return((const MagickInfo *) NULL);
-#if defined(MAGICKCORE_MODULES_SUPPORT)
-  if ((name != (const char *) NULL) && (LocaleCompare(name,"*") == 0))
-    (void) OpenModules(exception);
-#endif
-  /*
-    Find name in list.
-  */
-  LockSemaphoreInfo(magick_semaphore);
-  ResetSplayTreeIterator(magick_list);
-  p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
-  if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
+  magick_info=(const MagickInfo *) NULL;
+  if ((name != (const char *) NULL) && (*name != '\0'))
     {
-      ResetSplayTreeIterator(magick_list);
-      p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
+      LockSemaphoreInfo(magick_semaphore);
+      if (LocaleCompare(name,"*") == 0)
+#if defined(MAGICKCORE_BUILD_MODULES)
+        (void) OpenModules(exception);
+#else
+        RegisterStaticModules();
+#endif
+      else
+        {
+          magick_info=(const MagickInfo *) GetValueFromSplayTree(magick_list,
+            name);
+          if (magick_info == (const MagickInfo *) NULL)
+#if defined(MAGICKCORE_BUILD_MODULES)
+            (void) OpenModule(name,exception);
+#else
+            (void) RegisterStaticModule(name,exception);
+#endif
+        }
       UnlockSemaphoreInfo(magick_semaphore);
-      return(p);
     }
-  while (p != (const MagickInfo *) NULL)
-  {
-    if (LocaleCompare(p->name,name) == 0)
-      break;
-    p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
-  }
-#if defined(MAGICKCORE_MODULES_SUPPORT)
-  if (p == (const MagickInfo *) NULL)
-    {
-      if (*name != '\0')
-        (void) OpenModule(name,exception);
-      ResetSplayTreeIterator(magick_list);
-      p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
-      while (p != (const MagickInfo *) NULL)
-      {
-        if (LocaleCompare(p->name,name) == 0)
-          break;
-        p=(const MagickInfo *) GetNextValueInSplayTree(magick_list);
-      }
-    }
-#endif
-  UnlockSemaphoreInfo(magick_semaphore);
-  return(p);
+  if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
+    return((const MagickInfo *) GetRootValueFromSplayTree(magick_list));
+  if (magick_info == (const MagickInfo *) NULL)
+    magick_info=(const MagickInfo *) GetValueFromSplayTree(magick_list,name);
+  return(magick_info);
 }
 
 /*
@@ -661,7 +654,7 @@ MagickExport const char *GetMagickMimeType(const MagickInfo *magick_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
-  assert(magick_info->signature == MagickSignature);
+  assert(magick_info->signature == MagickCoreSignature);
   return(magick_info->mime_type);
 }
 
@@ -717,7 +710,7 @@ MagickExport MagickBooleanType GetMagickRawSupport(
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
-  assert(magick_info->signature == MagickSignature);
+  assert(magick_info->signature == MagickCoreSignature);
   return(magick_info->raw);
 }
 
@@ -749,7 +742,7 @@ MagickExport MagickBooleanType GetMagickSeekableStream(
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
-  assert(magick_info->signature == MagickSignature);
+  assert(magick_info->signature == MagickCoreSignature);
   return(magick_info->seekable_stream);
 }
 
@@ -780,7 +773,7 @@ MagickExport MagickStatusType GetMagickThreadSupport(
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(magick_info != (MagickInfo *) NULL);
-  assert(magick_info->signature == MagickSignature);
+  assert(magick_info->signature == MagickCoreSignature);
   return(magick_info->thread_support);
 }
 
@@ -833,13 +826,12 @@ static void *DestroyMagickNode(void *magick_info)
 
 static MagickBooleanType IsMagickTreeInstantiated(ExceptionInfo *exception)
 {
-  (void) exception;
-  if (magick_list == (SplayTreeInfo *) NULL)
+  if (magick_list_initialized == MagickFalse)
     {
       if (magick_semaphore == (SemaphoreInfo *) NULL)
         ActivateSemaphoreInfo(&magick_semaphore);
       LockSemaphoreInfo(magick_semaphore);
-      if (magick_list == (SplayTreeInfo *) NULL)
+      if (magick_list_initialized == MagickFalse)
         {
           MagickBooleanType
             status;
@@ -861,9 +853,7 @@ static MagickBooleanType IsMagickTreeInstantiated(ExceptionInfo *exception)
 #if defined(MAGICKCORE_MODULES_SUPPORT)
           (void) GetModuleInfo((char *) NULL,exception);
 #endif
-#if !defined(MAGICKCORE_BUILD_MODULES)
-          RegisterStaticModules();
-#endif
+          magick_list_initialized=MagickTrue;
         }
       UnlockSemaphoreInfo(magick_semaphore);
     }
@@ -1089,7 +1079,10 @@ MagickExport void MagickComponentTerminus(void)
     ActivateSemaphoreInfo(&magick_semaphore);
   LockSemaphoreInfo(magick_semaphore);
   if (magick_list != (SplayTreeInfo *) NULL)
-    magick_list=DestroySplayTree(magick_list);
+    {
+      magick_list=DestroySplayTree(magick_list);
+      magick_list_initialized=MagickFalse;
+    }
   UnlockSemaphoreInfo(magick_semaphore);
   DestroySemaphoreInfo(&magick_semaphore);
 }
@@ -1143,6 +1136,9 @@ static SignalHandler *SetMagickSignalHandler(int signal_number,
   action.sa_flags=0;
 #if defined(SA_INTERRUPT)
   action.sa_flags|=SA_INTERRUPT;
+#endif
+#if defined(SA_ONSTACK)
+  action.sa_flags|=SA_ONSTACK;
 #endif
   status=sigaction(signal_number,&action,&previous_action);
   if (status < 0)
@@ -1168,6 +1164,10 @@ static void MagickSignalHandler(int signal_number)
   if (signal_number == SIGABRT)
     abort();
 #endif
+#if defined(SIGBUS)
+  if (signal_number == SIGBUS)
+    abort();
+#endif
 #if defined(SIGFPE)
   if (signal_number == SIGFPE)
     abort();
@@ -1279,6 +1279,10 @@ MagickExport void MagickCoreGenesis(const char *path,
       if (signal_handlers[SIGABRT] == (SignalHandler *) NULL)
         signal_handlers[SIGABRT]=RegisterMagickSignalHandler(SIGABRT);
 #endif
+#if defined(SIGBUS)
+      if (signal_handlers[SIGBUS] == (SignalHandler *) NULL)
+        signal_handlers[SIGBUS]=RegisterMagickSignalHandler(SIGBUS);
+#endif
 #if defined(SIGSEGV)
       if (signal_handlers[SIGSEGV] == (SignalHandler *) NULL)
         signal_handlers[SIGSEGV]=RegisterMagickSignalHandler(SIGSEGV);
@@ -1389,6 +1393,7 @@ MagickExport void MagickCoreTerminus(void)
   ModuleComponentTerminus();
 #endif
   CoderComponentTerminus();
+  AsynchronousResourceComponentTerminus();
   ResourceComponentTerminus();
   CacheComponentTerminus();
   PolicyComponentTerminus();
@@ -1436,7 +1441,7 @@ MagickExport MagickInfo *RegisterMagickInfo(MagickInfo *magick_info)
     Register a new image format.
   */
   assert(magick_info != (MagickInfo *) NULL);
-  assert(magick_info->signature == MagickSignature);
+  assert(magick_info->signature == MagickCoreSignature);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",magick_info->name);
   if (magick_list == (SplayTreeInfo *) NULL)
     return((MagickInfo *) NULL);
@@ -1486,13 +1491,13 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
   magick_info=(MagickInfo *) AcquireMagickMemory(sizeof(*magick_info));
   if (magick_info == (MagickInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(magick_info,0,sizeof(*magick_info));
+  (void) memset(magick_info,0,sizeof(*magick_info));
   magick_info->name=ConstantString(name);
   magick_info->adjoin=MagickTrue;
   magick_info->blob_support=MagickTrue;
   magick_info->thread_support=(MagickStatusType) (DecoderThreadSupport |
     EncoderThreadSupport);
-  magick_info->signature=MagickSignature;
+  magick_info->signature=MagickCoreSignature;
   return(magick_info);
 }
 
@@ -1546,7 +1551,7 @@ MagickExport int SetMagickPrecision(const int precision)
       magick_precision=MagickPrecision;
       limit=GetEnvironmentValue("MAGICK_PRECISION");
       if (limit == (char *) NULL)
-        limit=GetPolicyValue("precision");
+        limit=GetPolicyValue("system:precision");
       if (limit != (char *) NULL)
         {
           magick_precision=StringToInteger(limit);
