@@ -15,13 +15,13 @@
 %                                 July 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/MagicksToolkit/script/license.php             %
+%    http://imagemagick.org/MagicksToolkit/script/license.php             %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -611,7 +611,7 @@ MagickExport const char *GetMimeDescription(const MimeInfo *mime_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(mime_info != (MimeInfo *) NULL);
-  assert(mime_info->signature == MagickSignature);
+  assert(mime_info->signature == MagickCoreSignature);
   return(mime_info->description);
 }
 
@@ -641,7 +641,7 @@ MagickExport const char *GetMimeType(const MimeInfo *mime_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(mime_info != (MimeInfo *) NULL);
-  assert(mime_info->signature == MagickSignature);
+  assert(mime_info->signature == MagickCoreSignature);
   return(mime_info->type);
 }
 
@@ -833,7 +833,7 @@ static MagickBooleanType LoadMimeCache(LinkedListInfo *cache,const char *xml,
     attribute=GetXMLTreeAttribute(include,"file");
     if (attribute != (const char *) NULL)
       {
-        if (depth > 200)
+        if (depth > MagickMaxRecursionDepth)
           (void) ThrowMagickException(exception,GetMagickModule(),
             ConfigureError,"IncludeElementNestedTooDeeply","`%s'",filename);
         else
@@ -872,9 +872,9 @@ static MagickBooleanType LoadMimeCache(LinkedListInfo *cache,const char *xml,
     mime_info=(MimeInfo *) AcquireMagickMemory(sizeof(*mime_info));
     if (mime_info == (MimeInfo *) NULL)
       ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-    (void) ResetMagickMemory(mime_info,0,sizeof(*mime_info));
+    (void) memset(mime_info,0,sizeof(*mime_info));
     mime_info->path=ConstantString(filename);
-    mime_info->signature=MagickSignature;
+    mime_info->signature=MagickCoreSignature;
     attribute=GetXMLTreeAttribute(mime,"data-type");
     if (attribute != (const char *) NULL)
       mime_info->data_type=(DataType) ParseCommandOption(MagickDataTypeOptions,
