@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -162,10 +162,10 @@ MagickExport MontageInfo *CloneMontageInfo(const ImageInfo *image_info,
 */
 MagickExport MontageInfo *DestroyMontageInfo(MontageInfo *montage_info)
 {
+  assert(montage_info != (MontageInfo *) NULL);
   if (montage_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  assert(montage_info != (MontageInfo *) NULL);
-  assert(montage_info->signature == MagickSignature);
+  assert(montage_info->signature == MagickCoreSignature);
   if (montage_info->geometry != (char *) NULL)
     montage_info->geometry=(char *)
       RelinquishMagickMemory(montage_info->geometry);
@@ -180,7 +180,7 @@ MagickExport MontageInfo *DestroyMontageInfo(MontageInfo *montage_info)
       montage_info->texture);
   if (montage_info->font != (char *) NULL)
     montage_info->font=DestroyString(montage_info->font);
-  montage_info->signature=(~MagickSignature);
+  montage_info->signature=(~MagickCoreSignature);
   montage_info=(MontageInfo *) RelinquishMagickMemory(montage_info);
   return(montage_info);
 }
@@ -214,12 +214,12 @@ MagickExport void GetMontageInfo(const ImageInfo *image_info,
   MontageInfo *montage_info)
 {
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(montage_info != (MontageInfo *) NULL);
-  (void) ResetMagickMemory(montage_info,0,sizeof(*montage_info));
+  (void) memset(montage_info,0,sizeof(*montage_info));
   (void) CopyMagickString(montage_info->filename,image_info->filename,
     MaxTextExtent);
   montage_info->geometry=AcquireString(DefaultTileGeometry);
@@ -233,7 +233,7 @@ MagickExport void GetMontageInfo(const ImageInfo *image_info,
   montage_info->border_color=image_info->border_color;
   montage_info->matte_color=image_info->matte_color;
   montage_info->debug=IsEventLogging();
-  montage_info->signature=MagickSignature;
+  montage_info->signature=MagickCoreSignature;
 }
 
 /*
@@ -283,20 +283,6 @@ static void GetMontageGeometry(char *geometry,const size_t number_images,
     *tiles_per_row=(size_t) ceil((double) number_images/(*tiles_per_column));
 }
 
-static inline ssize_t MagickMax(const ssize_t x,const ssize_t y)
-{
-  if (x > y)
-    return(x);
-  return(y);
-}
-
-static inline ssize_t MagickMin(const ssize_t x,const ssize_t y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
@@ -384,9 +370,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     geometry,
     extract_info;
 
-
   size_t
-    bevel_width,
     border_width,
     extent,
     height,
@@ -403,6 +387,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     width;
 
   ssize_t
+    bevel_width,
     tile,
     x,
     x_offset,
@@ -416,19 +401,19 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     Create image tiles.
   */
   assert(images != (Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   assert(montage_info != (MontageInfo *) NULL);
-  assert(montage_info->signature == MagickSignature);
+  assert(montage_info->signature == MagickCoreSignature);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   number_images=GetImageListLength(images);
   master_list=ImageListToArray(images,exception);
+  if (master_list == (Image **) NULL)
+    return((Image *) NULL);
   image_list=master_list;
   image=image_list[0];
-  if (master_list == (Image **) NULL)
-    ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
   thumbnail=NewImageList();
   for (i=0; i < (ssize_t) number_images; i++)
   {
@@ -438,7 +423,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     (void) ParseAbsoluteGeometry("0x0+0+0",&image->page);
     progress_monitor=SetImageProgressMonitor(image,(MagickProgressMonitor) NULL,
       image->client_data);
-    flags=ParseRegionGeometry(image,montage_info->geometry,&geometry,exception);
+    (void) ParseRegionGeometry(image,montage_info->geometry,&geometry,exception);
     thumbnail=ThumbnailImage(image,geometry.width,geometry.height,exception);
     if (thumbnail == (Image *) NULL)
       break;
@@ -452,6 +437,8 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
   }
   if (i < (ssize_t) number_images)
     {
+      if (image != (Image *) NULL)
+        image=DestroyImage(image);
       if (thumbnail == (Image *) NULL)
         i--;
       for (tile=0; (ssize_t) tile <= i; tile++)
@@ -498,7 +485,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     }
   border_width=montage_info->border_width;
   bevel_width=0;
-  (void) ResetMagickMemory(&frame_info,0,sizeof(frame_info));
+  (void) memset(&frame_info,0,sizeof(frame_info));
   if (montage_info->frame != (char *) NULL)
     {
       char
@@ -513,7 +500,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
       if ((flags & HeightValue) == 0)
         frame_info.height=frame_info.width;
       if ((flags & XiValue) == 0)
-        frame_info.outer_bevel=(ssize_t) frame_info.width/2;
+        frame_info.outer_bevel=(ssize_t) frame_info.width/2-1;
       if ((flags & PsiValue) == 0)
         frame_info.inner_bevel=frame_info.outer_bevel;
       frame_info.x=(ssize_t) frame_info.width;
@@ -650,7 +637,14 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
       sizeof(*montage->directory));
     if ((montage->montage == (char *) NULL) ||
         (montage->directory == (char *) NULL))
-      ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
+      {
+        if (montage->montage != (char *) NULL)
+          montage->montage=(char *) RelinquishMagickMemory(montage->montage);
+        if (montage->directory != (char *) NULL)
+          montage->directory=(char *) RelinquishMagickMemory(
+            montage->directory);
+        ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
+      }
     x_offset=0;
     y_offset=0;
     if (montage_info->tile != (char *) NULL)
@@ -669,7 +663,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     {
       (void) ConcatenateMagickString(montage->directory,
         image_list[tile]->filename,extent);
-      (void) ConcatenateMagickString(montage->directory,"\n",extent);
+      (void) ConcatenateMagickString(montage->directory,"\xff",extent);
       tile++;
     }
     progress_monitor=SetImageProgressMonitor(montage,(MagickProgressMonitor)
@@ -722,6 +716,8 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
         Copy this tile to the composite.
       */
       image=CloneImage(image_list[tile],0,0,MagickTrue,exception);
+      if (image == (Image *) NULL)
+        ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
       progress_monitor=SetImageProgressMonitor(image,
         (MagickProgressMonitor) NULL,image->client_data);
       width=concatenate != MagickFalse ? image->columns : extract_info.width;
@@ -766,10 +762,10 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
         tile_image->gravity=image->gravity;
       (void) FormatLocaleString(tile_geometry,MaxTextExtent,"%.20gx%.20g+0+0",
         (double) image->columns,(double) image->rows);
-      flags=ParseGravityGeometry(tile_image,tile_geometry,&geometry,exception);
+      (void) ParseGravityGeometry(tile_image,tile_geometry,&geometry,exception);
       x=(ssize_t) (geometry.x+border_width);
       y=(ssize_t) (geometry.y+border_width);
-      if ((montage_info->frame != (char *) NULL) && (bevel_width != 0))
+      if ((montage_info->frame != (char *) NULL) && (bevel_width > 0))
         {
           FrameInfo
             extract_info;
