@@ -18,13 +18,13 @@
 %                                December 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -69,162 +69,6 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   C r o p I m a g e T o H B i t m a p                                       %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  CropImageToHBITMAP() extracts a specified region of the image and returns
-%  it as a Windows HBITMAP. While the same functionality can be accomplished by
-%  invoking CropImage() followed by ImageToHBITMAP(), this method is more
-%  efficient since it copies pixels directly to the HBITMAP.
-%
-%  The format of the CropImageToHBITMAP method is:
-%
-%      HBITMAP CropImageToHBITMAP(Image* image,const RectangleInfo *geometry,
-%        ExceptionInfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o image: the image.
-%
-%    o geometry: Define the region of the image to crop with members
-%      x, y, width, and height.
-%
-%    o exception: return any errors or warnings in this structure.
-%
-*/
-MagickExport void *CropImageToHBITMAP(Image *image,
-  const RectangleInfo *geometry,ExceptionInfo *exception)
-{
-#define CropImageTag  "Crop/Image"
-
-  BITMAP
-    bitmap;
-
-  HBITMAP
-    bitmapH;
-
-  HANDLE
-    bitmap_bitsH;
-
-  MagickBooleanType
-    proceed;
-
-  RectangleInfo
-    page;
-
-  register const PixelPacket
-    *p;
-
-  register RGBQUAD
-    *q;
-
-  RGBQUAD
-    *bitmap_bits;
-
-  ssize_t
-    y;
-
-  /*
-    Check crop geometry.
-  */
-  assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  assert(geometry != (const RectangleInfo *) NULL);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  if (((geometry->x+(ssize_t) geometry->width) < 0) ||
-      ((geometry->y+(ssize_t) geometry->height) < 0) ||
-      (geometry->x >= (ssize_t) image->columns) ||
-      (geometry->y >= (ssize_t) image->rows))
-    ThrowImageException(OptionError,"GeometryDoesNotContainImage");
-  page=(*geometry);
-  if ((page.x+(ssize_t) page.width) > (ssize_t) image->columns)
-    page.width=image->columns-page.x;
-  if ((page.y+(ssize_t) page.height) > (ssize_t) image->rows)
-    page.height=image->rows-page.y;
-  if (page.x < 0)
-    {
-      page.width+=page.x;
-      page.x=0;
-    }
-  if (page.y < 0)
-    {
-      page.height+=page.y;
-      page.y=0;
-    }
-
-  if ((page.width == 0) || (page.height == 0))
-    ThrowImageException(OptionError,"GeometryDimensionsAreZero");
-  /*
-    Initialize crop image attributes.
-  */
-  bitmap.bmType         = 0;
-  bitmap.bmWidth        = (LONG) page.width;
-  bitmap.bmHeight       = (LONG) page.height;
-  bitmap.bmWidthBytes   = bitmap.bmWidth * 4;
-  bitmap.bmPlanes       = 1;
-  bitmap.bmBitsPixel    = 32;
-  bitmap.bmBits         = NULL;
-
-  bitmap_bitsH=(HANDLE) GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE,page.width*
-    page.height*bitmap.bmBitsPixel);
-  if (bitmap_bitsH == NULL)
-    return(NULL);
-  bitmap_bits=(RGBQUAD *) GlobalLock((HGLOBAL) bitmap_bitsH);
-  if ( bitmap.bmBits == NULL )
-    bitmap.bmBits = bitmap_bits;
-  if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-    SetImageColorspace(image,sRGBColorspace);
-  /*
-    Extract crop image.
-  */
-  q=bitmap_bits;
-  for (y=0; y < (ssize_t) page.height; y++)
-  {
-    register ssize_t
-      x;
-
-    p=GetVirtualPixels(image,page.x,page.y+y,page.width,1,exception);
-    if (p == (const PixelPacket *) NULL)
-      break;
-
-    /* Transfer pixels, scaling to Quantum */
-    for( x=(ssize_t) page.width ; x> 0 ; x-- )
-    {
-      q->rgbRed = ScaleQuantumToChar(GetPixelRed(p));
-      q->rgbGreen = ScaleQuantumToChar(GetPixelGreen(p));
-      q->rgbBlue = ScaleQuantumToChar(GetPixelBlue(p));
-      q->rgbReserved = 0;
-      p++;
-      q++;
-    }
-    proceed=SetImageProgress(image,CropImageTag,y,page.height);
-    if (proceed == MagickFalse)
-      break;
-  }
-  if (y < (ssize_t) page.height)
-    {
-      GlobalUnlock((HGLOBAL) bitmap_bitsH);
-      GlobalFree((HGLOBAL) bitmap_bitsH);
-      return((void *) NULL);
-    }
-  bitmap.bmBits=bitmap_bits;
-  bitmapH=CreateBitmapIndirect(&bitmap);
-  GlobalUnlock((HGLOBAL) bitmap_bitsH);
-  GlobalFree((HGLOBAL) bitmap_bitsH);
-  return((void *) bitmapH);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   I s M a g i c k C o n f l i c t                                           %
 %                                                                             %
 %                                                                             %
@@ -377,9 +221,9 @@ MagickExport MagickBooleanType NTAcquireTypeCache(SplayTreeInfo *type_cache,
         type_info=(TypeInfo *) AcquireMagickMemory(sizeof(*type_info));
         if (type_info == (TypeInfo *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-        (void) ResetMagickMemory(type_info,0,sizeof(TypeInfo));
+        (void) memset(type_info,0,sizeof(TypeInfo));
         type_info->path=ConstantString("Windows Fonts");
-        type_info->signature=MagickSignature;
+        type_info->signature=MagickCoreSignature;
         (void) CopyMagickString(buffer,value_name,MaxTextExtent);  /* name */
         for (pos=buffer; *pos != 0; pos++)
           if (*pos == ' ')
@@ -535,113 +379,4 @@ MagickExport MagickBooleanType NTAcquireTypeCache(SplayTreeInfo *type_cache,
   return(MagickTrue);
 }
 
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   I m a g e T o H B i t m a p                                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  ImageToHBITMAP() creates a Windows HBITMAP from an image.
-%
-%  The format of the ImageToHBITMAP method is:
-%
-%      HBITMAP ImageToHBITMAP(Image *image,Exceptioninfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o image: the image to convert.
-%
-*/
-MagickExport void *ImageToHBITMAP(Image *image,ExceptionInfo *exception)
-{
-  BITMAP
-    bitmap;
-
-  HANDLE
-    bitmap_bitsH;
-
-  HBITMAP
-    bitmapH;
-
-  register ssize_t
-    x;
-
-  register const PixelPacket
-    *p;
-
-  register RGBQUAD
-    *q;
-
-  RGBQUAD
-    *bitmap_bits;
-
-  size_t
-    length;
-
-  ssize_t
-    y;
-
-  (void) ResetMagickMemory(&bitmap,0,sizeof(bitmap));
-  bitmap.bmType=0;
-  bitmap.bmWidth=(LONG) image->columns;
-  bitmap.bmHeight=(LONG) image->rows;
-  bitmap.bmWidthBytes=4*bitmap.bmWidth;
-  bitmap.bmPlanes=1;
-  bitmap.bmBitsPixel=32;
-  bitmap.bmBits=NULL;
-  length=bitmap.bmWidthBytes*bitmap.bmHeight;
-  bitmap_bitsH=(HANDLE) GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE,length);
-  if (bitmap_bitsH == NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed","`%s'",message);
-      message=DestroyString(message);
-      return(NULL);
-    }
-  bitmap_bits=(RGBQUAD *) GlobalLock((HGLOBAL) bitmap_bitsH);
-  q=bitmap_bits;
-  if (bitmap.bmBits == NULL)
-    bitmap.bmBits=bitmap_bits;
-  (void) SetImageColorspace(image,sRGBColorspace);
-  for (y=0; y < (ssize_t) image->rows; y++)
-  {
-    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
-    if (p == (const PixelPacket *) NULL)
-      break;
-    for (x=0; x < (ssize_t) image->columns; x++)
-    {
-      q->rgbRed=ScaleQuantumToChar(GetPixelRed(p));
-      q->rgbGreen=ScaleQuantumToChar(GetPixelGreen(p));
-      q->rgbBlue=ScaleQuantumToChar(GetPixelBlue(p));
-      q->rgbReserved=0;
-      p++;
-      q++;
-    }
-  }
-  bitmap.bmBits=bitmap_bits;
-  bitmapH=CreateBitmapIndirect(&bitmap);
-  if (bitmapH == NULL)
-    {
-      char
-        *message;
-
-      message=GetExceptionMessage(errno);
-      (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed","`%s'",message);
-      message=DestroyString(message);
-    }
-  GlobalUnlock((HGLOBAL) bitmap_bitsH);
-  GlobalFree((HGLOBAL) bitmap_bitsH);
-  return((void *) bitmapH);
-}
-
 #endif
