@@ -1,11 +1,11 @@
 /*
-Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
+Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization
 dedicated to making software imaging solutions freely available.
 
 You may not use this file except in compliance with the License.
 obtain a copy of the License at
 
-http://www.imagemagick.org/script/license.php
+https://imagemagick.org/script/license.php
 
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
@@ -100,7 +100,13 @@ typedef CL_API_ENTRY cl_mem (CL_API_CALL *MAGICKpfn_clCreateBuffer)(
     void *       host_ptr,
     cl_int *     errcode_ret) CL_API_SUFFIX__VERSION_1_0;
 
-typedef CL_API_ENTRY cl_int (CL_API_CALL *MAGICKpfn_clReleaseMemObject)(cl_mem memobj) CL_API_SUFFIX__VERSION_1_0;
+typedef CL_API_ENTRY cl_int
+  (CL_API_CALL *MAGICKpfn_clRetainMemObject)(cl_mem memobj)
+    CL_API_SUFFIX__VERSION_1_0;
+
+typedef CL_API_ENTRY cl_int
+  (CL_API_CALL *MAGICKpfn_clReleaseMemObject)(cl_mem memobj)
+    CL_API_SUFFIX__VERSION_1_0;
 
 /* Program Object APIs */
 typedef CL_API_ENTRY cl_program (CL_API_CALL *MAGICKpfn_clCreateProgramWithSource)(
@@ -214,6 +220,11 @@ typedef CL_API_ENTRY cl_int(CL_API_CALL *MAGICKpfn_clGetEventProfilingInfo)(
     void *param_value,
     size_t *param_value_size_ret) CL_API_SUFFIX__VERSION_1_0;
 
+typedef CL_API_ENTRY cl_int
+  (CL_API_CALL *MAGICKpfn_clGetEventInfo)(cl_event event,
+    cl_profiling_info param_name,size_t param_value_size,void *param_value,
+    size_t *param_value_size_ret) CL_API_SUFFIX__VERSION_1_0;
+
 typedef CL_API_ENTRY cl_int(CL_API_CALL *MAGICKpfn_clWaitForEvents)(
     cl_uint num_events,
     const cl_event *event_list) CL_API_SUFFIX__VERSION_1_0;
@@ -259,6 +270,7 @@ struct MagickLibraryRec
   MAGICKpfn_clFinish                  clFinish;
 
   MAGICKpfn_clCreateBuffer            clCreateBuffer;
+  MAGICKpfn_clRetainMemObject         clRetainMemObject;
   MAGICKpfn_clReleaseMemObject        clReleaseMemObject;
   MAGICKpfn_clCreateProgramWithSource clCreateProgramWithSource;
   MAGICKpfn_clCreateProgramWithBinary clCreateProgramWithBinary;
@@ -278,6 +290,7 @@ struct MagickLibraryRec
 
   MAGICKpfn_clGetEventProfilingInfo   clGetEventProfilingInfo;
 
+  MAGICKpfn_clGetEventInfo            clGetEventInfo;
   MAGICKpfn_clWaitForEvents           clWaitForEvents;
   MAGICKpfn_clReleaseEvent            clReleaseEvent;
   MAGICKpfn_clRetainEvent             clRetainEvent;
@@ -404,9 +417,7 @@ extern MagickPrivate const char*
   GetOpenCLCachedFilesDirectory();
 
 extern MagickPrivate void
-  OpenCLLog(const char*);
-
-extern MagickPrivate void
+  OpenCLLog(const char*),
   OpenCLTerminus();
 
 /* #define OPENCLLOG_ENABLED 1 */
