@@ -17,13 +17,13 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -60,6 +60,7 @@
 #include "magick/layer.h"
 #include "magick/mime-private.h"
 #include "magick/memory_.h"
+#include "magick/memory-private.h"
 #include "magick/monitor.h"
 #include "magick/montage.h"
 #include "magick/morphology.h"
@@ -212,6 +213,9 @@ MagickPrivate MagickBooleanType RecordProfileData(MagickCLEnv clEnv, ProfiledKer
   }
   return(MagickTrue);
 #else
+  magick_unreferenced(clEnv);
+  magick_unreferenced(kernel);
+  magick_unreferenced(event);
   return(MagickFalse);
 #endif
 }
@@ -223,7 +227,9 @@ void DumpProfileData()
 
   OpenCLLog("====================================================");
 
-  // Write out the device info to the profile
+  /*
+    Write out the device info to the profile.
+  */
   if (0 == 1)
   {
     MagickCLEnv clEnv;
@@ -251,7 +257,9 @@ void DumpProfileData()
     strcpy(indent, "                              ");
     strncpy(indent, kernelNames[i], min(strlen(kernelNames[i]), strlen(indent) - 1));
     sprintf(buf, "%s%d\t(%d calls)   \t%d -> %d", indent, profileRecords[i].count > 0 ? (profileRecords[i].total / profileRecords[i].count) : 0, profileRecords[i].count, profileRecords[i].min, profileRecords[i].max);
-    //printf("%s%d\t(%d calls)   \t%d -> %d\n", indent, profileRecords[i].count > 0 ? (profileRecords[i].total / profileRecords[i].count) : 0, profileRecords[i].count, profileRecords[i].min, profileRecords[i].max);
+    /*
+      printf("%s%d\t(%d calls)   \t%d -> %d\n", indent, profileRecords[i].count > 0 ? (profileRecords[i].total / profileRecords[i].count) : 0, profileRecords[i].count, profileRecords[i].min, profileRecords[i].max);
+    */
     OpenCLLog(buf);
   }
   OpenCLLog("====================================================");
@@ -405,6 +413,7 @@ static MagickBooleanType bindOpenCLFunctions(void* library)
   BIND(clReleaseContext);
 
   BIND(clCreateBuffer);
+  BIND(clRetainMemObject);
   BIND(clReleaseMemObject);
 
   BIND(clCreateProgramWithSource);
@@ -430,6 +439,7 @@ static MagickBooleanType bindOpenCLFunctions(void* library)
   BIND(clReleaseCommandQueue);
 
   BIND(clGetEventProfilingInfo);
+  BIND(clGetEventInfo);
   BIND(clWaitForEvents);
   BIND(clReleaseEvent);
   BIND(clRetainEvent);
@@ -888,11 +898,24 @@ static void saveBinaryCLProgram(MagickCLEnv clEnv,MagickOpenCLProgram prog,
       binary_program_size=num_devices*sizeof(*binary_program);
       binary_program=(unsigned char **) AcquireMagickMemory(
         binary_program_size);
+      if (binary_program == (unsigned char **) NULL)
+        {
+          program_sizes=(size_t *) RelinquishMagickMemory(program_sizes);
+          return;
+        }
       for (i = 0; i < num_devices; i++)
+      {
         binary_program[i]=AcquireQuantumMemory(MagickMax(*(program_sizes+i),1),
           sizeof(**binary_program));
-      status=clEnv->library->clGetProgramInfo(clEnv->programs[prog],
-        CL_PROGRAM_BINARIES,binary_program_size,binary_program,NULL);
+        if (binary_program[i] == (unsigned char *) NULL)
+        {
+          status=CL_OUT_OF_HOST_MEMORY;
+          break;
+        }
+      }
+      if (status == CL_SUCCESS)
+        status=clEnv->library->clGetProgramInfo(clEnv->programs[prog],
+          CL_PROGRAM_BINARIES,binary_program_size,binary_program,NULL);
       if (status == CL_SUCCESS)
         {
           for (i = 0; i < num_devices; i++)
@@ -1129,7 +1152,7 @@ static MagickBooleanType CompileOpenCLKernels(MagickCLEnv clEnv, ExceptionInfo*
           char* log;
           size_t logSize;
           clEnv->library->clGetProgramBuildInfo(clEnv->programs[i], clEnv->device, CL_PROGRAM_BUILD_LOG, 0, NULL, &logSize);
-          log = (char*)AcquireMagickMemory(logSize);
+          log = (char*)AcquireCriticalMemory(logSize);
           clEnv->library->clGetProgramBuildInfo(clEnv->programs[i], clEnv->device, CL_PROGRAM_BUILD_LOG, logSize, log, &logSize);
 
           (void) FormatLocaleString(path,MaxTextExtent,"%s%s%s"
@@ -1383,7 +1406,7 @@ MagickBooleanType InitOpenCLEnvInternal(MagickCLEnv clEnv, ExceptionInfo* except
   {
     int status = clInitializePerfMarkerAMD();
     if (status == AP_SUCCESS) {
-      //printf("PerfMarker successfully initialized\n");
+      /* printf("PerfMarker successfully initialized\n"); */
     }
   }
 #endif
@@ -2440,7 +2463,7 @@ static ds_status AcceleratePerfEvaluator(ds_device *device,
       cl_uint
         event_count;
 
-      const cl_event
+      cl_event
         *events;
 
       Image
@@ -2470,6 +2493,7 @@ static ds_status AcceleratePerfEvaluator(ds_device *device,
           events=GetOpenCLEvents(resizedImage,&event_count);
           if (event_count > 0)
             clEnv->library->clWaitForEvents(event_count,events);
+          events=(cl_event *) RelinquishMagickMemory(events);
         }
 
 #ifdef MAGICKCORE_CLPERFMARKER
@@ -2791,7 +2815,7 @@ MagickBooleanType OpenCLThrowMagickException(ExceptionInfo *exception,
   clEnv = GetDefaultOpenCLEnv();
 
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
 
   if (severity!=0) {
     cl_device_type dType;
@@ -2895,7 +2919,7 @@ const char* GetOpenCLCachedFilesDirectory() {
 
         if (mkdirStatus==0)
         {
-          temp = (char*)AcquireMagickMemory(strlen(path)+1);
+          temp = (char*)AcquireCriticalMemory(strlen(path)+1);
           CopyMagickString(temp,path,strlen(path)+1);
         }
         home=DestroyString(home);
@@ -2940,7 +2964,7 @@ const char* GetOpenCLCachedFilesDirectory() {
 
           if (mkdirStatus==0)
           {
-            temp = (char*)AcquireMagickMemory(strlen(path)+1);
+            temp = (char*)AcquireCriticalMemory(strlen(path)+1);
             CopyMagickString(temp,path,strlen(path)+1);
           }
           home=DestroyString(home);
