@@ -16,13 +16,13 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -118,7 +118,7 @@ MagickExport QuantumInfo *AcquireQuantumInfo(const ImageInfo *image_info,
   quantum_info=(QuantumInfo *) AcquireMagickMemory(sizeof(*quantum_info));
   if (quantum_info == (QuantumInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  quantum_info->signature=MagickSignature;
+  quantum_info->signature=MagickCoreSignature;
   GetQuantumInfo(image_info,quantum_info);
   if (image == (const Image *) NULL)
     return(quantum_info);
@@ -161,29 +161,29 @@ static MagickBooleanType AcquireQuantumPixels(QuantumInfo *quantum_info,
     i;
 
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->number_threads=(size_t) GetMagickResourceLimit(ThreadResource);
-  quantum_info->pixels=(unsigned char **) AcquireQuantumMemory(
+  quantum_info->pixels=(MemoryInfo **) AcquireQuantumMemory(
     quantum_info->number_threads,sizeof(*quantum_info->pixels));
-  if (quantum_info->pixels == (unsigned char **) NULL)
+  if (quantum_info->pixels == (MemoryInfo **) NULL)
     return(MagickFalse);
   quantum_info->extent=extent;
-  (void) ResetMagickMemory(quantum_info->pixels,0,quantum_info->number_threads*
+  (void) memset(quantum_info->pixels,0,quantum_info->number_threads*
     sizeof(*quantum_info->pixels));
   for (i=0; i < (ssize_t) quantum_info->number_threads; i++)
   {
-    quantum_info->pixels[i]=(unsigned char *) AcquireQuantumMemory(extent+1,
-      sizeof(**quantum_info->pixels));
-    if (quantum_info->pixels[i] == (unsigned char *) NULL)
+    unsigned char
+      *pixels;
+
+    quantum_info->pixels[i]=AcquireVirtualMemory((extent+1),sizeof(*pixels));
+    if (quantum_info->pixels[i] == (MemoryInfo *) NULL)
       {
-        while (--i >= 0)
-          quantum_info->pixels[i]=(unsigned char *) RelinquishMagickMemory(
-            quantum_info->pixels[i]);
+        DestroyQuantumPixels(quantum_info);
         return(MagickFalse);
       }
-    (void) ResetMagickMemory(quantum_info->pixels[i],0,(extent+1)*
-      sizeof(**quantum_info->pixels));
-    quantum_info->pixels[i][extent]=QuantumSignature;
+    pixels=(unsigned char *)  GetVirtualMemoryBlob(quantum_info->pixels[i]);
+    (void) memset(pixels,0,(extent+1)*sizeof(*pixels));
+    pixels[extent]=QuantumSignature;
   }
   return(MagickTrue);
 }
@@ -214,12 +214,12 @@ static MagickBooleanType AcquireQuantumPixels(QuantumInfo *quantum_info,
 MagickExport QuantumInfo *DestroyQuantumInfo(QuantumInfo *quantum_info)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
-  if (quantum_info->pixels != (unsigned char **) NULL)
+  assert(quantum_info->signature == MagickCoreSignature);
+  if (quantum_info->pixels != (MemoryInfo **) NULL)
     DestroyQuantumPixels(quantum_info);
   if (quantum_info->semaphore != (SemaphoreInfo *) NULL)
     DestroySemaphoreInfo(&quantum_info->semaphore);
-  quantum_info->signature=(~MagickSignature);
+  quantum_info->signature=(~MagickCoreSignature);
   quantum_info=(QuantumInfo *) RelinquishMagickMemory(quantum_info);
   return(quantum_info);
 }
@@ -255,20 +255,24 @@ static void DestroyQuantumPixels(QuantumInfo *quantum_info)
     extent;
 
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
-  assert(quantum_info->pixels != (unsigned char **) NULL);
+  assert(quantum_info->signature == MagickCoreSignature);
+  assert(quantum_info->pixels != (MemoryInfo **) NULL);
   extent=(ssize_t) quantum_info->extent;
   for (i=0; i < (ssize_t) quantum_info->number_threads; i++)
-    if (quantum_info->pixels[i] != (unsigned char *) NULL)
+    if (quantum_info->pixels[i] != (MemoryInfo *) NULL)
       {
+        unsigned char
+          *pixels;
+
         /*
           Did we overrun our quantum buffer?
         */
-        assert(quantum_info->pixels[i][extent] == QuantumSignature);
-        quantum_info->pixels[i]=(unsigned char *) RelinquishMagickMemory(
+        pixels=(unsigned char *) GetVirtualMemoryBlob(quantum_info->pixels[i]);
+        assert(pixels[extent] == QuantumSignature);
+        quantum_info->pixels[i]=RelinquishVirtualMemory(
           quantum_info->pixels[i]);
       }
-  quantum_info->pixels=(unsigned char **) RelinquishMagickMemory(
+  quantum_info->pixels=(MemoryInfo **) RelinquishMagickMemory(
     quantum_info->pixels);
 }
 
@@ -307,7 +311,7 @@ MagickExport size_t GetQuantumExtent(const Image *image,
     packet_size;
 
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   packet_size=1;
   switch (quantum_type)
   {
@@ -355,7 +359,7 @@ MagickExport size_t GetQuantumExtent(const Image *image,
 MagickExport EndianType GetQuantumEndian(const QuantumInfo *quantum_info)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   return(quantum_info->endian);
 }
 
@@ -384,7 +388,7 @@ MagickExport EndianType GetQuantumEndian(const QuantumInfo *quantum_info)
 MagickExport QuantumFormatType GetQuantumFormat(const QuantumInfo *quantum_info)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   return(quantum_info->format);
 }
 
@@ -419,13 +423,13 @@ MagickExport void GetQuantumInfo(const ImageInfo *image_info,
     *option;
 
   assert(quantum_info != (QuantumInfo *) NULL);
-  (void) ResetMagickMemory(quantum_info,0,sizeof(*quantum_info));
+  (void) memset(quantum_info,0,sizeof(*quantum_info));
   quantum_info->quantum=8;
   quantum_info->maximum=1.0;
   quantum_info->scale=QuantumRange;
   quantum_info->pack=MagickTrue;
   quantum_info->semaphore=AllocateSemaphoreInfo();
-  quantum_info->signature=MagickSignature;
+  quantum_info->signature=MagickCoreSignature;
   if (image_info == (const ImageInfo *) NULL)
     return;
   option=GetImageOption(image_info,"quantum:format");
@@ -489,8 +493,8 @@ MagickExport unsigned char *GetQuantumPixels(const QuantumInfo *quantum_info)
     id = GetOpenMPThreadId();
 
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
-  return(quantum_info->pixels[id]);
+  assert(quantum_info->signature == MagickCoreSignature);
+  return((unsigned char *) GetVirtualMemoryBlob(quantum_info->pixels[id]));
 }
 
 /*
@@ -521,7 +525,7 @@ MagickExport QuantumType GetQuantumType(Image *image,ExceptionInfo *exception)
     quantum_type;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   (void) exception;
@@ -585,7 +589,7 @@ MagickPrivate void ResetQuantumState(QuantumInfo *quantum_info)
   };
 
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->state.inverse_scale=1.0;
   if (fabs(quantum_info->scale) >= MagickEpsilon)
     quantum_info->state.inverse_scale/=quantum_info->scale;
@@ -623,7 +627,7 @@ MagickExport void SetQuantumAlphaType(QuantumInfo *quantum_info,
   const QuantumAlphaType type)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->alpha_type=type;
 }
 
@@ -665,11 +669,11 @@ MagickExport MagickBooleanType SetQuantumDepth(const Image *image,
     Allocate the quantum pixel buffer.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->depth=depth;
   if (quantum_info->format == FloatingPointQuantumFormat)
     {
@@ -681,7 +685,7 @@ MagickExport MagickBooleanType SetQuantumDepth(const Image *image,
         else
           quantum_info->depth=16;
     }
-  if (quantum_info->pixels != (unsigned char **) NULL)
+  if (quantum_info->pixels != (MemoryInfo **) NULL)
     DestroyQuantumPixels(quantum_info);
   quantum=(quantum_info->pad+6)*(quantum_info->depth+7)/8;
   extent=MagickMax(image->columns,image->rows)*quantum;
@@ -722,11 +726,11 @@ MagickExport MagickBooleanType SetQuantumEndian(const Image *image,
   QuantumInfo *quantum_info,const EndianType endian)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->endian=endian;
   return(SetQuantumDepth(image,quantum_info,quantum_info->depth));
 }
@@ -762,11 +766,11 @@ MagickExport MagickBooleanType SetQuantumFormat(const Image *image,
   QuantumInfo *quantum_info,const QuantumFormatType format)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->format=format;
   return(SetQuantumDepth(image,quantum_info,quantum_info->depth));
 }
@@ -801,7 +805,7 @@ MagickExport void SetQuantumImageType(Image *image,
   const QuantumType quantum_type)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   switch (quantum_type)
@@ -867,7 +871,7 @@ MagickExport void SetQuantumPack(QuantumInfo *quantum_info,
   const MagickBooleanType pack)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->pack=pack;
 }
 
@@ -902,11 +906,11 @@ MagickExport MagickBooleanType SetQuantumPad(const Image *image,
   QuantumInfo *quantum_info,const size_t pad)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->pad=pad;
   return(SetQuantumDepth(image,quantum_info,quantum_info->depth));
 }
@@ -940,7 +944,7 @@ MagickExport void SetQuantumMinIsWhite(QuantumInfo *quantum_info,
   const MagickBooleanType min_is_white)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->min_is_white=min_is_white;
 }
 
@@ -973,7 +977,7 @@ MagickExport void SetQuantumQuantum(QuantumInfo *quantum_info,
   const size_t quantum)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->quantum=quantum;
 }
 
@@ -1004,6 +1008,6 @@ MagickExport void SetQuantumQuantum(QuantumInfo *quantum_info,
 MagickExport void SetQuantumScale(QuantumInfo *quantum_info,const double scale)
 {
   assert(quantum_info != (QuantumInfo *) NULL);
-  assert(quantum_info->signature == MagickSignature);
+  assert(quantum_info->signature == MagickCoreSignature);
   quantum_info->scale=scale;
 }
