@@ -16,13 +16,13 @@
 %                              December 2001                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -44,7 +44,7 @@
 #if defined(__VMS)
 #include <time.h>
 #endif
-#if defined(__MINGW32__) || defined(__MINGW64__)
+#if defined(__MINGW32__)
 #include <sys/time.h>
 #endif
 #include "magick/studio.h"
@@ -173,7 +173,7 @@ MagickExport RandomInfo *AcquireRandomInfo(void)
   random_info=(RandomInfo *) AcquireMagickMemory(sizeof(*random_info));
   if (random_info == (RandomInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(random_info,0,sizeof(*random_info));
+  (void) memset(random_info,0,sizeof(*random_info));
   random_info->signature_info=AcquireSignatureInfo();
   random_info->nonce=AcquireStringInfo(2*GetSignatureDigestsize(
     random_info->signature_info));
@@ -187,7 +187,7 @@ MagickExport RandomInfo *AcquireRandomInfo(void)
   random_info->protocol_minor=RandomProtocolMinorVersion;
   random_info->semaphore=AllocateSemaphoreInfo();
   random_info->timestamp=(ssize_t) time(0);
-  random_info->signature=MagickSignature;
+  random_info->signature=MagickCoreSignature;
   /*
     Seed random nonce.
   */
@@ -219,7 +219,7 @@ MagickExport RandomInfo *AcquireRandomInfo(void)
   if (random_info->secret_key == ~0UL)
     {
       key=GetRandomKey(random_info,sizeof(random_info->secret_key));
-      (void) CopyMagickMemory(random_info->seed,GetStringInfoDatum(key),
+      (void) memcpy(random_info->seed,GetStringInfoDatum(key),
         GetStringInfoLength(key));
       key=DestroyStringInfo(key);
     }
@@ -235,7 +235,7 @@ MagickExport RandomInfo *AcquireRandomInfo(void)
       key=DestroyStringInfo(key);
       FinalizeSignature(signature_info);
       digest=GetSignatureDigest(signature_info);
-      (void) CopyMagickMemory(random_info->seed,GetStringInfoDatum(digest),
+      (void) memcpy(random_info->seed,GetStringInfoDatum(digest),
         MagickMin(GetSignatureDigestsize(signature_info),
         sizeof(*random_info->seed)));
       signature_info=DestroySignatureInfo(signature_info);
@@ -273,7 +273,7 @@ MagickExport RandomInfo *DestroyRandomInfo(RandomInfo *random_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(random_info != (RandomInfo *) NULL);
-  assert(random_info->signature == MagickSignature);
+  assert(random_info->signature == MagickCoreSignature);
   LockSemaphoreInfo(random_info->semaphore);
   if (random_info->reservoir != (StringInfo *) NULL)
     random_info->reservoir=DestroyStringInfo(random_info->reservoir);
@@ -282,8 +282,8 @@ MagickExport RandomInfo *DestroyRandomInfo(RandomInfo *random_info)
   if (random_info->signature_info != (SignatureInfo *) NULL)
     random_info->signature_info=DestroySignatureInfo(
       random_info->signature_info);
-  (void) ResetMagickMemory(random_info->seed,0,sizeof(*random_info->seed));
-  random_info->signature=(~MagickSignature);
+  (void) memset(random_info->seed,0,sizeof(random_info->seed));
+  random_info->signature=(~MagickCoreSignature);
   UnlockSemaphoreInfo(random_info->semaphore);
   DestroySemaphoreInfo(&random_info->semaphore);
   random_info=(RandomInfo *) RelinquishMagickMemory(random_info);
@@ -457,7 +457,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     int
       file;
 
-    (void) GetPathTemplate(path);
+    (void) strcpy(path,"XXXXXX");
     file=mkstemp(path);
     if (file != -1)
       {
@@ -912,7 +912,7 @@ MagickExport void SetRandomKey(RandomInfo *random_info,const size_t length,
     UpdateSignature(signature_info,random_info->nonce);
     FinalizeSignature(signature_info);
     IncrementRandomNonce(random_info->nonce);
-    (void) CopyMagickMemory(p,GetStringInfoDatum(GetSignatureDigest(
+    (void) memcpy(p,GetStringInfoDatum(GetSignatureDigest(
       signature_info)),GetSignatureDigestsize(signature_info));
     p+=GetSignatureDigestsize(signature_info);
     i-=GetSignatureDigestsize(signature_info);
