@@ -18,13 +18,13 @@
 %                                August 2007                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -210,24 +210,24 @@ MagickExport ResampleFilter *AcquireResampleFilter(const Image *image,
     *resample_filter;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
 
   resample_filter=(ResampleFilter *) AcquireMagickMemory(
     sizeof(*resample_filter));
   if (resample_filter == (ResampleFilter *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(resample_filter,0,sizeof(*resample_filter));
+  (void) memset(resample_filter,0,sizeof(*resample_filter));
 
   resample_filter->exception=exception;
   resample_filter->image=ReferenceImage((Image *) image);
   resample_filter->view=AcquireVirtualCacheView(resample_filter->image,exception);
 
   resample_filter->debug=IsEventLogging();
-  resample_filter->signature=MagickSignature;
+  resample_filter->signature=MagickCoreSignature;
 
   resample_filter->image_area=(ssize_t) (image->columns*image->rows);
   resample_filter->average_defined = MagickFalse;
@@ -270,7 +270,7 @@ MagickExport ResampleFilter *DestroyResampleFilter(
   ResampleFilter *resample_filter)
 {
   assert(resample_filter != (ResampleFilter *) NULL);
-  assert(resample_filter->signature == MagickSignature);
+  assert(resample_filter->signature == MagickCoreSignature);
   assert(resample_filter->image != (Image *) NULL);
   if (resample_filter->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
@@ -280,7 +280,7 @@ MagickExport ResampleFilter *DestroyResampleFilter(
 #if ! FILTER_LUT
   resample_filter->filter_def=DestroyResizeFilter(resample_filter->filter_def);
 #endif
-  resample_filter->signature=(~MagickSignature);
+  resample_filter->signature=(~MagickCoreSignature);
   resample_filter=(ResampleFilter *) RelinquishMagickMemory(resample_filter);
   return(resample_filter);
 }
@@ -331,7 +331,7 @@ MagickExport MagickBooleanType ResamplePixelColor(
   register const PixelPacket *pixels;
   register const IndexPacket *indexes;
   assert(resample_filter != (ResampleFilter *) NULL);
-  assert(resample_filter->signature == MagickSignature);
+  assert(resample_filter->signature == MagickCoreSignature);
 
   status=MagickTrue;
   /* GetMagickPixelPacket(resample_filter->image,pixel); */
@@ -1053,7 +1053,7 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
   double A,B,C,F;
 
   assert(resample_filter != (ResampleFilter *) NULL);
-  assert(resample_filter->signature == MagickSignature);
+  assert(resample_filter->signature == MagickCoreSignature);
 
   resample_filter->limit_reached = MagickFalse;
 
@@ -1259,7 +1259,7 @@ MagickExport void SetResampleFilter(ResampleFilter *resample_filter,
      *resize_filter;
 
   assert(resample_filter != (ResampleFilter *) NULL);
-  assert(resample_filter->signature == MagickSignature);
+  assert(resample_filter->signature == MagickCoreSignature);
 
   resample_filter->do_interpolate = MagickFalse;
   resample_filter->filter = filter;
@@ -1419,7 +1419,7 @@ MagickExport MagickBooleanType SetResampleFilterInterpolateMethod(
   ResampleFilter *resample_filter,const InterpolatePixelMethod method)
 {
   assert(resample_filter != (ResampleFilter *) NULL);
-  assert(resample_filter->signature == MagickSignature);
+  assert(resample_filter->signature == MagickCoreSignature);
   assert(resample_filter->image != (Image *) NULL);
   if (resample_filter->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
@@ -1458,7 +1458,7 @@ MagickExport MagickBooleanType SetResampleFilterVirtualPixelMethod(
   ResampleFilter *resample_filter,const VirtualPixelMethod method)
 {
   assert(resample_filter != (ResampleFilter *) NULL);
-  assert(resample_filter->signature == MagickSignature);
+  assert(resample_filter->signature == MagickCoreSignature);
   assert(resample_filter->image != (Image *) NULL);
   if (resample_filter->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
