@@ -16,13 +16,13 @@
 %                              December 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -35,8 +35,7 @@
 %
 %
 */
-
-
+
 /*
   Include declarations.
 */
@@ -57,8 +56,7 @@
 */
 #define SignatureBlocksize  64
 #define SignatureDigestsize  32
-
-
+
 /*
   Typedef declarations.
 */
@@ -78,7 +76,7 @@ struct _SignatureInfo
     high_order;
 
   size_t
-    offset;
+    extent;
 
   MagickBooleanType
     lsb_first;
@@ -89,15 +87,13 @@ struct _SignatureInfo
   size_t
     signature;
 };
-
-
+
 /*
   Forward declarations.
 */
 static void
   TransformSignature(SignatureInfo *);
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -121,13 +117,13 @@ MagickExport SignatureInfo *AcquireSignatureInfo(void)
   SignatureInfo
     *signature_info;
 
-  unsigned int
+  unsigned long
     lsb_first;
 
   signature_info=(SignatureInfo *) AcquireMagickMemory(sizeof(*signature_info));
   if (signature_info == (SignatureInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(signature_info,0,sizeof(*signature_info));
+  (void) memset(signature_info,0,sizeof(*signature_info));
   signature_info->digestsize=SignatureDigestsize;
   signature_info->blocksize=SignatureBlocksize;
   signature_info->digest=AcquireStringInfo(SignatureDigestsize);
@@ -140,12 +136,11 @@ MagickExport SignatureInfo *AcquireSignatureInfo(void)
   signature_info->lsb_first=(int) (*(char *) &lsb_first) == 1 ? MagickTrue :
     MagickFalse;
   signature_info->timestamp=(ssize_t) time(0);
-  signature_info->signature=MagickSignature;
+  signature_info->signature=MagickCoreSignature;
   InitializeSignature(signature_info);
   return(signature_info);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -173,7 +168,7 @@ MagickExport SignatureInfo *DestroySignatureInfo(SignatureInfo *signature_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(signature_info != (SignatureInfo *) NULL);
-  assert(signature_info->signature == MagickSignature);
+  assert(signature_info->signature == MagickCoreSignature);
   if (signature_info->accumulator != (unsigned int *) NULL)
     signature_info->accumulator=(unsigned int *) RelinquishMagickMemory(
       signature_info->accumulator);
@@ -181,12 +176,11 @@ MagickExport SignatureInfo *DestroySignatureInfo(SignatureInfo *signature_info)
     signature_info->message=DestroyStringInfo(signature_info->message);
   if (signature_info->digest != (StringInfo *) NULL)
     signature_info->digest=DestroyStringInfo(signature_info->digest);
-  signature_info->signature=(~MagickSignature);
+  signature_info->signature=(~MagickCoreSignature);
   signature_info=(SignatureInfo *) RelinquishMagickMemory(signature_info);
   return(signature_info);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -220,11 +214,13 @@ MagickExport void FinalizeSignature(SignatureInfo *signature_info)
   register unsigned int
     *p;
 
+  size_t
+    extent;
+
   unsigned char
     *datum;
 
   unsigned int
-    count,
     high_order,
     low_order;
 
@@ -233,21 +229,21 @@ MagickExport void FinalizeSignature(SignatureInfo *signature_info)
   */
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(signature_info != (SignatureInfo *) NULL);
-  assert(signature_info->signature == MagickSignature);
+  assert(signature_info->signature == MagickCoreSignature);
   low_order=signature_info->low_order;
   high_order=signature_info->high_order;
-  count=((low_order >> 3) & 0x3f);
+  extent=((low_order >> 3) & 0x3f);
   datum=GetStringInfoDatum(signature_info->message);
-  datum[count++]=(unsigned char) 0x80;
-  if (count <= (unsigned int) (GetStringInfoLength(signature_info->message)-8))
-    (void) ResetMagickMemory(datum+count,0,GetStringInfoLength(
-      signature_info->message)-8-count);
+  datum[extent++]=(unsigned char) 0x80;
+  if (extent <= (unsigned int) (GetStringInfoLength(signature_info->message)-8))
+    (void) memset(datum+extent,0,GetStringInfoLength(
+      signature_info->message)-8-extent);
   else
     {
-      (void) ResetMagickMemory(datum+count,0,GetStringInfoLength(
-        signature_info->message)-count);
+      (void) memset(datum+extent,0,GetStringInfoLength(
+        signature_info->message)-extent);
       TransformSignature(signature_info);
-      (void) ResetMagickMemory(datum,0,GetStringInfoLength(
+      (void) memset(datum,0,GetStringInfoLength(
         signature_info->message)-8);
     }
   datum[56]=(unsigned char) (high_order >> 24);
@@ -269,15 +265,8 @@ MagickExport void FinalizeSignature(SignatureInfo *signature_info)
     *q++=(unsigned char) (*p & 0xff);
     p++;
   }
-  /*
-    Reset working registers.
-  */
-  count=0;
-  high_order=0;
-  low_order=0;
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -305,11 +294,10 @@ MagickExport unsigned int GetSignatureBlocksize(
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(signature_info != (SignatureInfo *) NULL);
-  assert(signature_info->signature == MagickSignature);
+  assert(signature_info->signature == MagickCoreSignature);
   return(signature_info->blocksize);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -337,11 +325,10 @@ MagickExport const StringInfo *GetSignatureDigest(
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(signature_info != (SignatureInfo *) NULL);
-  assert(signature_info->signature == MagickSignature);
+  assert(signature_info->signature == MagickCoreSignature);
   return(signature_info->digest);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -369,11 +356,10 @@ MagickExport unsigned int GetSignatureDigestsize(
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(signature_info != (SignatureInfo *) NULL);
-  assert(signature_info->signature == MagickSignature);
+  assert(signature_info->signature == MagickCoreSignature);
   return(signature_info->digestsize);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -400,7 +386,7 @@ MagickExport void InitializeSignature(SignatureInfo *signature_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(signature_info != (SignatureInfo *) NULL);
-  assert(signature_info->signature == MagickSignature);
+  assert(signature_info->signature == MagickCoreSignature);
   signature_info->accumulator[0]=0x6a09e667U;
   signature_info->accumulator[1]=0xbb67ae85U;
   signature_info->accumulator[2]=0x3c6ef372U;
@@ -411,10 +397,9 @@ MagickExport void InitializeSignature(SignatureInfo *signature_info)
   signature_info->accumulator[7]=0x5be0cd19U;
   signature_info->low_order=0;
   signature_info->high_order=0;
-  signature_info->offset=0;
+  signature_info->extent=0;
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -447,11 +432,10 @@ MagickExport void SetSignatureDigest(SignatureInfo *signature_info,
     Set the signature accumulator.
   */
   assert(signature_info != (SignatureInfo *) NULL);
-  assert(signature_info->signature == MagickSignature);
+  assert(signature_info->signature == MagickCoreSignature);
   SetStringInfo(signature_info->digest,digest);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -516,9 +500,10 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
     Compute image digital signature.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  exception=(&image->exception);
   quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
   if (quantum_info == (QuantumInfo *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
@@ -535,7 +520,6 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
   signature_info=AcquireSignatureInfo();
   signature=AcquireStringInfo(quantum_info->extent);
   pixels=GetQuantumPixels(quantum_info);
-  exception=(&image->exception);
   image_view=AcquireVirtualCacheView(image,exception);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
@@ -562,8 +546,7 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
   signature_info=DestroySignatureInfo(signature_info);
   return(MagickTrue);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -751,10 +734,9 @@ RestoreMSCWarning
   T=0;
   T1=0;
   T2=0;
-  (void) ResetMagickMemory(W,0,sizeof(W));
+  (void) memset(W,0,sizeof(W));
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -798,7 +780,7 @@ MagickExport void UpdateSignature(SignatureInfo *signature_info,
     Update the Signature accumulator.
   */
   assert(signature_info != (SignatureInfo *) NULL);
-  assert(signature_info->signature == MagickSignature);
+  assert(signature_info->signature == MagickCoreSignature);
   n=GetStringInfoLength(message);
   length=Trunc32((unsigned int) (signature_info->low_order+(n << 3)));
   if (length < signature_info->low_order)
@@ -806,17 +788,17 @@ MagickExport void UpdateSignature(SignatureInfo *signature_info,
   signature_info->low_order=length;
   signature_info->high_order+=(unsigned int) (n >> 29);
   p=GetStringInfoDatum(message);
-  if (signature_info->offset != 0)
+  if (signature_info->extent != 0)
     {
-      i=GetStringInfoLength(signature_info->message)-signature_info->offset;
+      i=GetStringInfoLength(signature_info->message)-signature_info->extent;
       if (i > n)
         i=n;
-      (void) CopyMagickMemory(GetStringInfoDatum(signature_info->message)+
-        signature_info->offset,p,i);
+      (void) memcpy(GetStringInfoDatum(signature_info->message)+
+        signature_info->extent,p,i);
       n-=i;
       p+=i;
-      signature_info->offset+=i;
-      if (signature_info->offset !=
+      signature_info->extent+=i;
+      if (signature_info->extent !=
           GetStringInfoLength(signature_info->message))
         return;
       TransformSignature(signature_info);
@@ -828,12 +810,6 @@ MagickExport void UpdateSignature(SignatureInfo *signature_info,
     n-=GetStringInfoLength(signature_info->message);
     TransformSignature(signature_info);
   }
-  (void) CopyMagickMemory(GetStringInfoDatum(signature_info->message),p,n);
-  signature_info->offset=n;
-  /*
-    Reset working registers.
-  */
-  i=0;
-  n=0;
-  length=0;
+  (void) memcpy(GetStringInfoDatum(signature_info->message),p,n);
+  signature_info->extent=n;
 }
