@@ -23,13 +23,13 @@
 %                               December 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -353,7 +353,7 @@ MagickExport SplayTreeInfo *CloneSplayTree(SplayTreeInfo *splay_tree,
     *clone_tree;
 
   assert(splay_tree != (SplayTreeInfo *) NULL);
-  assert(splay_tree->signature == MagickSignature);
+  assert(splay_tree->signature == MagickCoreSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   clone_tree=NewSplayTree(splay_tree->compare,splay_tree->relinquish_key,
@@ -490,7 +490,7 @@ MagickExport MagickBooleanType DeleteNodeByValueFromSplayTree(
     *node;
 
   assert(splay_tree != (SplayTreeInfo *) NULL);
-  assert(splay_tree->signature == MagickSignature);
+  assert(splay_tree->signature == MagickCoreSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   LockSemaphoreInfo(splay_tree->semaphore);
@@ -610,7 +610,7 @@ MagickExport MagickBooleanType DeleteNodeFromSplayTree(
     *right;
 
   assert(splay_tree != (SplayTreeInfo *) NULL);
-  assert(splay_tree->signature == MagickSignature);
+  assert(splay_tree->signature == MagickCoreSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (splay_tree->root == (NodeInfo *) NULL)
@@ -734,7 +734,7 @@ MagickExport SplayTreeInfo *DestroySplayTree(SplayTreeInfo *splay_tree)
         }
       }
     }
-  splay_tree->signature=(~MagickSignature);
+  splay_tree->signature=(~MagickCoreSignature);
   UnlockSemaphoreInfo(splay_tree->semaphore);
   DestroySemaphoreInfo(&splay_tree->semaphore);
   splay_tree=(SplayTreeInfo *) RelinquishMagickMemory(splay_tree);
@@ -774,7 +774,7 @@ MagickExport const void *GetNextKeyInSplayTree(SplayTreeInfo *splay_tree)
     *key;
 
   assert(splay_tree != (SplayTreeInfo *) NULL);
-  assert(splay_tree->signature == MagickSignature);
+  assert(splay_tree->signature == MagickCoreSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if ((splay_tree->root == (NodeInfo *) NULL) ||
@@ -828,7 +828,7 @@ MagickExport const void *GetNextValueInSplayTree(SplayTreeInfo *splay_tree)
     *value;
 
   assert(splay_tree != (SplayTreeInfo *) NULL);
-  assert(splay_tree->signature == MagickSignature);
+  assert(splay_tree->signature == MagickCoreSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if ((splay_tree->root == (NodeInfo *) NULL) ||
@@ -854,6 +854,47 @@ MagickExport const void *GetNextValueInSplayTree(SplayTreeInfo *splay_tree)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t R o o t V a l u e F r o m S p l a y T r e e                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetRootValueFromSplayTree() gets the root value from the splay-tree.
+%
+%  The format of the GetRootValueFromSplayTree method is:
+%
+%      const void *GetRootValueFromSplayTree(SplayTreeInfo *splay_tree)
+%
+%  A description of each parameter follows:
+%
+%    o splay_tree: the splay tree.
+%
+%    o key: the key.
+%
+*/
+MagickExport const void *GetRootValueFromSplayTree(SplayTreeInfo *splay_tree)
+{
+  const void
+    *value;
+
+  assert(splay_tree != (SplayTreeInfo *) NULL);
+  assert(splay_tree->signature == MagickCoreSignature);
+  if (splay_tree->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  value=(const void *) NULL;
+  LockSemaphoreInfo(splay_tree->semaphore);
+  if (splay_tree->root != (NodeInfo *) NULL)
+    value=splay_tree->root->value;
+  UnlockSemaphoreInfo(splay_tree->semaphore);
+  return(value);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t V a l u e F r o m S p l a y T r e e                                 %
 %                                                                             %
 %                                                                             %
@@ -886,7 +927,7 @@ MagickExport const void *GetValueFromSplayTree(SplayTreeInfo *splay_tree,
     *value;
 
   assert(splay_tree != (SplayTreeInfo *) NULL);
-  assert(splay_tree->signature == MagickSignature);
+  assert(splay_tree->signature == MagickCoreSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (splay_tree->root == (NodeInfo *) NULL)
@@ -935,7 +976,7 @@ MagickExport size_t GetNumberOfNodesInSplayTree(
   const SplayTreeInfo *splay_tree)
 {
   assert(splay_tree != (SplayTreeInfo *) NULL);
-  assert(splay_tree->signature == MagickSignature);
+  assert(splay_tree->signature == MagickCoreSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   return(splay_tree->nodes);
@@ -1106,7 +1147,7 @@ MagickExport SplayTreeInfo *NewSplayTree(
   splay_tree=(SplayTreeInfo *) AcquireMagickMemory(sizeof(*splay_tree));
   if (splay_tree == (SplayTreeInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(splay_tree,0,sizeof(*splay_tree));
+  (void) memset(splay_tree,0,sizeof(*splay_tree));
   splay_tree->root=(NodeInfo *) NULL;
   splay_tree->compare=compare;
   splay_tree->relinquish_key=relinquish_key;
@@ -1117,7 +1158,7 @@ MagickExport SplayTreeInfo *NewSplayTree(
   splay_tree->nodes=0;
   splay_tree->debug=IsEventLogging();
   splay_tree->semaphore=AllocateSemaphoreInfo();
-  splay_tree->signature=MagickSignature;
+  splay_tree->signature=MagickCoreSignature;
   return(splay_tree);
 }
 
@@ -1158,7 +1199,7 @@ MagickExport void *RemoveNodeByValueFromSplayTree(SplayTreeInfo *splay_tree,
     *key;
 
   assert(splay_tree != (SplayTreeInfo *) NULL);
-  assert(splay_tree->signature == MagickSignature);
+  assert(splay_tree->signature == MagickCoreSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   key=(void *) NULL;
@@ -1270,7 +1311,7 @@ MagickExport void *RemoveNodeFromSplayTree(SplayTreeInfo *splay_tree,
     *value;
 
   assert(splay_tree != (SplayTreeInfo *) NULL);
-  assert(splay_tree->signature == MagickSignature);
+  assert(splay_tree->signature == MagickCoreSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   value=(void *) NULL;
@@ -1347,7 +1388,7 @@ MagickExport void ResetSplayTree(SplayTreeInfo *splay_tree)
     *pend;
 
   assert(splay_tree != (SplayTreeInfo *) NULL);
-  assert(splay_tree->signature == MagickSignature);
+  assert(splay_tree->signature == MagickCoreSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   LockSemaphoreInfo(splay_tree->semaphore);
@@ -1432,7 +1473,7 @@ MagickExport void ResetSplayTree(SplayTreeInfo *splay_tree)
 MagickExport void ResetSplayTreeIterator(SplayTreeInfo *splay_tree)
 {
   assert(splay_tree != (SplayTreeInfo *) NULL);
-  assert(splay_tree->signature == MagickSignature);
+  assert(splay_tree->signature == MagickCoreSignature);
   if (splay_tree->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   LockSemaphoreInfo(splay_tree->semaphore);
