@@ -17,13 +17,13 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -44,6 +44,7 @@
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/memory_.h"
 #include "magick/string_.h"
 #include "magick/string-private.h"
@@ -98,7 +99,7 @@ MagickExport TokenInfo *AcquireTokenInfo(void)
   token_info=(TokenInfo *) AcquireMagickMemory(sizeof(*token_info));
   if (token_info == (TokenInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  token_info->signature=MagickSignature;
+  token_info->signature=MagickCoreSignature;
   return(token_info);
 }
 
@@ -129,8 +130,8 @@ MagickExport TokenInfo *DestroyTokenInfo(TokenInfo *token_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(token_info != (TokenInfo *) NULL);
-  assert(token_info->signature == MagickSignature);
-  token_info->signature=(~MagickSignature);
+  assert(token_info->signature == MagickCoreSignature);
+  token_info->signature=(~MagickCoreSignature);
   token_info=(TokenInfo *) RelinquishMagickMemory(token_info);
   return(token_info);
 }
@@ -174,15 +175,22 @@ MagickExport void GetNextToken(const char *start,const char **end,
   double
     value;
 
+  register char
+    *q;
+
   register const char
     *p;
 
   register ssize_t
     i;
 
+  size_t
+    length;
+
   assert(start != (const char *) NULL);
   assert(token != (char *) NULL);
   i=0;
+  length=strlen(start);
   p=start;
   while ((isspace((int) ((unsigned char) *p)) != 0) && (*p != '\0'))
     p++;
@@ -218,16 +226,22 @@ MagickExport void GetNextToken(const char *start,const char **end,
             }
         if (i < (ssize_t) (extent-1))
           token[i++]=(*p);
+        if ((size_t) (p-start) >= length)
+          break;
       }
       break;
     }
     case '/':
     {
       if (i < (ssize_t) (extent-1))
-        token[i++]=(*p++);
+        token[i++]=(*p);
+      p++;
       if ((*p == '>') || (*p == '/'))
-        if (i < (ssize_t) (extent-1))
-          token[i++]=(*p++);
+        {
+          if (i < (ssize_t) (extent-1))
+            token[i++]=(*p);
+          p++;
+        }
       break;
     }
     default:
@@ -240,18 +254,26 @@ MagickExport void GetNextToken(const char *start,const char **end,
       if ((p != q) && (*p != ','))
         {
           for ( ; (p < q) && (*p != ','); p++)
+          {
             if (i < (ssize_t) (extent-1))
               token[i++]=(*p);
+            if ((size_t) (p-start) >= length)
+              break;
+          }
           if (*p == '%')
-            if (i < (ssize_t) (extent-1))
-              token[i++]=(*p++);
+            {
+              if (i < (ssize_t) (extent-1))
+                token[i++]=(*p);
+              p++;
+            }
           break;
         }
       if ((*p != '\0') && (isalpha((int) ((unsigned char) *p)) == 0) &&
           (*p != *DirectorySeparator) && (*p != '#') && (*p != '<'))
         {
           if (i < (ssize_t) (extent-1))
-            token[i++]=(*p++);
+            token[i++]=(*p);
+          p++;
           break;
         }
       for ( ; *p != '\0'; p++)
@@ -272,23 +294,21 @@ MagickExport void GetNextToken(const char *start,const char **end,
               token[i++]=(*p);
             if ((*p == ')') && (*(p-1) != '\\'))
               break;
+            if ((size_t) (p-start) >= length)
+              break;
           }
+        if ((size_t) (p-start) >= length)
+          break;
       }
       break;
     }
   }
   token[i]='\0';
-  if (LocaleNCompare(token,"url(",4) == 0)
+  q=strrchr(token,')');
+  if ((LocaleNCompare(token,"url(#",5) == 0) && (q != (char *) NULL))
     {
-      ssize_t
-        offset;
-
-      offset=4;
-      if (token[offset] == '#')
-        offset++;
-      i=(ssize_t) strlen(token);
-      (void) CopyMagickString(token,token+offset,MaxTextExtent);
-      token[i-offset-1]='\0';
+      *q='\0';
+      (void) memmove(token,token+5,(size_t) (q-token-4));
     }
   while (isspace((int) ((unsigned char) *p)) != 0)
     p++;
@@ -384,7 +404,8 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
           status;
 
         status=MagickFalse;
-        pattern+=GetUTFOctets(pattern);
+        while (GetUTFCode(pattern) == '*')
+          pattern+=GetUTFOctets(pattern);
         while ((GetUTFCode(expression) != 0) && (status == MagickFalse))
         {
           status=GlobExpression(expression,pattern,case_insensitive);
@@ -478,65 +499,36 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
       }
       case '{':
       {
-        register const char
+        char
+          *target;
+
+        register char
           *p;
 
-        pattern+=GetUTFOctets(pattern);
+        target=AcquireString(pattern);
+        p=target;
+        pattern++;
         while ((GetUTFCode(pattern) != '}') && (GetUTFCode(pattern) != 0))
         {
-          p=expression;
-          match=MagickTrue;
-          while ((GetUTFCode(p) != 0) && (GetUTFCode(pattern) != 0) &&
-                 (GetUTFCode(pattern) != ',') && (GetUTFCode(pattern) != '}') &&
-                 (match != MagickFalse))
-          {
-            if (GetUTFCode(pattern) == '\\')
+          *p++=(*pattern++);
+          if ((GetUTFCode(pattern) == ',') || (GetUTFCode(pattern) == '}'))
+            {
+              *p='\0';
+              match=GlobExpression(expression,target,case_insensitive);
+              if (match != MagickFalse)
+                {
+                  expression+=MagickMin(strlen(expression),strlen(target));
+                  break;
+                }
+              p=target;
               pattern+=GetUTFOctets(pattern);
-            match=(GetUTFCode(pattern) == GetUTFCode(p)) ? MagickTrue :
-              MagickFalse;
-            p+=GetUTFOctets(p);
-            pattern+=GetUTFOctets(pattern);
-          }
-          if (GetUTFCode(pattern) == 0)
-            {
-              match=MagickFalse;
-              done=MagickTrue;
-              break;
             }
-          if (match != MagickFalse)
-            {
-              expression=p;
-              while ((GetUTFCode(pattern) != '}') &&
-                     (GetUTFCode(pattern) != 0))
-              {
-                pattern+=GetUTFOctets(pattern);
-                if (GetUTFCode(pattern) == '\\')
-                  {
-                    pattern+=GetUTFOctets(pattern);
-                    if (GetUTFCode(pattern) == '}')
-                      pattern+=GetUTFOctets(pattern);
-                  }
-              }
-            }
-          else
-            {
-              while ((GetUTFCode(pattern) != '}') &&
-                     (GetUTFCode(pattern) != ',') &&
-                     (GetUTFCode(pattern) != 0))
-              {
-                pattern+=GetUTFOctets(pattern);
-                if (GetUTFCode(pattern) == '\\')
-                  {
-                    pattern+=GetUTFOctets(pattern);
-                    if ((GetUTFCode(pattern) == '}') ||
-                        (GetUTFCode(pattern) == ','))
-                      pattern+=GetUTFOctets(pattern);
-                  }
-              }
-            }
-            if (GetUTFCode(pattern) != 0)
-              pattern+=GetUTFOctets(pattern);
-          }
+        }
+        while ((GetUTFCode(pattern) != '}') && (GetUTFCode(pattern) != 0))
+          pattern+=GetUTFOctets(pattern);
+        if (GetUTFCode(pattern) != 0)
+          pattern+=GetUTFOctets(pattern);
+        target=DestroyString(target);
         break;
       }
       case '\\':
