@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -102,9 +102,9 @@ MagickExport Image *AutoOrientImage(const Image *image,
     *orient_image;
 
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   orient_image=(Image *) NULL;
   switch(orientation)
   {
@@ -212,11 +212,11 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
     Check chop geometry.
   */
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   assert(chop_info != (RectangleInfo *) NULL);
   if (((chop_info->x+(ssize_t) chop_info->width) < 0) ||
       ((chop_info->y+(ssize_t) chop_info->height) < 0) ||
@@ -250,23 +250,23 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   image_view=AcquireVirtualCacheView(image,exception);
   chop_view=AcquireAuthenticCacheView(chop_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(status) \
-    magick_threads(image,chop_image,1,1)
+  #pragma omp parallel for schedule(static) shared(status) \
+    magick_number_threads(image,chop_image,extent.y,1)
 #endif
   for (y=0; y < (ssize_t) extent.y; y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register IndexPacket
-      *restrict chop_indexes,
-      *restrict indexes;
+      *magick_restrict chop_indexes,
+      *magick_restrict indexes;
 
     register ssize_t
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -313,23 +313,23 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
     Extract chop image.
   */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(status) \
-    magick_threads(image,image,1,1)
+  #pragma omp parallel for schedule(static) shared(status) \
+    magick_number_threads(image,image,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) (image->rows-(extent.y+extent.height)); y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register IndexPacket
-      *restrict chop_indexes,
-      *restrict indexes;
+      *magick_restrict chop_indexes,
+      *magick_restrict indexes;
 
     register ssize_t
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -427,16 +427,15 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     Consolidate separate C, M, Y, and K planes into a single image.
   */
   assert(images != (Image *) NULL);
-  assert(images->signature == MagickSignature);
+  assert(images->signature == MagickCoreSignature);
   if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   cmyk_images=NewImageList();
   for (i=0; i < (ssize_t) GetImageListLength(images); i+=4)
   {
-    cmyk_image=CloneImage(images,images->columns,images->rows,MagickTrue,
-      exception);
+    cmyk_image=CloneImage(images,0,0,MagickTrue,exception);
     if (cmyk_image == (Image *) NULL)
       break;
     if (SetImageStorageClass(cmyk_image,DirectClass) == MagickFalse)
@@ -447,13 +446,13 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     for (y=0; y < (ssize_t) images->rows; y++)
     {
       register const PixelPacket
-        *restrict p;
+        *magick_restrict p;
 
       register ssize_t
         x;
 
       register PixelPacket
-        *restrict q;
+        *magick_restrict q;
 
       p=GetCacheViewVirtualPixels(image_view,0,y,images->columns,1,exception);
       q=QueueCacheViewAuthenticPixels(cmyk_view,0,y,cmyk_image->columns,1,
@@ -479,13 +478,13 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     for (y=0; y < (ssize_t) images->rows; y++)
     {
       register const PixelPacket
-        *restrict p;
+        *magick_restrict p;
 
       register ssize_t
         x;
 
       register PixelPacket
-        *restrict q;
+        *magick_restrict q;
 
       p=GetCacheViewVirtualPixels(image_view,0,y,images->columns,1,exception);
       q=GetCacheViewAuthenticPixels(cmyk_view,0,y,cmyk_image->columns,1,
@@ -511,13 +510,13 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     for (y=0; y < (ssize_t) images->rows; y++)
     {
       register const PixelPacket
-        *restrict p;
+        *magick_restrict p;
 
       register ssize_t
         x;
 
       register PixelPacket
-        *restrict q;
+        *magick_restrict q;
 
       p=GetCacheViewVirtualPixels(image_view,0,y,images->columns,1,exception);
       q=GetCacheViewAuthenticPixels(cmyk_view,0,y,cmyk_image->columns,1,
@@ -543,16 +542,16 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     for (y=0; y < (ssize_t) images->rows; y++)
     {
       register const PixelPacket
-        *restrict p;
+        *magick_restrict p;
 
       register IndexPacket
-        *restrict indexes;
+        *magick_restrict indexes;
 
       register ssize_t
         x;
 
       register PixelPacket
-        *restrict q;
+        *magick_restrict q;
 
       p=GetCacheViewVirtualPixels(image_view,0,y,images->columns,1,exception);
       q=GetCacheViewAuthenticPixels(cmyk_view,0,y,cmyk_image->columns,1,
@@ -638,12 +637,12 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
     Check crop geometry.
   */
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(geometry != (const RectangleInfo *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   bounding_box=image->page;
   if ((bounding_box.width == 0) || (bounding_box.height == 0))
     {
@@ -741,22 +740,22 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   image_view=AcquireVirtualCacheView(image,exception);
   crop_view=AcquireAuthenticCacheView(crop_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(status) \
-    magick_threads(image,crop_image,1,1)
+  #pragma omp parallel for schedule(static) shared(status) \
+    magick_number_threads(image,crop_image,crop_image->rows,1)
 #endif
   for (y=0; y < (ssize_t) crop_image->rows; y++)
   {
     register const IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register IndexPacket
-      *restrict crop_indexes;
+      *magick_restrict crop_indexes;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -771,10 +770,10 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
       }
     indexes=GetCacheViewVirtualIndexQueue(image_view);
     crop_indexes=GetCacheViewAuthenticIndexQueue(crop_view);
-    (void) CopyMagickMemory(q,p,(size_t) crop_image->columns*sizeof(*p));
+    (void) memcpy(q,p,(size_t) crop_image->columns*sizeof(*p));
     if ((indexes != (IndexPacket *) NULL) &&
         (crop_indexes != (IndexPacket *) NULL))
-      (void) CopyMagickMemory(crop_indexes,indexes,(size_t) crop_image->columns*
+      (void) memcpy(crop_indexes,indexes,(size_t) crop_image->columns*
         sizeof(*crop_indexes));
     if (SyncCacheViewAuthenticPixels(crop_view,exception) == MagickFalse)
       status=MagickFalse;
@@ -851,7 +850,7 @@ MagickExport Image *CropImageToTiles(const Image *image,
     geometry;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   crop_image=NewImageList();
@@ -936,12 +935,9 @@ MagickExport Image *CropImageToTiles(const Image *image,
           crop.width-=crop.x;
           crop.x+=image->page.x;
           next=CropImage(image,&crop,exception);
-          if (next == (Image *) NULL)
-            break;
-          AppendImageToList(&crop_image,next);
+          if (next != (Image *) NULL)
+            AppendImageToList(&crop_image,next);
         }
-        if (next == (Image *) NULL)
-          break;
       }
       ClearMagickException(exception);
       return(crop_image);
@@ -1064,12 +1060,12 @@ MagickExport Image *ExcerptImage(const Image *image,
     Allocate excerpt image.
   */
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(geometry != (const RectangleInfo *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   excerpt_image=CloneImage(image,geometry->width,geometry->height,MagickTrue,
     exception);
   if (excerpt_image == (Image *) NULL)
@@ -1082,20 +1078,20 @@ MagickExport Image *ExcerptImage(const Image *image,
   image_view=AcquireVirtualCacheView(image,exception);
   excerpt_view=AcquireAuthenticCacheView(excerpt_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(image,excerpt_image,excerpt_image->rows,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(image,excerpt_image,excerpt_image->rows,1)
 #endif
   for (y=0; y < (ssize_t) excerpt_image->rows; y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register IndexPacket
-      *restrict excerpt_indexes,
-      *restrict indexes;
+      *magick_restrict excerpt_indexes,
+      *magick_restrict indexes;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -1108,13 +1104,13 @@ MagickExport Image *ExcerptImage(const Image *image,
         status=MagickFalse;
         continue;
       }
-    (void) CopyMagickMemory(q,p,(size_t) excerpt_image->columns*sizeof(*q));
+    (void) memcpy(q,p,(size_t) excerpt_image->columns*sizeof(*q));
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
     if (indexes != (IndexPacket *) NULL)
       {
         excerpt_indexes=GetCacheViewAuthenticIndexQueue(excerpt_view);
         if (excerpt_indexes != (IndexPacket *) NULL)
-          (void) CopyMagickMemory(excerpt_indexes,indexes,(size_t)
+          (void) memcpy(excerpt_indexes,indexes,(size_t)
             excerpt_image->columns*sizeof(*excerpt_indexes));
       }
     if (SyncCacheViewAuthenticPixels(excerpt_view,exception) == MagickFalse)
@@ -1176,23 +1172,38 @@ MagickExport Image *ExtentImage(const Image *image,
   Image
     *extent_image;
 
+  MagickBooleanType
+    status;
+
   /*
     Allocate extent image.
   */
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(geometry != (const RectangleInfo *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   extent_image=CloneImage(image,geometry->width,geometry->height,MagickTrue,
     exception);
   if (extent_image == (Image *) NULL)
     return((Image *) NULL);
-  (void) SetImageBackgroundColor(extent_image);
-  (void) CompositeImage(extent_image,image->compose,image,-geometry->x,
+  status=SetImageBackgroundColor(extent_image);
+  if (status == MagickFalse)
+    {
+      InheritException(exception,&extent_image->exception);
+      extent_image=DestroyImage(extent_image);
+      return((Image *) NULL);
+    }
+  status=CompositeImage(extent_image,image->compose,image,-geometry->x,
     -geometry->y);
+  if (status == MagickFalse)
+    {
+      InheritException(exception,&extent_image->exception);
+      extent_image=DestroyImage(extent_image);
+      return((Image *) NULL);
+    }
   return(extent_image);
 }
 
@@ -1245,12 +1256,12 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
     y;
 
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  flip_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
+  assert(exception->signature == MagickCoreSignature);
+  flip_image=CloneImage(image,0,0,MagickTrue,exception);
   if (flip_image == (Image *) NULL)
     return((Image *) NULL);
   /*
@@ -1262,22 +1273,22 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
   image_view=AcquireVirtualCacheView(image,exception);
   flip_view=AcquireAuthenticCacheView(flip_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(status) \
-    magick_threads(image,flip_image,1,1)
+  #pragma omp parallel for schedule(static) shared(status) \
+    magick_number_threads(image,flip_image,flip_image->rows,1)
 #endif
   for (y=0; y < (ssize_t) flip_image->rows; y++)
   {
     register const IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register IndexPacket
-      *restrict flip_indexes;
+      *magick_restrict flip_indexes;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -1289,13 +1300,13 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
         status=MagickFalse;
         continue;
       }
-    (void) CopyMagickMemory(q,p,(size_t) image->columns*sizeof(*q));
+    (void) memcpy(q,p,(size_t) image->columns*sizeof(*q));
     indexes=GetCacheViewVirtualIndexQueue(image_view);
     if (indexes != (const IndexPacket *) NULL)
       {
         flip_indexes=GetCacheViewAuthenticIndexQueue(flip_view);
         if (flip_indexes != (IndexPacket *) NULL)
-          (void) CopyMagickMemory(flip_indexes,indexes,(size_t) image->columns*
+          (void) memcpy(flip_indexes,indexes,(size_t) image->columns*
             sizeof(*flip_indexes));
       }
     if (SyncCacheViewAuthenticPixels(flip_view,exception) == MagickFalse)
@@ -1373,12 +1384,12 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
     y;
 
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  flop_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
+  assert(exception->signature == MagickCoreSignature);
+  flop_image=CloneImage(image,0,0,MagickTrue,exception);
   if (flop_image == (Image *) NULL)
     return((Image *) NULL);
   /*
@@ -1390,25 +1401,25 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
   image_view=AcquireVirtualCacheView(image,exception);
   flop_view=AcquireAuthenticCacheView(flop_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(status) \
-    magick_threads(image,flop_image,1,1)
+  #pragma omp parallel for schedule(static) shared(status) \
+    magick_number_threads(image,flop_image,flop_image->rows,1)
 #endif
   for (y=0; y < (ssize_t) flop_image->rows; y++)
   {
     register const IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register IndexPacket
-      *restrict flop_indexes;
+      *magick_restrict flop_indexes;
 
     register ssize_t
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -1487,10 +1498,8 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
 %
 */
 
-static inline MagickBooleanType CopyImageRegion(Image *destination,
-  const Image *source,const size_t columns,const size_t rows,
-  const ssize_t sx,const ssize_t sy,const ssize_t dx,const ssize_t dy,
-  ExceptionInfo *exception)
+static MagickBooleanType CopyImageRegion(Image *destination,const Image *source,  const size_t columns,const size_t rows,const ssize_t sx,const ssize_t sy,
+  const ssize_t dx,const ssize_t dy,ExceptionInfo *exception)
 {
   CacheView
     *source_view,
@@ -1508,8 +1517,8 @@ static inline MagickBooleanType CopyImageRegion(Image *destination,
   source_view=AcquireVirtualCacheView(source,exception);
   destination_view=AcquireAuthenticCacheView(destination,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(status) \
-    magick_threads(source,destination,rows,1)
+  #pragma omp parallel for schedule(static) shared(status) \
+    magick_number_threads(source,destination,rows,1)
 #endif
   for (y=0; y < (ssize_t) rows; y++)
   {
@@ -1517,16 +1526,16 @@ static inline MagickBooleanType CopyImageRegion(Image *destination,
       sync;
 
     register const IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register IndexPacket
-      *restrict destination_indexes;
+      *magick_restrict destination_indexes;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     /*
       Transfer scanline.
@@ -1541,12 +1550,12 @@ static inline MagickBooleanType CopyImageRegion(Image *destination,
         continue;
       }
     indexes=GetCacheViewVirtualIndexQueue(source_view);
-    (void) CopyMagickMemory(q,p,(size_t) columns*sizeof(*p));
+    (void) memcpy(q,p,(size_t) columns*sizeof(*p));
     if (indexes != (IndexPacket *) NULL)
       {
         destination_indexes=GetCacheViewAuthenticIndexQueue(destination_view);
         if (destination_indexes != (IndexPacket *) NULL)
-          (void) CopyMagickMemory(destination_indexes,indexes,(size_t)
+          (void) memcpy(destination_indexes,indexes,(size_t)
             columns*sizeof(*indexes));
       }
     sync=SyncCacheViewAuthenticPixels(destination_view,exception);
@@ -1576,12 +1585,12 @@ MagickExport Image *RollImage(const Image *image,const ssize_t x_offset,
     Initialize roll image attributes.
   */
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  roll_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
+  assert(exception->signature == MagickCoreSignature);
+  roll_image=CloneImage(image,0,0,MagickTrue,exception);
   if (roll_image == (Image *) NULL)
     return((Image *) NULL);
   offset.x=x_offset;
@@ -1661,7 +1670,7 @@ MagickExport Image *ShaveImage(const Image *image,
     geometry;
 
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (((2*shave_info->width) >= image->columns) ||
@@ -1733,18 +1742,19 @@ MagickExport Image *SpliceImage(const Image *image,
     splice_geometry;
 
   ssize_t
+    columns,
     y;
 
   /*
     Allocate splice image.
   */
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(geometry != (const RectangleInfo *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   splice_geometry=(*geometry);
   splice_image=CloneImage(image,image->columns+splice_geometry.width,
     image->rows+splice_geometry.height,MagickTrue,exception);
@@ -1817,30 +1827,32 @@ MagickExport Image *SpliceImage(const Image *image,
   */
   status=MagickTrue;
   progress=0;
+  columns=MagickMin(splice_geometry.x,(ssize_t) splice_image->columns);
   image_view=AcquireVirtualCacheView(image,exception);
   splice_view=AcquireAuthenticCacheView(splice_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(image,splice_image,1,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(image,splice_image,splice_geometry.y,1)
 #endif
   for (y=0; y < (ssize_t) splice_geometry.y; y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register IndexPacket
-      *restrict indexes,
-      *restrict splice_indexes;
+      *magick_restrict indexes,
+      *magick_restrict splice_indexes;
 
     register ssize_t
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
-    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,splice_image->columns,1,
+      exception);
     q=QueueCacheViewAuthenticPixels(splice_view,0,y,splice_image->columns,1,
       exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
@@ -1850,7 +1862,7 @@ MagickExport Image *SpliceImage(const Image *image,
       }
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
     splice_indexes=GetCacheViewAuthenticIndexQueue(splice_view);
-    for (x=0; x < splice_geometry.x; x++)
+    for (x=0; x < columns; x++)
     {
       SetPixelRed(q,GetPixelRed(p));
       SetPixelGreen(q,GetPixelGreen(p));
@@ -1897,31 +1909,31 @@ MagickExport Image *SpliceImage(const Image *image,
       }
   }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(image,splice_image,1,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(image,splice_image,splice_image->rows,1)
 #endif
   for (y=(ssize_t) (splice_geometry.y+splice_geometry.height);
        y < (ssize_t) splice_image->rows; y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register IndexPacket
-      *restrict indexes,
-      *restrict splice_indexes;
+      *magick_restrict indexes,
+      *magick_restrict splice_indexes;
 
     register ssize_t
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
-    p=GetCacheViewVirtualPixels(image_view,0,y-(ssize_t) splice_geometry.height,
-      image->columns,1,exception);
-    if ((y < 0) || (y >= (ssize_t) splice_image->rows))
+    if ((y < 0) || (y >= (ssize_t)splice_image->rows))
       continue;
+    p=GetCacheViewVirtualPixels(image_view,0,y-(ssize_t) splice_geometry.height,
+      splice_image->columns,1,exception);
     q=QueueCacheViewAuthenticPixels(splice_view,0,y,splice_image->columns,1,
       exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
@@ -1931,7 +1943,7 @@ MagickExport Image *SpliceImage(const Image *image,
       }
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
     splice_indexes=GetCacheViewAuthenticIndexQueue(splice_view);
-    for (x=0; x < splice_geometry.x; x++)
+    for (x=0; x < columns; x++)
     {
       SetPixelRed(q,GetPixelRed(p));
       SetPixelGreen(q,GetPixelGreen(p));
@@ -2045,7 +2057,7 @@ MagickExport MagickBooleanType TransformImage(Image **image,
     geometry;
 
   assert(image != (Image **) NULL);
-  assert((*image)->signature == MagickSignature);
+  assert((*image)->signature == MagickCoreSignature);
   if ((*image)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",(*image)->filename);
   transform_image=(*image);
@@ -2133,7 +2145,7 @@ MagickExport MagickBooleanType TransformImages(Image **images,
     i;
 
   assert(images != (Image **) NULL);
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
   if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       (*images)->filename);
@@ -2202,11 +2214,11 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
     y;
 
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   transpose_image=CloneImage(image,image->rows,image->columns,MagickTrue,
     exception);
   if (transpose_image == (Image *) NULL)
@@ -2219,20 +2231,20 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
   image_view=AcquireVirtualCacheView(image,exception);
   transpose_view=AcquireAuthenticCacheView(transpose_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(image,transpose_image,image->rows,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(image,transpose_image,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register IndexPacket
-      *restrict transpose_indexes,
-      *restrict indexes;
+      *magick_restrict transpose_indexes,
+      *magick_restrict indexes;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -2245,13 +2257,13 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
         status=MagickFalse;
         continue;
       }
-    (void) CopyMagickMemory(q,p,(size_t) image->columns*sizeof(*q));
+    (void) memcpy(q,p,(size_t) image->columns*sizeof(*q));
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
     if (indexes != (IndexPacket *) NULL)
       {
         transpose_indexes=GetCacheViewAuthenticIndexQueue(transpose_view);
         if (transpose_indexes != (IndexPacket *) NULL)
-          (void) CopyMagickMemory(transpose_indexes,indexes,(size_t)
+          (void) memcpy(transpose_indexes,indexes,(size_t)
             image->columns*sizeof(*transpose_indexes));
       }
     if (SyncCacheViewAuthenticPixels(transpose_view,exception) == MagickFalse)
@@ -2331,11 +2343,11 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
     y;
 
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   transverse_image=CloneImage(image,image->rows,image->columns,MagickTrue,
     exception);
   if (transverse_image == (Image *) NULL)
@@ -2348,8 +2360,8 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
   image_view=AcquireVirtualCacheView(image,exception);
   transverse_view=AcquireAuthenticCacheView(transverse_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(image,transverse_image,image->rows,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(image,transverse_image,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
@@ -2357,17 +2369,17 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
       sync;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register IndexPacket
-      *restrict transverse_indexes,
-      *restrict indexes;
+      *magick_restrict transverse_indexes,
+      *magick_restrict indexes;
 
     register ssize_t
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -2456,7 +2468,7 @@ MagickExport Image *TrimImage(const Image *image,ExceptionInfo *exception)
     geometry;
 
   assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   geometry=GetImageBoundingBox(image,exception);
