@@ -17,13 +17,13 @@
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -133,12 +133,15 @@ MagickExport const char *GetMagickDelegates(void)
 #if defined(MAGICKCORE_FREETYPE_DELEGATE)
   "freetype "
 #endif
-#if defined(MAGICKCORE_GS_DELEGATE)
+#if defined(MAGICKCORE_GS_DELEGATE) || defined(MAGICKCORE_WINDOWS_SUPPORT)
   "gslib "
 #endif
 #if defined(MAGICKCORE_GVC_DELEGATE)
   "gvc "
 #endif
+#if defined(MAGICKCORE_HEIC_DELEGATE)
+  "heic "
+#endif
 #if defined(MAGICKCORE_JBIG_DELEGATE)
   "jbig "
 #endif
@@ -172,9 +175,13 @@ MagickExport const char *GetMagickDelegates(void)
 #if defined(MAGICKCORE_PNG_DELEGATE)
   "png "
 #endif
-#if defined(MAGICKCORE_DPS_DELEGATE) || defined(MAGICKCORE_GS_DELEGATE) || defined(WIN32)
+#if defined(MAGICKCORE_DPS_DELEGATE) || defined(MAGICKCORE_GS_DELEGATE) || \
+    defined(MAGICKCORE_WINDOWS_SUPPORT)
   "ps "
 #endif
+#if defined(MAGICKCORE_RAW_R_DELEGATE)
+  "raw "
+#endif
 #if defined(MAGICKCORE_RSVG_DELEGATE)
   "rsvg "
 #endif
@@ -444,13 +451,13 @@ MagickExport const char *GetMagickReleaseDate(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetMagickSignature() returns a signature that uniquely encodes the
+%  GetMagickCoreSignature() returns a signature that uniquely encodes the
 %  MagickCore libary version, quantum depth, HDRI status, OS word size, and
 %  endianness.
 %
-%  The format of the GetMagickSignature method is:
+%  The format of the GetMagickCoreSignature method is:
 %
-%      unsigned int GetMagickSignature(const StringInfo *nonce)
+%      unsigned int GetMagickCoreSignature(const StringInfo *nonce)
 %
 %  A description of each parameter follows:
 %
@@ -501,7 +508,7 @@ static unsigned int CRC32(const unsigned char *message,const size_t length)
   return(crc ^ 0xFFFFFFFF);
 }
 
-MagickExport unsigned int GetMagickSignature(const StringInfo *nonce)
+MagickExport unsigned int GetMagickCoreSignature(const StringInfo *nonce)
 {
   register unsigned char
     *p;
@@ -600,4 +607,21 @@ MagickExport void ListMagickVersion(FILE *file)
   (void) FormatLocaleFile(file,"Features: %s\n",GetMagickFeatures());
   (void) FormatLocaleFile(file,"Delegates (built-in): %s\n",
     GetMagickDelegates());
+  if (IsEventLogging() != MagickFalse)
+    {
+      (void) FormatLocaleFile(file,"Wizard attributes: ");
+      (void) FormatLocaleFile(file,"QuantumRange=%g; ",(double) QuantumRange);
+      (void) FormatLocaleFile(file,"QuantumScale=%.*g; ",GetMagickPrecision(),
+        (double) QuantumScale);
+      (void) FormatLocaleFile(file,"MagickEpsilon=%.*g; ",GetMagickPrecision(),
+        (double) MagickEpsilon);
+      (void) FormatLocaleFile(file,"MaxMap=%g; ",(double) MaxMap);
+      (void) FormatLocaleFile(file,"MaxTextExtent=%g; ",(double) MaxTextExtent);
+      (void) FormatLocaleFile(file,"sizeof(Quantum)=%g; ",(double)
+        sizeof(Quantum));
+      (void) FormatLocaleFile(file,"sizeof(MagickSizeType)=%g; ",(double)
+        sizeof(MagickSizeType));
+      (void) FormatLocaleFile(file,"sizeof(MagickOffsetType)=%g",(double)
+        sizeof(MagickOffsetType));
+    }
 }
