@@ -17,13 +17,13 @@
 %                               September 2014                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -188,13 +188,12 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
     Initialize connected components image attributes.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  component_image=CloneImage(image,image->columns,image->rows,MagickTrue,
-    exception);
+  assert(exception->signature == MagickCoreSignature);
+  component_image=CloneImage(image,0,0,MagickTrue,exception);
   if (component_image == (Image *) NULL)
     return((Image *) NULL);
   component_image->depth=MAGICKCORE_QUANTUM_DEPTH;
@@ -227,7 +226,7 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
       component_image=DestroyImage(component_image);
       ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
     }
-  (void) ResetMagickMemory(object,0,MaxColormapSize*sizeof(*object));
+  (void) memset(object,0,MaxColormapSize*sizeof(*object));
   for (i=0; i < (ssize_t) MaxColormapSize; i++)
   {
     object[i].id=i;
@@ -367,8 +366,7 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
     p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     q=QueueCacheViewAuthenticPixels(component_view,0,y,component_image->columns,
       1,exception);
-    if ((p == (const PixelPacket *) NULL) ||
-        (q == (PixelPacket *) NULL))
+    if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
@@ -383,33 +381,30 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
 
       offset=y*image->columns+x;
       status=GetMatrixElement(equivalences,offset,0,&id);
-      if (id == offset)
-        {
-          id=n++;
-          if (n > (ssize_t) MaxColormapSize)
-            break;
-          status=SetMatrixElement(equivalences,offset,0,&id);
-        }
+      if (id != offset)
+        status=GetMatrixElement(equivalences,id,0,&id);
       else
         {
-          status=GetMatrixElement(equivalences,id,0,&id);
-          status=SetMatrixElement(equivalences,offset,0,&id);
+          id=n++;
+          if (id >= (ssize_t) MaxColormapSize)
+            break;
         }
+      status=SetMatrixElement(equivalences,offset,0,&id);
       if (x < object[id].bounding_box.x)
         object[id].bounding_box.x=x;
-      if (x > (ssize_t) object[id].bounding_box.width)
+      if (x >= (ssize_t) object[id].bounding_box.width)
         object[id].bounding_box.width=(size_t) x;
       if (y < object[id].bounding_box.y)
         object[id].bounding_box.y=y;
-      if (y > (ssize_t) object[id].bounding_box.height)
+      if (y >=(ssize_t) object[id].bounding_box.height)
         object[id].bounding_box.height=(size_t) y;
-      object[id].color.red+=p->red;
-      object[id].color.green+=p->green;
-      object[id].color.blue+=p->blue;
+      object[id].color.red+=QuantumScale*p->red;
+      object[id].color.green+=QuantumScale*p->green;
+      object[id].color.blue+=QuantumScale*p->blue;
       if (image->matte != MagickFalse)
-        object[id].color.opacity+=p->opacity;
+        object[id].color.opacity+=QuantumScale*p->opacity;
       if (image->colorspace == CMYKColorspace)
-        object[id].color.index+=indexes[x];
+        object[id].color.index+=QuantumScale*indexes[x];
       object[id].centroid.x+=x;
       object[id].centroid.y+=y;
       object[id].area++;
@@ -446,13 +441,14 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
   {
     object[i].bounding_box.width-=(object[i].bounding_box.x-1);
     object[i].bounding_box.height-=(object[i].bounding_box.y-1);
-    object[i].color.red=object[i].color.red/object[i].area;
-    object[i].color.green=object[i].color.green/object[i].area;
-    object[i].color.blue=object[i].color.blue/object[i].area;
+    object[i].color.red=QuantumRange*(object[i].color.red/object[i].area);
+    object[i].color.green=QuantumRange*(object[i].color.green/object[i].area);
+    object[i].color.blue=QuantumRange*(object[i].color.blue/object[i].area);
     if (image->matte != MagickFalse)
-      object[i].color.opacity=object[i].color.opacity/object[i].area;
+      object[i].color.opacity=QuantumRange*(object[i].color.opacity/
+        object[i].area);
     if (image->colorspace == CMYKColorspace)
-      object[i].color.index=object[i].color.index/object[i].area;
+      object[i].color.index=QuantumRange*(object[i].color.index/object[i].area);
     object[i].centroid.x=object[i].centroid.x/object[i].area;
     object[i].centroid.y=object[i].centroid.y/object[i].area;
   }
@@ -482,7 +478,7 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
 
         if (status == MagickFalse)
           continue;
-        if ((double) object[i].area > area_threshold)
+        if ((double) object[i].area >= area_threshold)
           continue;
         for (j=0; j < (ssize_t) component_image->colors; j++)
           object[j].census=0;
@@ -585,19 +581,20 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
       {
         while ((isspace((int) ((unsigned char) *c)) != 0) || (*c == ','))
           c++;
-        first=strtol(c,&c,10);
+        first=(ssize_t) strtol(c,&c,10);
         if (first < 0)
-          first+=(long) component_image->colors;
+          first+=(ssize_t) component_image->colors;
         last=first;
         while (isspace((int) ((unsigned char) *c)) != 0)
           c++;
         if (*c == '-')
           {
-            last=strtol(c+1,&c,10);
+            last=(ssize_t) strtol(c+1,&c,10);
             if (last < 0)
-              last+=(long) component_image->colors;
+              last+=(ssize_t) component_image->colors;
           }
-        for (step=first > last ? -1 : 1; first != (last+step); first+=step)
+        step=(ssize_t) (first > last ? -1 : 1);
+        for ( ; first != (last+step); first+=step)
           object[first].census++;
       }
       for (i=0; i < (ssize_t) component_image->colors; i++)
@@ -618,19 +615,20 @@ MagickExport Image *ConnectedComponentsImage(const Image *image,
       {
         while ((isspace((int) ((unsigned char) *c)) != 0) || (*c == ','))
           c++;
-        first=strtol(c,&c,10);
+        first=(ssize_t) strtol(c,&c,10);
         if (first < 0)
-          first+=(long) component_image->colors;
+          first+=(ssize_t) component_image->colors;
         last=first;
         while (isspace((int) ((unsigned char) *c)) != 0)
           c++;
         if (*c == '-')
           {
-            last=strtol(c+1,&c,10);
+            last=(ssize_t) strtol(c+1,&c,10);
             if (last < 0)
-              last+=(long) component_image->colors;
+              last+=(ssize_t) component_image->colors;
           }
-        for (step=first > last ? -1 : 1; first != (last+step); first+=step)
+        step=(ssize_t) (first > last ? -1 : 1);
+        for ( ; first != (last+step); first+=step)
         {
           component_image->matte=MagickTrue;
           component_image->colormap[first].opacity=TransparentOpacity;
