@@ -17,13 +17,13 @@
 %                                July 1992                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -116,7 +116,6 @@ static MagickBooleanType AnimateUsage(void)
       "-crop geometry       preferred size and location of the cropped image",
       "-extract geometry    extract area from image",
       "-monochrome          transform image to black and white",
-      "-repage geometry     size and location of an image canvas (operator)",
       "-resample geometry   change the resolution of an image",
       "-resize geometry     resize the image",
       "-rotate degrees      apply Paeth rotation to the image",
@@ -164,6 +163,7 @@ static MagickBooleanType AnimateUsage(void)
       "-quiet               suppress all warning messages",
       "-regard-warnings     pay attention to warning messages",
       "-remote command      execute a command in an remote display process",
+      "-repage geometry     size and location of an image canvas (operator)",
       "-respect-parentheses settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
@@ -305,7 +305,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
     Set defaults.
   */
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(exception != (ExceptionInfo *) NULL);
@@ -329,7 +329,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
   pend=MagickFalse;
   respect_parenthesis=MagickFalse;
   resource_database=(XrmDatabase) NULL;
-  (void) ResetMagickMemory(&resource_info,0,sizeof(XResourceInfo));
+  (void) memset(&resource_info,0,sizeof(XResourceInfo));
   server_name=(char *) NULL;
   status=MagickTrue;
   /*
@@ -346,6 +346,8 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
       Check command line for server name.
     */
     option=argv[i];
+    if (IsCommandOption(option) == MagickFalse)
+      continue;
     if (LocaleCompare("display",option+1) == 0)
       {
         /*
@@ -1015,7 +1017,7 @@ WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
             status=MogrifyImageInfo(image_info,(int) (i-j+1),(const char **)
               argv+j,exception);
             DestroyAnimate();
-            return(status == 0 ? MagickTrue : MagickFalse);
+            return(status == 0 ? MagickFalse : MagickTrue);
           }
         if (LocaleCompare("log",option+1) == 0)
           {
