@@ -17,13 +17,13 @@
 %                               December 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -94,12 +94,34 @@ static MagickBooleanType CompareUsage(void)
       "-log format          format of debugging information",
       (char *) NULL
     },
+    *operators[]=
+    {
+      "-brightness-contrast geometry",
+      "                     improve brightness / contrast of the image",
+      "-distort method args",
+      "                     distort images according to given method and args",
+      "-level value         adjust the level of image contrast",
+      "-resize geometry     resize the image",
+      "-rotate degrees      apply Paeth rotation to the image",
+      "-sigmoidal-contrast geometry",
+      "                     increase the contrast without saturating highlights or",
+      "-trim                trim image edges",
+      (char *) NULL
+    },
+    *sequence_operators[]=
+    {
+      "-crop geometry       cut out a rectangular region of the image",
+      "-separate            separate an image channel into a grayscale image",
+      "-write filename      write images to this file",
+      (char *) NULL
+    },
     *settings[]=
     {
       "-alpha option        on, activate, off, deactivate, set, opaque, copy",
       "                     transparent, extract, background, or shape",
       "-authenticate password",
       "                     decipher image with this password",
+      "-background color    background color",
       "-channel type        apply option to select image channels",
       "-colorspace type     alternate image colorspace",
       "-compose operator    set image composite operator",
@@ -115,6 +137,7 @@ static MagickBooleanType CompareUsage(void)
       "-extract geometry    extract area from image",
       "-format \"string\"     output formatted image characteristics",
       "-fuzz distance       colors within this distance are considered equal",
+      "-gravity type        horizontal and vertical text placement",
       "-highlight-color color",
       "                     empasize pixel differences with this color",
       "-identify            identify the format and characteristics of the image",
@@ -131,6 +154,7 @@ static MagickBooleanType CompareUsage(void)
       "-quiet               suppress all warning messages",
       "-quantize colorspace reduce colors in this colorspace",
       "-regard-warnings     pay attention to warning messages",
+      "-repage geometry     size and location of an image canvas",
       "-respect-parentheses settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
@@ -151,6 +175,11 @@ static MagickBooleanType CompareUsage(void)
       "-virtual-pixel method",
       "                     virtual pixel access method",
       (char *) NULL
+    },
+    *stack_operators[]=
+    {
+      "-delete indexes      delete the image from the image sequence",
+      (char *) NULL
     };
 
   ListMagickVersion(stdout);
@@ -159,6 +188,15 @@ static MagickBooleanType CompareUsage(void)
   (void) printf("\nImage Settings:\n");
   for (p=settings; *p != (char *) NULL; p++)
     (void) printf("  %s\n",*p);
+  (void) printf("\nImage Operators:\n");
+  for (p=operators; *p != (char *) NULL; p++)
+    (void) printf("  %s\n",*p);
+  (void) printf("\nImage Sequence Operators:\n");
+  for (p=sequence_operators; *p != (char *) NULL; p++)
+    (void) printf("  %s\n",*p);
+  (void) printf("\nImage Stack Operators:\n");
+  for (p=stack_operators; *p != (char *) NULL; p++)
+    (void) printf("  %s\n",*p);
   (void) printf("\nMiscellaneous Options:\n");
   for (p=miscellaneous; *p != (char *) NULL; p++)
     (void) printf("  %s\n",*p);
@@ -261,7 +299,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
     Set defaults.
   */
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(exception != (ExceptionInfo *) NULL);
@@ -278,7 +316,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   if (argc < 3)
     return(CompareUsage());
   restore_info=image_info;
-  channels=CompositeChannels;
+  channels=DefaultChannels;
   difference_image=NewImageList();
   similarity_image=NewImageList();
   dissimilarity_threshold=DefaultDissimilarityThreshold;
@@ -377,6 +415,28 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           }
         ThrowCompareException(OptionError,"UnrecognizedOption",option);
       }
+      case 'b':
+      {
+        if (LocaleCompare("background",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            break;
+          }
+        if (LocaleCompare("brightness-contrast",option+1) == 0)
+          {
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            break;
+          }
+        ThrowCompareException(OptionError,"UnrecognizedOption",option);
+      }
       case 'c':
       {
         if (LocaleCompare("cache",option+1) == 0)
@@ -460,6 +520,17 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("concurrent",option+1) == 0)
           break;
+        if (LocaleCompare("crop",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            break;
+          }
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
       case 'd':
@@ -506,6 +577,17 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               }
             break;
           }
+        if (LocaleCompare("delete",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsSceneGeometry(argv[i],MagickFalse) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("density",option+1) == 0)
           {
             if (*option == '+')
@@ -543,6 +625,23 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               dissimilarity_threshold=StringToDouble(argv[i],(char **) NULL);
             break;
           }
+        if (LocaleCompare("distort",option+1) == 0)
+          {
+            ssize_t
+              op;
+
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            op=ParseCommandOption(MagickDistortOptions,MagickFalse,argv[i]);
+            if (op < 0)
+              ThrowCompareException(OptionError,"UnrecognizedDistortMethod",
+                argv[i]);
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            break;
+          }
         if (LocaleCompare("duration",option+1) == 0)
           {
             if (*option == '+')
@@ -605,6 +704,27 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           }
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
+      case 'g':
+      {
+        if (LocaleCompare("gravity",option+1) == 0)
+          {
+            ssize_t
+              gravity;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            gravity=ParseCommandOption(MagickGravityOptions,MagickFalse,
+              argv[i]);
+            if (gravity < 0)
+              ThrowCompareException(OptionError,"UnrecognizedGravityType",
+                argv[i]);
+            break;
+          }
+        ThrowCompareException(OptionError,"UnrecognizedOption",option)
+      }
       case 'h':
       {
         if ((LocaleCompare("help",option+1) == 0) ||
@@ -646,6 +766,15 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       }
       case 'l':
       {
+        if (LocaleCompare("level",option+1) == 0)
+          {
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("limit",option+1) == 0)
           {
             char
@@ -692,7 +821,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
             status=MogrifyImageInfo(image_info,(int) (i-j+1),(const char **)
               argv+j,exception);
             DestroyCompare();
-            return(status == 0 ? MagickTrue : MagickFalse);
+            return(status == 0 ? MagickFalse : MagickTrue);
           }
         if (LocaleCompare("log",option+1) == 0)
           {
@@ -806,6 +935,37 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("regard-warnings",option+1) == 0)
           break;
+        if (LocaleCompare("repage",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            break;
+          }
+        if (LocaleCompare("resize",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            break;
+          }
+        if (LocaleCompare("rotate",option+1) == 0)
+          {
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleNCompare("respect-parentheses",option+1,17) == 0)
           {
             respect_parenthesis=(*option == '-') ? MagickTrue : MagickFalse;
@@ -837,6 +997,8 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               ThrowCompareInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("separate",option+1) == 0)
+          break;
         if (LocaleCompare("set",option+1) == 0)
           {
             i++;
@@ -849,6 +1011,15 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               ThrowCompareException(OptionError,"MissingArgument",option);
             break;
           }
+        if (LocaleCompare("sigmoidal-contrast",option+1) == 0)
+          {
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompareInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("similarity-threshold",option+1) == 0)
           {
             if (*option == '+')
@@ -902,6 +1073,8 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               ThrowCompareException(OptionError,"MissingArgument",option);
             break;
           }
+        if (LocaleCompare("trim",option+1) == 0)
+          break;
         if (LocaleCompare("type",option+1) == 0)
           {
             ssize_t
@@ -949,6 +1122,17 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           }
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
+      case 'w':
+      {
+        if (LocaleCompare("write",option+1) == 0)
+          {
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowCompareException(OptionError,"MissingArgument",option);
+            break;
+          }
+        ThrowCompareException(OptionError,"UnrecognizedOption",option)
+      }
       case '?':
         break;
       default:
@@ -991,14 +1175,8 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       metric,&distortion,exception);
   else
     if (similarity_image == (Image *) NULL)
-      {
-        if (metric == PerceptualHashErrorMetric)
-          difference_image=CompareImageChannels(image,reconstruct_image,
-            channels,metric,&distortion,exception);
-        else
-          ThrowCompareException(OptionError,"ImageWidthsOrHeightsDiffer",
-            image->filename);
-      }
+      difference_image=CompareImageChannels(image,reconstruct_image,channels,
+        metric,&distortion,exception);
     else
       {
         Image
@@ -1159,6 +1337,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
                       channel_distortion[OpacityChannel]);
                   break;
                 }
+                case LinearGRAYColorspace:
                 case GRAYColorspace:
                 {
                   (void) FormatLocaleFile(stderr,"    gray: %g (%g)\n",
@@ -1212,6 +1391,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
                       channel_distortion[OpacityChannel]);
                   break;
                 }
+                case LinearGRAYColorspace:
                 case GRAYColorspace:
                 {
                   (void) FormatLocaleFile(stderr,"    gray: %g\n",
