@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -115,9 +115,9 @@ static MagickBooleanType CompositeImageList(ImageInfo *image_info,Image **image,
     status;
 
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image **) NULL);
-  assert((*image)->signature == MagickSignature);
+  assert((*image)->signature == MagickCoreSignature);
   if ((*image)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",(*image)->filename);
   assert(exception != (ExceptionInfo *) NULL);
@@ -125,7 +125,7 @@ static MagickBooleanType CompositeImageList(ImageInfo *image_info,Image **image,
   status=MagickTrue;
   if (composite_image != (Image *) NULL)
     {
-      assert(composite_image->signature == MagickSignature);
+      assert(composite_image->signature == MagickCoreSignature);
       switch( composite_options->compose )
       {
         case BlendCompositeOp:
@@ -136,7 +136,7 @@ static MagickBooleanType CompositeImageList(ImageInfo *image_info,Image **image,
         case ModulateCompositeOp:
         case ThresholdCompositeOp:
         {
-          (void) SetImageArtifact(composite_image,"compose:args",
+          (void) SetImageArtifact(*image,"compose:args",
             composite_options->compose_args);
           break;
         }
@@ -258,7 +258,6 @@ static MagickBooleanType CompositeUsage(void)
       "-negate              replace every pixel with its complementary color ",
       "-profile filename    add ICM or IPTC information profile to image",
       "-quantize colorspace reduce colors in this colorspace",
-      "-repage geometry     size and location of an image canvas (operator)",
       "-rotate degrees      apply Paeth rotation to the image",
       "-resize geometry     resize the image",
       "-sharpen geometry    sharpen the image",
@@ -313,6 +312,7 @@ static MagickBooleanType CompositeUsage(void)
       "-quiet               suppress all warning messages",
       "-red-primary point   chromaticity red primary point",
       "-regard-warnings     pay attention to warning messages",
+      "-repage geometry     size and location of an image canvas (operator)",
       "-respect-parentheses settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
@@ -369,7 +369,7 @@ static MagickBooleanType CompositeUsage(void)
 
 static void GetCompositeOptions(CompositeOptions *composite_options)
 {
-  (void) ResetMagickMemory(composite_options,0,sizeof(*composite_options));
+  (void) memset(composite_options,0,sizeof(*composite_options));
   composite_options->channel=DefaultChannels;
   composite_options->compose=OverCompositeOp;
 }
@@ -449,7 +449,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
     Set default.
   */
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(exception != (ExceptionInfo *) NULL);
@@ -1157,7 +1157,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
             status=MogrifyImageInfo(image_info,(int) (i-j+1),(const char **)
               argv+j,exception);
             DestroyComposite();
-            return(status == 0 ? MagickTrue : MagickFalse);
+            return(status == 0 ? MagickFalse : MagickTrue);
           }
         if (LocaleCompare("log",option+1) == 0)
           {
