@@ -17,13 +17,13 @@
 %                                July 1992                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -187,6 +187,7 @@ static MagickBooleanType DisplayUsage(void)
       "-map type            display image using this Standard Colormap",
       "-matte               store matte channel if the image has one",
       "-monitor             monitor progress",
+      "-nostdin             do not try to open stdin",
       "-page geometry       size and location of an image canvas",
       "-profile filename    add, delete, or apply an image profile",
       "-quality value       JPEG/MIFF/PNG compression level",
@@ -315,6 +316,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
 
   MagickBooleanType
     fire,
+    nostdin,
     pend,
     respect_parenthesis;
 
@@ -352,7 +354,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
     Set defaults.
   */
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(exception != (ExceptionInfo *) NULL);
@@ -377,8 +379,9 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
   option=(char *) NULL;
   pend=MagickFalse;
   respect_parenthesis=MagickFalse;
+  nostdin=MagickFalse;
   resource_database=(XrmDatabase) NULL;
-  (void) ResetMagickMemory(&resource_info,0,sizeof(resource_info));
+  (void) memset(&resource_info,0,sizeof(resource_info));
   server_name=(char *) NULL;
   state=0;
   status=MagickTrue;
@@ -403,6 +406,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
       Check command line for server name.
     */
     option=argv[i];
+    if (IsCommandOption(option) == MagickFalse)
+      continue;
     if (LocaleCompare("display",option+1) == 0)
       {
         /*
@@ -413,6 +418,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
           ThrowDisplayException(OptionError,"MissingArgument",option);
         server_name=argv[i];
       }
+    if (LocaleCompare("nostdin",option+1) == 0)
+      nostdin=MagickTrue;
     if ((LocaleCompare("help",option+1) == 0) ||
         (LocaleCompare("-help",option+1) == 0))
       return(DisplayUsage());
@@ -460,7 +467,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
       if (image != (Image *) NULL)
         break;
       else
-        if (isatty(STDIN_FILENO) != MagickFalse)
+        if (isatty(STDIN_FILENO) != MagickFalse || (nostdin != MagickFalse))
           option="logo:";
         else
           option="-";
@@ -1329,7 +1336,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             status=MogrifyImageInfo(image_info,(int) (i-j+1),(const char **)
               argv+j,exception);
             DestroyDisplay();
-            return(status == 0 ? MagickTrue : MagickFalse);
+            return(status == 0 ? MagickFalse : MagickTrue);
           }
         if (LocaleCompare("log",option+1) == 0)
           {
@@ -1425,6 +1432,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
           break;
         if (LocaleCompare("normalize",option+1) == 0)
           break;
+        if (LocaleCompare("nostdin",option+1) == 0)
+      	  break;
         ThrowDisplayException(OptionError,"UnrecognizedOption",option);
       }
       case 'p':
