@@ -23,13 +23,13 @@
 %                                 August 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -126,7 +126,7 @@ WandExport MagickWand *CloneMagickWand(const MagickWand *wand)
   if (clone_wand == (MagickWand *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       wand->name);
-  (void) ResetMagickMemory(clone_wand,0,sizeof(*clone_wand));
+  (void) memset(clone_wand,0,sizeof(*clone_wand));
   clone_wand->id=AcquireWandId();
   (void) FormatLocaleString(clone_wand->name,MaxTextExtent,"%s-%.20g",
     MagickWandId,(double) clone_wand->id);
@@ -421,7 +421,9 @@ WandExport char *MagickQueryConfigureOption(const char *option)
   exception=DestroyExceptionInfo(exception);
   if (configure_info == (const ConfigureInfo **) NULL)
     return((char *) NULL);
-  value=AcquireString(configure_info[0]->value);
+  value=(char *) NULL;
+  if (number_options != 0)
+    value=AcquireString(configure_info[0]->value);
   configure_info=(const ConfigureInfo **)
     RelinquishMagickMemory((void *) configure_info);
   return(value);
@@ -550,7 +552,7 @@ WandExport double *MagickQueryFontMetrics(MagickWand *wand,
       return((double *) NULL);
     }
   (void) CloneString(&draw_info->text,text);
-  (void) ResetMagickMemory(&metrics,0,sizeof(metrics));
+  (void) memset(&metrics,0,sizeof(metrics));
   status=GetTypeMetrics(wand->images,draw_info,&metrics);
   draw_info=DestroyDrawInfo(draw_info);
   if (status == MagickFalse)
@@ -658,7 +660,7 @@ WandExport double *MagickQueryMultilineFontMetrics(MagickWand *wand,
       return((double *) NULL);
     }
   (void) CloneString(&draw_info->text,text);
-  (void) ResetMagickMemory(&metrics,0,sizeof(metrics));
+  (void) memset(&metrics,0,sizeof(metrics));
   status=GetMultilineTypeMetrics(wand->images,draw_info,&metrics);
   draw_info=DestroyDrawInfo(draw_info);
   if (status == MagickFalse)
@@ -1075,7 +1077,7 @@ WandExport MagickWand *NewMagickWand(void)
   if (wand == (MagickWand *) NULL)
     ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
       GetExceptionMessage(errno));
-  (void) ResetMagickMemory(wand,0,sizeof(*wand));
+  (void) memset(wand,0,sizeof(*wand));
   wand->id=AcquireWandId();
   (void) FormatLocaleString(wand->name,MaxTextExtent,"%s-%.20g",MagickWandId,
     (double) wand->id);
